@@ -24,18 +24,15 @@
 #include <stdio.h>
 #include <string.h>
 #include <sys/types.h>
-#include <sys/stat.h>
 #include <math.h>
 #include <glib.h>
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gtk/gtk.h>
 
-/* Added by Andrew Ferrier: for use of gnome_about_new() */
-
-#ifdef GNOME
-#  include <gnome.h>
-#endif
+ #ifdef GNOME 	 
+ #  include <gnome.h> 	 
+ #endif 	 
 
 #ifdef G_OS_WIN32
 /*
@@ -77,22 +74,22 @@ ShellExecuteA (long        hwnd,
 #include "text.h"
 #include "dia_dirs.h"
 #include "focus.h"
-#include "gdk/gdk.h"
-#include "gdk/gdkkeysyms.h"
+#include <gdk/gdk.h>
+#include <gdk/gdkkeysyms.h>
 #include "lib/properties.h"
+#include "lib/parent.h"
 #include "dia-props.h"
 #include "diagram_tree_window.h"
 #include "authors.h"                /* master contributors data */
 
-GdkPixbuf *logo;
-
-void file_quit_callback(gpointer data, guint action, GtkWidget *widget)
+void 
+file_quit_callback (GtkAction *action)
 {
   app_exit();
 }
 
 void
-file_pagesetup_callback(gpointer data, guint action, GtkWidget *widget)
+file_pagesetup_callback (GtkAction *action)
 {
   Diagram *dia;
 
@@ -102,7 +99,7 @@ file_pagesetup_callback(gpointer data, guint action, GtkWidget *widget)
 }
 
 void
-file_print_callback(gpointer data, guint action, GtkWidget *widget)
+file_print_callback (GtkAction *action)
 {
   Diagram *dia;
 
@@ -118,35 +115,39 @@ file_print_callback(gpointer data, guint action, GtkWidget *widget)
 }
 
 void
-file_close_callback(gpointer data, guint action, GtkWidget *widget)
+file_close_callback (GtkAction *action)
 {
-  ddisplay_close(ddisplay_active());
-} 
+  /* some people use tear-off menus and insist to close non existing displays */
+  if (ddisplay_active())
+    ddisplay_close(ddisplay_active());
+}
 
 void
-file_new_callback(gpointer data, guint action, GtkWidget *widget)
+file_new_callback (GtkAction *action)
 {
   Diagram *dia;
   DDisplay *ddisp;
   static int untitled_nr = 1;
-  char buffer[24];
+  gchar *name, *filename;
 
-  g_snprintf(buffer, 24, _("Diagram%d.dia"), untitled_nr++);
-  
-  dia = new_diagram(buffer);
+  name = g_strdup_printf(_("Diagram%d.dia"), untitled_nr++);
+  filename = g_filename_from_utf8(name, -1, NULL, NULL, NULL);
+  dia = new_diagram(filename);
   ddisp = new_display(dia);
   diagram_tree_add(diagram_tree(), dia);
+  g_free (name);
+  g_free (filename);
 }
 
 void
-file_preferences_callback(gpointer data, guint action, GtkWidget *widget)
+file_preferences_callback (GtkAction *action)
 {
   prefs_show();
 }
 
 
 void
-edit_copy_callback(gpointer data, guint action, GtkWidget *widget)
+edit_copy_callback (GtkAction *action)
 {
   GList *copy_list;
   DDisplay *ddisp;
@@ -155,14 +156,14 @@ edit_copy_callback(gpointer data, guint action, GtkWidget *widget)
   if (!ddisp) return;
   copy_list = parent_list_affected(diagram_get_sorted_selected(ddisp->diagram));
 
-  cnp_store_objects(object_copy_list(copy_list));
+  cnp_store_objects(object_copy_list(copy_list), 1);
   g_list_free(copy_list);
 
   ddisplay_do_update_menu_sensitivity(ddisp);
 }
 
 void
-edit_cut_callback(gpointer data, guint action, GtkWidget *widget)
+edit_cut_callback (GtkAction *action)
 {
   GList *cut_list;
   DDisplay *ddisp;
@@ -175,7 +176,7 @@ edit_cut_callback(gpointer data, guint action, GtkWidget *widget)
 
   cut_list = parent_list_affected(diagram_get_sorted_selected(ddisp->diagram));
 
-  cnp_store_objects(object_copy_list(cut_list));
+  cnp_store_objects(object_copy_list(cut_list), 0);
 
   change = undo_delete_objects_children(ddisp->diagram, cut_list);
   (change->apply)(change, ddisp->diagram);
@@ -183,20 +184,20 @@ edit_cut_callback(gpointer data, guint action, GtkWidget *widget)
   ddisplay_do_update_menu_sensitivity(ddisp);
   diagram_flush(ddisp->diagram);
 
-
   diagram_modified(ddisp->diagram);
+  diagram_update_extents(ddisp->diagram);
   undo_set_transactionpoint(ddisp->diagram->undo);
-
 }
 
 void
-edit_paste_callback(gpointer data, guint action, GtkWidget *widget)
+edit_paste_callback (GtkAction *action)
 {
   GList *paste_list;
   DDisplay *ddisp;
   Point paste_corner;
   Point delta;
   Change *change;
+  int generation = 0;
   
   ddisp = ddisplay_active();
   if (!ddisp) return;
@@ -206,7 +207,7 @@ edit_paste_callback(gpointer data, guint action, GtkWidget *widget)
     return;
   }
   
-  paste_list = cnp_get_stored_objects(); /* Gets a copy */
+  paste_list = cnp_get_stored_objects(&generation); /* Gets a copy */
 
   paste_corner = object_list_corner(paste_list);
   
@@ -214,10 +215,11 @@ edit_paste_callback(gpointer data, guint action, GtkWidget *widget)
   delta.y = ddisp->visible.top - paste_corner.y;
 
   /* Move down some 10% of the visible area. */
-  delta.x += (ddisp->visible.right - ddisp->visible.left)*0.1;
-  delta.y += (ddisp->visible.bottom - ddisp->visible.top)*0.1;
+  delta.x += (ddisp->visible.right - ddisp->visible.left) * 0.1 * generation;
+  delta.y += (ddisp->visible.bottom - ddisp->visible.top) * 0.1 * generation;
 
-  object_list_move_delta(paste_list, &delta);
+  if (generation)
+    object_list_move_delta(paste_list, &delta);
 
   change = undo_insert_objects(ddisp->diagram, paste_list, 0);
   (change->apply)(change, ddisp->diagram);
@@ -228,6 +230,7 @@ edit_paste_callback(gpointer data, guint action, GtkWidget *widget)
   diagram_remove_all_selected(ddisp->diagram, TRUE);
   diagram_select_list(ddisp->diagram, paste_list);
 
+  diagram_update_extents(ddisp->diagram);
   diagram_flush(ddisp->diagram);
 }
 
@@ -241,7 +244,7 @@ edit_paste_callback(gpointer data, guint action, GtkWidget *widget)
  * completely untested, basically it is copy+paste munged together
  */
 void
-edit_duplicate_callback(gpointer data, guint action, GtkWidget *widget)
+edit_duplicate_callback (GtkAction *action)
 { 
   GList *duplicate_list;
   DDisplay *ddisp;
@@ -288,7 +291,7 @@ insert_text(DDisplay *ddisp, Focus *focus, const gchar *text)
 {
   ObjectChange *change = NULL;
   int modified = FALSE, any_modified = FALSE;
-  DiaObject *obj = focus->obj;
+  DiaObject *obj = focus_get_object(focus);
 
   while (text != NULL) {
     gchar *next_line = g_utf8_strchr(text, -1, '\n');
@@ -360,7 +363,7 @@ make_text_prop_singleton(GPtrArray **props, TextProperty **prop)
 
 
 void
-edit_copy_text_callback(gpointer data, guint action, GtkWidget *widget)
+edit_copy_text_callback (GtkAction *action)
 {
   Focus *focus = active_focus();
   DDisplay *ddisp;
@@ -373,7 +376,7 @@ edit_copy_text_callback(gpointer data, guint action, GtkWidget *widget)
   ddisp = ddisplay_active();
   if (!ddisp) return;
 
-  obj = focus->obj;
+  obj = focus_get_object(focus);
 
   if (obj->ops->get_props == NULL) 
     return;
@@ -396,7 +399,7 @@ edit_copy_text_callback(gpointer data, guint action, GtkWidget *widget)
 }
 
 void
-edit_cut_text_callback(gpointer data, guint action, GtkWidget *widget)
+edit_cut_text_callback (GtkAction *action)
 {
   Focus *focus = active_focus();
   DDisplay *ddisp;
@@ -411,8 +414,8 @@ edit_cut_text_callback(gpointer data, guint action, GtkWidget *widget)
   ddisp = ddisplay_active();
   if (!ddisp) return;
 
-  obj = focus->obj;
-  text = (Text *)focus->user_data; 
+  obj = focus_get_object(focus);
+  text = (Text*)focus->user_data;
 
   if (obj->ops->get_props == NULL) 
     return;
@@ -444,7 +447,7 @@ edit_cut_text_callback(gpointer data, guint action, GtkWidget *widget)
 }
 
 void
-edit_paste_text_callback(gpointer data, guint action, GtkWidget *widget)
+edit_paste_text_callback (GtkAction *action)
 {
   DDisplay *ddisp;
 
@@ -461,10 +464,9 @@ edit_paste_text_callback(gpointer data, guint action, GtkWidget *widget)
 }
 
 void
-edit_delete_callback(gpointer data, guint action, GtkWidget *widget)
+edit_delete_callback (GtkAction *action)
 {
   GList *delete_list;
-  GList *ptr;
   DDisplay *ddisp;
 
   Change *change;
@@ -480,6 +482,7 @@ edit_delete_callback(gpointer data, guint action, GtkWidget *widget)
   g_list_free(delete_list);
   
   diagram_modified(ddisp->diagram);
+  diagram_update_extents(ddisp->diagram);
 
   ddisplay_do_update_menu_sensitivity(ddisp);
   diagram_flush(ddisp->diagram);
@@ -488,7 +491,7 @@ edit_delete_callback(gpointer data, guint action, GtkWidget *widget)
 } 
 
 void
-edit_undo_callback(gpointer data, guint action, GtkWidget *widget)
+edit_undo_callback (GtkAction *action)
 {
   Diagram *dia;
   
@@ -497,12 +500,13 @@ edit_undo_callback(gpointer data, guint action, GtkWidget *widget)
 
   undo_revert_to_last_tp(dia->undo);
   diagram_modified(dia);
+  diagram_update_extents(dia);
 
   diagram_flush(dia);
 } 
 
 void
-edit_redo_callback(gpointer data, guint action, GtkWidget *widget)
+edit_redo_callback (GtkAction *action)
 {
   Diagram *dia;
   
@@ -511,13 +515,17 @@ edit_redo_callback(gpointer data, guint action, GtkWidget *widget)
 
   undo_apply_to_next_tp(dia->undo);
   diagram_modified(dia);
+  diagram_update_extents(dia);
 
   diagram_flush(dia);
 } 
 
 void
-help_manual_callback(gpointer data, guint action, GtkWidget *widget)
+help_manual_callback (GtkAction *action)
 {
+#ifdef GNOME
+  gnome_help_display("dia", NULL, NULL);
+#else
   char *helpdir, *helpindex = NULL, *command;
   guint bestscore = G_MAXINT;
   GDir *dp;
@@ -546,7 +554,7 @@ help_manual_callback(gpointer data, guint action, GtkWidget *widget)
     if (score < bestscore) {
       if (helpindex)
 	g_free(helpindex);
-      #ifdef G_OS_WIN32
+#ifdef G_OS_WIN32
       /* use HTML Help on win32 if available */
       helpindex = g_strconcat(helpdir, G_DIR_SEPARATOR_S, dentry,
 			      G_DIR_SEPARATOR_S "dia-manual.chm", NULL);
@@ -554,10 +562,10 @@ help_manual_callback(gpointer data, guint action, GtkWidget *widget)
 	helpindex = g_strconcat(helpdir, G_DIR_SEPARATOR_S, dentry,
 			      G_DIR_SEPARATOR_S "index.html", NULL);
       }
-      #else
+#else
       helpindex = g_strconcat(helpdir, G_DIR_SEPARATOR_S, dentry,
 			      G_DIR_SEPARATOR_S "index.html", NULL);
-      #endif
+#endif
       bestscore = score;
     }
   }
@@ -569,191 +577,79 @@ help_manual_callback(gpointer data, guint action, GtkWidget *widget)
   }
 
 #ifdef G_OS_WIN32
-  #define SW_SHOWNORMAL 1
+# define SW_SHOWNORMAL 1
   ShellExecuteA (0, "open", helpindex, NULL, helpdir, SW_SHOWNORMAL);
 #else
   command = getenv("BROWSER");
-  command = g_strdup_printf("%s 'file://%s' &", command ? command : "netscape", helpindex);
+  command = g_strdup_printf("%s 'file://%s' &", command ? command : "gnome-open", helpindex);
   system(command);
   g_free(command);
 #endif
 
   g_free(helpindex);
+#endif
 }
 
-void
-help_about_callback(gpointer data, guint action, GtkWidget *widget)
+static void 
+activate_url (GtkAboutDialog *about,
+              const gchar    *link,
+	      gpointer        data)
 {
-#ifdef GNOME
-
-  /*  Take advantage of gnome_about_new(),
-   *  which is much cleaner and GNOME2 HIG compliant,
-   *  Originally implemented by Xing Wang, modified
-   *  by Andrew Ferrier.
-   *
-   *  Note: in this function there is no need to discriminate
-   *  between the different kinds of 'authors'.
-   */
-  
-  static GtkWidget *about;
-  
-  /*
-   * Translators should localize the following string
-   * which will give them credit in the About box.
-   * E.g. "Fulano de Tal <fulano@detal.com>"
-   */
-  
-  gchar *translators = _("translator_credits-PLEASE_ADD_YOURSELF_HERE");
-  gchar logo_file[100];
-  
-  if (!about) {
-    GdkPixbuf *logo;
-  
-    gchar* datadir = dia_get_data_directory(""); 
-    g_snprintf(logo_file, sizeof(logo_file),
-                             "%s%sdia_logo.png", datadir, G_DIR_SEPARATOR_S);
-
-  	logo = gdk_pixbuf_new_from_file(logo_file, NULL);
-    g_free(datadir);
-  
-    about = gnome_about_new(
-      _("Dia"),
-      VERSION,
-      _("Copyright (C) 1998-2002 The Free Software Foundation and the authors"),
-      _("Dia is a program for drawing structured diagrams.\n"
-      "Please visit http://www.gnome.org/projects/dia for more information."),
-      authors,
-      documentors,
-      (strcmp (translators, "translator_credits-PLEASE_ADD_YOURSELF_HERE")
-      ? translators : NULL),
-      logo);
-  
-    if (logo)
-        g_object_unref (logo);
-      
-  	g_signal_connect (about, "destroy",
-         G_CALLBACK (gtk_widget_destroyed),
-         &about);
-  }
-
-  gtk_widget_show_now (about);
-  
+#ifdef G_OS_WIN32
+  ShellExecuteA (0, "open", link, NULL, NULL, SW_SHOWNORMAL);
 #else
-
-  /* No GNOME, fall back to the old GTK method */
-
-  const gint nauthors = (sizeof(authors) / sizeof(authors[0])) - 1;
-  const gint ndocumentors = (sizeof(documentors) / sizeof(documentors[0])) - 1;
-
-  GtkWidget *dialog;
-  GtkWidget *vbox;
-  GtkWidget *table;
-  GtkWidget *bbox;
-  GtkWidget *frame;
-  GtkWidget *label;
-  GtkWidget *button;
-  char str[100];
-  gint i;
-  
-  GtkWidget *gpixmap;
-  
-  dialog = gtk_dialog_new ();
-  gtk_window_set_role (GTK_WINDOW (dialog), "about_dialog");
-  gtk_window_set_title (GTK_WINDOW (dialog), _("About Dia"));
-  gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-  gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
-  g_signal_connect (GTK_OBJECT (dialog), "destroy",
-		    G_CALLBACK (gtk_widget_destroy), 
-          GTK_OBJECT (dialog));
-
-  vbox = gtk_vbox_new (FALSE, 1);
-  gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
-  gtk_container_add (GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), vbox);
-
-  if (!logo) {
-      gchar* datadir = dia_get_data_directory(""); 
-      g_snprintf(str, sizeof(str), "%s%sdia_logo.png", datadir, G_DIR_SEPARATOR_S);
-      logo = gdk_pixbuf_new_from_file(str, NULL);
-      g_free(datadir);
-  }
-
-  if (logo) {
-      GdkPixmap *pixmap;
-      GdkBitmap *bitmap;
-
-      frame = gtk_frame_new (NULL);
-      gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
-      gtk_container_set_border_width (GTK_CONTAINER (frame), 1);
-      gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, TRUE, 1);
-
-      gdk_pixbuf_render_pixmap_and_mask(logo, &pixmap, &bitmap, 128);
-      gpixmap = gtk_pixmap_new(pixmap, bitmap);
-      gdk_pixmap_unref(pixmap);
-      if (bitmap) gdk_bitmap_unref(bitmap);
-      gtk_container_add (GTK_CONTAINER(frame), gpixmap);
-  }
-
-  frame = gtk_frame_new (NULL);
-  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
-  gtk_container_set_border_width (GTK_CONTAINER (frame), 1);
-  gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, TRUE, 1);
-
-  table = gtk_table_new(3, 2, FALSE);
-  gtk_container_set_border_width (GTK_CONTAINER (table), 1);
-  gtk_container_add (GTK_CONTAINER (frame), table);
-
-  g_snprintf(str, sizeof(str), _("Dia v %s by Alexander Larsson"), VERSION);
-  label = gtk_label_new (str);
-  gtk_table_attach(GTK_TABLE(table), label, 0,2, 0,1,
-       GTK_FILL|GTK_EXPAND, GTK_FILL, 0,2);
-
-      /* Exact spelling is Ch&eacute;p&eacute;lov (using *ML entities) */
-  label = gtk_label_new(_("Maintainers: Lars Clausen and Cyrille Chepelov"));
-  gtk_table_attach(GTK_TABLE(table), label, 0,2, 1,2,
-       GTK_FILL|GTK_EXPAND, GTK_FILL, 0,2);
-
-  label = gtk_label_new (_("Please visit http://www.gnome.org/projects/dia "
-         "for more information"));
-  gtk_table_attach(GTK_TABLE(table), label, 0,2, 2,3,
-       GTK_FILL|GTK_EXPAND, GTK_FILL, 0,2);
-
-  label = gtk_label_new (_("Contributors:"));
-  gtk_table_attach(GTK_TABLE(table), label, 0,2, 3,4,
-       GTK_FILL|GTK_EXPAND, GTK_FILL, 0,2);
-
-  for (i = 0; i < nauthors; i++) {
-    label = gtk_label_new(authors[i]);
-    gtk_table_attach(GTK_TABLE(table), label, i%2,i%2+1, i/2+4,i/2+5,
-       GTK_FILL|GTK_EXPAND, GTK_FILL, 0,0);
-  }
-
-  for (i = nauthors; i < nauthors + ndocumentors; i++) {
-    label = gtk_label_new(documentors[i - nauthors]);
-    gtk_table_attach(GTK_TABLE(table), label, i%2,i%2+1, i/2+4,i/2+5,
-       GTK_FILL|GTK_EXPAND, GTK_FILL, 0,0);
-  }
-
-  gtk_table_set_col_spacings(GTK_TABLE(table), 1);
-  gtk_table_set_row_spacings(GTK_TABLE(table), 1);
-
-  bbox = gtk_hbutton_box_new();
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), bbox, TRUE, TRUE, 5);
-  gtk_button_box_set_child_size(GTK_BUTTON_BOX(bbox), 80, 0);
-  gtk_button_box_set_spacing(GTK_BUTTON_BOX(bbox), 10);
-
-  button = gtk_button_new_from_stock(GTK_STOCK_OK);
-  gtk_container_add(GTK_CONTAINER(bbox), button);
-  g_signal_connect_swapped(GTK_OBJECT (button), "clicked",
-			   G_CALLBACK(gtk_widget_destroy),
-          GTK_OBJECT(dialog));
-
-  gtk_widget_show_all (dialog);
-
-#endif /*  GNOME  */
+  gchar *command = getenv("BROWSER");
+  command = g_strdup_printf("%s '%s' &", command ? command : "gnome-open", link);
+  system(command);
+  g_free(command);
+#endif
 }
 
 void
-view_zoom_in_callback(gpointer data, guint action, GtkWidget *widget)
+help_about_callback (GtkAction *action)
+{
+  const gchar *translators = _("translator_credits-PLEASE_ADD_YOURSELF_HERE");
+  const gchar *license = _(
+	"This program is free software; you can redistribute it and/or modify\n"
+	"it under the terms of the GNU General Public License as published by\n"
+	"the Free Software Foundation; either version 2 of the License, or\n"
+	"(at your option) any later version.\n"
+	"\n"
+	"This program is distributed in the hope that it will be useful,\n"
+	"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
+	"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
+	"GNU General Public License for more details.\n"
+	"\n"
+	"You should have received a copy of the GNU General Public License\n"
+	"along with this program; if not, write to the Free Software\n"
+	"Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.\n");
+
+  gchar *dirname = dia_get_data_directory("");
+  gchar *filename = g_build_filename (dirname, "dia_logo.png", NULL);
+  GdkPixbuf *logo = gdk_pixbuf_new_from_file(filename, NULL);
+
+  gtk_about_dialog_set_url_hook (activate_url, NULL, NULL);
+  gtk_show_about_dialog (NULL,
+	"logo", logo,
+        "name", "Dia",
+	"version", VERSION,
+	"comments", _("A program for drawing structured diagrams."),
+	"copyright", "(C) 1998-2006 The Free Software Foundation and the authors",
+	"website", "http://www.gnome.org/projects/dia/",
+	"authors", authors,
+	"documenters", documentors,
+	"translator-credits", strcmp (translators, "translator_credits-PLEASE_ADD_YOURSELF_HERE")
+			? translators : NULL,
+	"license", license,
+	NULL);
+  g_free (dirname);
+  g_free (filename);
+  if (logo)
+    g_object_unref (logo);
+}
+
+void
+view_zoom_in_callback (GtkAction *action)
 {
   DDisplay *ddisp;
   Point middle;
@@ -769,7 +665,7 @@ view_zoom_in_callback(gpointer data, guint action, GtkWidget *widget)
 }
 
 void
-view_zoom_out_callback(gpointer data, guint action, GtkWidget *widget)
+view_zoom_out_callback (GtkAction *action)
 {
   DDisplay *ddisp;
   Point middle;
@@ -785,26 +681,31 @@ view_zoom_out_callback(gpointer data, guint action, GtkWidget *widget)
 }
 
 void
-view_zoom_set_callback(gpointer data, guint action, GtkWidget *widget)
+view_zoom_set_callback (GtkAction *action)
 {
   DDisplay *ddisp;
   real scale;
   Point middle;
   Rectangle *visible;
-  
+  int factor;
+
   ddisp = ddisplay_active();
   if (!ddisp) return;
+
+  /* HACK the actual factor is a suffix to the action name */
+  factor = atoi (gtk_action_get_name (action) + strlen ("ViewZoom"));
+
   visible = &ddisp->visible;
   middle.x = visible->left*0.5 + visible->right*0.5;
   middle.y = visible->top*0.5 + visible->bottom*0.5;
 
-  scale = ((real) action)/1000.0 * DDISPLAY_NORMAL_ZOOM;
+  scale = ((real) factor)/1000.0 * DDISPLAY_NORMAL_ZOOM;
 
   ddisplay_zoom(ddisp, &middle, scale / ddisp->zoom_factor);  
 }
 
 void
-view_show_cx_pts_callback(gpointer data, guint action, GtkWidget *widget)
+view_show_cx_pts_callback (GtkToggleAction *action)
 {
   DDisplay *ddisp;
   int old_val;
@@ -813,7 +714,7 @@ view_show_cx_pts_callback(gpointer data, guint action, GtkWidget *widget)
   if (!ddisp) return;
 
   old_val = ddisp->show_cx_pts;
-  ddisp->show_cx_pts = GTK_CHECK_MENU_ITEM(widget)->active;
+  ddisp->show_cx_pts = gtk_toggle_action_get_active (action);
   
   if (old_val != ddisp->show_cx_pts) {
     ddisplay_add_update_all(ddisp);
@@ -821,8 +722,41 @@ view_show_cx_pts_callback(gpointer data, guint action, GtkWidget *widget)
   }
 }
 
+void 
+view_unfullscreen (void)
+{
+  DDisplay *ddisp;
+  GtkToggleAction *item;
+
+  ddisp = ddisplay_active();
+  if (!ddisp) return;
+
+  /* find the menuitem */
+  item = GTK_TOGGLE_ACTION (menus_get_action ("ViewFullscreen"));
+  if (item && gtk_toggle_action_get_active (item)) {
+    gtk_toggle_action_set_active (item, FALSE);
+  }
+}
+
 void
-view_aa_callback(gpointer data, guint action, GtkWidget *widget)
+view_fullscreen_callback (GtkToggleAction *action)
+{
+  DDisplay *ddisp;
+  int fs;
+
+  ddisp = ddisplay_active();
+  if (!ddisp) return;
+
+  fs = gtk_toggle_action_get_active (action);
+  
+  if (fs) /* it is already toggled */
+    gtk_window_fullscreen(GTK_WINDOW(ddisp->shell));
+  else
+    gtk_window_unfullscreen(GTK_WINDOW(ddisp->shell));
+}
+
+void
+view_aa_callback (GtkToggleAction *action)
 {
   DDisplay *ddisp;
   int aa;
@@ -830,7 +764,7 @@ view_aa_callback(gpointer data, guint action, GtkWidget *widget)
   ddisp = ddisplay_active();
   if (!ddisp) return;
  
-  aa =  GTK_CHECK_MENU_ITEM(widget)->active;
+  aa = gtk_toggle_action_get_active (action);
   
   if (aa != ddisp->aa_renderer) {
     ddisplay_set_renderer(ddisp, aa);
@@ -840,16 +774,16 @@ view_aa_callback(gpointer data, guint action, GtkWidget *widget)
 }
 
 void
-view_visible_grid_callback(gpointer data, guint action, GtkWidget *widget)
+view_visible_grid_callback (GtkToggleAction *action)
 {
   DDisplay *ddisp;
-  int old_val;
+  guint old_val;
 
   ddisp = ddisplay_active();
   if (!ddisp) return;
   
   old_val = ddisp->grid.visible;
-  ddisp->grid.visible = GTK_CHECK_MENU_ITEM(widget)->active; 
+  ddisp->grid.visible = gtk_toggle_action_get_active (action); 
 
   if (old_val != ddisp->grid.visible) {
     ddisplay_add_update_all(ddisp);
@@ -858,31 +792,36 @@ view_visible_grid_callback(gpointer data, guint action, GtkWidget *widget)
 }
 
 void
-view_snap_to_grid_callback(gpointer data, guint action, GtkWidget *widget)
+view_snap_to_grid_callback (GtkToggleAction *action)
 {
   DDisplay *ddisp;
 
   ddisp = ddisplay_active();
   if (!ddisp) return;
   
-  ddisplay_set_snap_to_grid(ddisp, GTK_CHECK_MENU_ITEM(widget)->active);
+  ddisplay_set_snap_to_grid(ddisp, gtk_toggle_action_get_active (action));
 }
 
-void view_toggle_rulers_callback(gpointer data, guint action, GtkWidget*widget)
+void
+view_snap_to_objects_callback (GtkToggleAction *action)
+{
+  DDisplay *ddisp;
+
+  ddisp = ddisplay_active();
+  if (!ddisp) return;
+  
+  ddisplay_set_snap_to_objects(ddisp, gtk_toggle_action_get_active (action));
+}
+
+void 
+view_toggle_rulers_callback (GtkToggleAction *action)
 {
   DDisplay *ddisp;
   
   ddisp = ddisplay_active();
   if (!ddisp) return;
 
-  /* The following is borrowed straight from the Gimp: */
-  
-  /* This routine use promiscuous knowledge of gtk internals
-   *  in order to hide and show the rulers "smoothly". This
-   *  is kludgy and a hack and may break if gtk is changed
-   *  internally.
-   */
-  if (!GTK_CHECK_MENU_ITEM(widget)->active) {
+  if (!gtk_toggle_action_get_active (action)) {
     if (GTK_WIDGET_VISIBLE (ddisp->origin)) {
       gtk_widget_unmap (ddisp->origin);
       gtk_widget_unmap (ddisp->hrule);
@@ -910,7 +849,7 @@ void view_toggle_rulers_callback(gpointer data, guint action, GtkWidget*widget)
 }
 
 extern void
-view_new_view_callback(gpointer data, guint action, GtkWidget *widget)
+view_new_view_callback (GtkAction *action)
 {
   Diagram *dia;
 
@@ -920,8 +859,19 @@ view_new_view_callback(gpointer data, guint action, GtkWidget *widget)
   new_display(dia);
 }
 
+extern void
+view_clone_view_callback (GtkAction *action)
+{
+  DDisplay *ddisp;
+
+  ddisp = ddisplay_active();
+  if (!ddisp) return;
+  
+  copy_display(ddisp);
+}
+
 void
-view_show_all_callback(gpointer data, guint action, GtkWidget *widget)
+view_show_all_callback (GtkAction *action)
 {
   DDisplay *ddisp;
   Diagram *dia;
@@ -936,17 +886,34 @@ view_show_all_callback(gpointer data, guint action, GtkWidget *widget)
   width = dia_renderer_get_width_pixels (ddisp->renderer);
   height = dia_renderer_get_height_pixels (ddisp->renderer);
 
-  magnify_x = (real)width /
-    (dia->data->extents.right - dia->data->extents.left) / ddisp->zoom_factor;
-  magnify_y = (real)height /
-    (dia->data->extents.bottom - dia->data->extents.top) / ddisp->zoom_factor;
+  /* if there is something selected show that instead of all exisiting objects */
+  if (dia->data->selected) {
+    GList *list = dia->data->selected;
+    Rectangle extents = ((DiaObject*)list->data)->bounding_box;
+    list = g_list_next(list);
+    while (list) {
+      DiaObject *obj = (DiaObject *)list->data;
+      rectangle_union(&extents, &(obj->bounding_box));
+      list = g_list_next(list);
+    }
+    magnify_x = (real)width / (extents.right - extents.left) / ddisp->zoom_factor;
+    magnify_y = (real)height / (extents.bottom - extents.top) / ddisp->zoom_factor;
+    middle.x = extents.left + (extents.right - extents.left) / 2.0;
+    middle.y = extents.top + (extents.bottom - extents.top) / 2.0;
+  } else {
+    magnify_x = (real)width /
+      (dia->data->extents.right - dia->data->extents.left) / ddisp->zoom_factor;
+    magnify_y = (real)height /
+      (dia->data->extents.bottom - dia->data->extents.top) / ddisp->zoom_factor;
 
-  middle.x = dia->data->extents.left +
-    (dia->data->extents.right - dia->data->extents.left) / 2.0;
-  middle.y = dia->data->extents.top +
-    (dia->data->extents.bottom - dia->data->extents.top) / 2.0;
+    middle.x = dia->data->extents.left +
+      (dia->data->extents.right - dia->data->extents.left) / 2.0;
+    middle.y = dia->data->extents.top +
+      (dia->data->extents.bottom - dia->data->extents.top) / 2.0;
+  }
 
-  ddisplay_zoom (ddisp, &middle, (magnify_x<magnify_y)?magnify_x:magnify_y);
+  ddisplay_zoom (ddisp, &middle, 
+		 ((magnify_x<magnify_y)?magnify_x:magnify_y)/1.05);
 
   ddisplay_update_scrollbars(ddisp);
   ddisplay_add_update_all(ddisp);
@@ -954,7 +921,7 @@ view_show_all_callback(gpointer data, guint action, GtkWidget *widget)
 }
 
 void
-view_redraw_callback(gpointer data, guint action, GtkWidget *widget)
+view_redraw_callback (GtkAction *action)
 {
   DDisplay *ddisp;
   ddisp = ddisplay_active();
@@ -964,7 +931,7 @@ view_redraw_callback(gpointer data, guint action, GtkWidget *widget)
 }
 
 void
-view_diagram_properties_callback(gpointer data, guint action, GtkWidget *widget)
+view_diagram_properties_callback (GtkAction *action)
 {
   DDisplay *ddisp;
 
@@ -975,61 +942,71 @@ view_diagram_properties_callback(gpointer data, guint action, GtkWidget *widget)
 
 
 void
-objects_place_over_callback(gpointer data, guint action, GtkWidget *widget)
+objects_place_over_callback (GtkAction *action)
 {
   diagram_place_over_selected(ddisplay_active_diagram());
 }
 
 void
-objects_place_under_callback(gpointer data, guint action, GtkWidget *widget)
+objects_place_under_callback (GtkAction *action)
 {
   diagram_place_under_selected(ddisplay_active_diagram());
 }
 
 void
-objects_place_up_callback(gpointer data, guint action, GtkWidget *widget)
+objects_place_up_callback (GtkAction *action)
 {
   diagram_place_up_selected(ddisplay_active_diagram());
 }
 
 void
-objects_place_down_callback(gpointer data, guint action, GtkWidget *widget)
+objects_place_down_callback (GtkAction *action)
 {
   diagram_place_down_selected(ddisplay_active_diagram());
 }
 
 void
-objects_parent_callback(gpointer data, guint action, GtkWidget *widget)
+objects_parent_callback (GtkAction *action)
 {
   diagram_parent_selected(ddisplay_active_diagram());
 }
 
 void
-objects_unparent_callback(gpointer data, guint action, GtkWidget *widget)
+objects_unparent_callback (GtkAction *action)
 {
   diagram_unparent_selected(ddisplay_active_diagram());
 }
 
 void
-objects_unparent_children_callback(gpointer data, guint action, GtkWidget *widget)
+objects_unparent_children_callback (GtkAction *action)
 {
   diagram_unparent_children_selected(ddisplay_active_diagram());
 }
 
 void
-objects_group_callback(gpointer data, guint action, GtkWidget *widget)
+objects_group_callback (GtkAction *action)
 {
+  DDisplay *ddisp;
+
+  ddisp = ddisplay_active();
+  if (!ddisp) return;
   diagram_group_selected(ddisplay_active_diagram());
+  ddisplay_do_update_menu_sensitivity(ddisp);
 } 
 
 void
-objects_ungroup_callback(gpointer data, guint action, GtkWidget *widget)
+objects_ungroup_callback (GtkAction *action)
 {
+  DDisplay *ddisp;
+
+  ddisp = ddisplay_active();
+  if (!ddisp) return;
   diagram_ungroup_selected(ddisplay_active_diagram());
+  ddisplay_do_update_menu_sensitivity(ddisp);
 } 
 
 void
-dialogs_properties_callback(gpointer data, guint action, GtkWidget *widget)
+dialogs_properties_callback (GtkAction *action)
 {
   Diagram *dia;
   DiaObject *selected;
@@ -1046,7 +1023,7 @@ dialogs_properties_callback(gpointer data, guint action, GtkWidget *widget)
 }
 
 void
-dialogs_layers_callback(gpointer data, guint action, GtkWidget *widget)
+dialogs_layers_callback (GtkAction *action)
 {
   layer_dialog_set_diagram(ddisplay_active_diagram());
   layer_dialog_show();
@@ -1054,13 +1031,34 @@ dialogs_layers_callback(gpointer data, guint action, GtkWidget *widget)
 
 
 void
-objects_align_h_callback(gpointer data, guint action, GtkWidget *widget)
+objects_align_h_callback (GtkAction *action)
 {
-  int align;
+  const gchar *a;
+  int align = DIA_ALIGN_LEFT;
   Diagram *dia;
   GList *objects;
 
-  align = action;
+  /* HACK align is suffix to action name */
+  a = gtk_action_get_name (action) + strlen ("ObjectsAlign");
+  if (0 == strcmp ("Left", a)) {
+	align = DIA_ALIGN_LEFT;
+  } 
+  else if (0 == strcmp ("Center", a)) {
+	align = DIA_ALIGN_CENTER;
+  } 
+  else if (0 == strcmp ("Right", a)) {
+	align = DIA_ALIGN_RIGHT;
+  } 
+  else if (0 == strcmp ("Spreadouthorizontally", a)) {
+	align = DIA_ALIGN_EQUAL;
+  } 
+  else if (0 == strcmp ("Adjacent", a)) {
+	align = DIA_ALIGN_ADJACENT;
+  }
+  else {
+	g_warning ("objects_align_v_callback() called without appropriate align");
+	return;
+  }
 
   dia = ddisplay_active_diagram();
   if (!dia) return;
@@ -1077,13 +1075,34 @@ objects_align_h_callback(gpointer data, guint action, GtkWidget *widget)
 }
 
 void
-objects_align_v_callback(gpointer data, guint action, GtkWidget *widget)
+objects_align_v_callback (GtkAction *action)
 {
+  const gchar *a;
   int align;
   Diagram *dia;
   GList *objects;
 
-  align = action;
+  /* HACK align is suffix to action name */
+  a = gtk_action_get_name (action) + strlen ("ObjectsAlign");
+  if (0 == strcmp ("Top", a)) {
+	align = DIA_ALIGN_TOP;
+  } 
+  else if (0 == strcmp ("Middle", a)) {
+	align = DIA_ALIGN_CENTER;
+  } 
+  else if (0 == strcmp ("Bottom", a)) {
+	align = DIA_ALIGN_BOTTOM;
+  } 
+  else if (0 == strcmp ("Spreadoutvertically", a)) {
+	align = DIA_ALIGN_EQUAL;
+  } 
+  else if (0 == strcmp ("Stacked", a)) {
+	align = DIA_ALIGN_ADJACENT;
+  }
+  else {
+	g_warning ("objects_align_v_callback() called without appropriate align");
+	return;
+  }
 
   dia = ddisplay_active_diagram();
   if (!dia) return;
