@@ -45,29 +45,24 @@ typedef struct _DDisplay DDisplay;
 #define DDISPLAY_MIN_ZOOM 0.01
 */
 struct _DDisplay {
-  Diagram *diagram;               /* pointer to the associated diagram */
+  Diagram *diagram;                  /* pointer to the associated diagram */
 
-  GtkWidget *shell;               /* shell widget for this ddisplay    */
-  GtkWidget *canvas;              /* canvas widget for this ddisplay   */
-  GtkWidget *hsb, *vsb;           /* widgets for scroll bars           */
-  GtkWidget *hrule, *vrule;       /* widgets for rulers                */
-  GtkWidget *origin;              /* widgets for rulers                */
-  GtkWidget *menu_bar;            /* widget for the menu bar           */
-  GtkItemFactory *mbar_item_factory; /* item factory used to create the menu bar */
+  GtkWidget      *shell;               /* shell widget for this ddisplay    */
+  GtkWidget      *canvas;              /* canvas widget for this ddisplay   */
+  GtkWidget      *hsb, *vsb;           /* widgets for scroll bars           */
+  GtkWidget      *hrule, *vrule;       /* widgets for rulers                */
+  GtkWidget      *origin;              /* widgets for rulers                */
+  GtkWidget      *menu_bar;            /* widget for the menu bar           */
+  GtkUIManager   *ui_manager;     /* ui manager used to create the menu bar */
+  GtkActionGroup *actions;        
 
   /* menu bar widgets */
   GtkMenuItem *rulers;
-  GtkMenuItem *visible_grid;
-  GtkMenuItem *snap_to_grid;
-  GtkMenuItem *show_cx_pts_mitem;
-#ifdef HAVE_LIBART
-  GtkMenuItem *antialiased;
-#endif
   UpdatableMenuItems updatable_menu_items;
-    
 
   GtkWidget *zoom_status;         
   GtkWidget *grid_status;
+  GtkWidget *mainpoint_status;
   GtkWidget *modified_status;
 
   GtkAccelGroup *accel_group;
@@ -83,6 +78,7 @@ struct _DDisplay {
 
   gboolean show_cx_pts;		  /* Connection points toggle boolean  */
   gboolean autoscroll;
+  gboolean mainpoint_magnetism;   /* Mainpoints snapped from entire obj*/
 
   int aa_renderer;
   DiaRenderer *renderer;
@@ -101,6 +97,7 @@ struct _DDisplay {
 extern GdkCursor *default_cursor;
 
 DDisplay *new_display(Diagram *dia);
+DDisplay *copy_display(DDisplay *orig_ddisp);
 /* Normal destroy is done through shell widget destroy event. */
 void ddisplay_really_destroy(DDisplay *ddisp); 
 void ddisplay_transform_coords_double(DDisplay *ddisp,
@@ -130,6 +127,7 @@ void ddisplay_set_origo(DDisplay *ddisp,
 void ddisplay_zoom(DDisplay *ddisp, Point *point,
 		   real zoom_factor);
 void ddisplay_set_snap_to_grid(DDisplay *ddisp, gboolean snap);
+void ddisplay_set_snap_to_objects(DDisplay *ddisp, gboolean magnetic);
 void ddisplay_set_renderer(DDisplay *ddisp, int aa_renderer);
 void ddisplay_resize_canvas(DDisplay *ddisp,
 			    int width,
