@@ -32,13 +32,22 @@
 /** Calculate the width (in cm) of the gap between grid lines in dynamic
  * grid mode.
  */
-static real
+static void
 calculate_dynamic_grid(DDisplay *ddisp, real *width_x, real *width_y)
 {
   real zoom = ddisplay_untransform_length(ddisp, 1.0);
+  real ret, tmp;
   /* Twiddle zoom to make change-over appropriate */
   zoom *= 5;
-  return pow(10, ceil(log10(zoom)));
+  ret = pow(10, ceil(log10(zoom)));
+  /* dont' make it too small or huge (this is in pixels) */
+  tmp = ddisplay_transform_length(ddisp, ret);
+  if (tmp < 10.0)
+    ret *= 2.0;
+  else if (tmp > 35.0)
+    ret /= 2.0;
+  *width_x = ret;
+  *width_y = ret;
 }
 
 static void
@@ -47,7 +56,7 @@ grid_draw_horizontal_lines(DDisplay *ddisp, Rectangle *update, real length)
   int x, y;
   real pos;
   int height, width;
-  guint major_lines = ddisp->diagram->data->grid.major_lines;
+  guint major_lines = ddisp->diagram->grid.major_lines;
   DiaRenderer *renderer = ddisp->renderer;
   DiaInteractiveRendererInterface *irenderer;
   int major_count = 0;
@@ -58,8 +67,19 @@ grid_draw_horizontal_lines(DDisplay *ddisp, Rectangle *update, real length)
   ddisplay_transform_coords(ddisp, update->left, pos, &x, &y);
   ddisplay_transform_coords(ddisp, update->right, update->bottom, &width, &height);
 
+  /*
+    Explanatory note from Lawrence Withers (lwithers@users.sf.net):
+    Think about it in terms of the maths and the modulo arithmetic; we
+    have a number P such that P < 0, and we want to find a number Q
+    such that Q >= 0, but (P % major_count) == (Q % major_count). To
+    do this, we say Q = P - (P * major_count), which is guaranteed to
+    give a correct answer if major_count > 0 (since the addition of
+    anf multiple of major_count won't alter the modulo).
+  */
+
   if (major_lines) {
-    major_count = pos/length;
+    major_count = (int)ceil(pos/length);
+    if(major_count < 0) major_count -= major_lines * major_count;
     major_count %= major_lines;
   }
 
@@ -72,7 +92,7 @@ grid_draw_horizontal_lines(DDisplay *ddisp, Rectangle *update, real length)
       major_count = (major_count+1)%major_lines;
     }
     irenderer->draw_pixel_line(renderer, x, y, width, y,
-			       &ddisp->diagram->data->grid.colour);
+			       &ddisp->diagram->grid.colour);
     pos += length;
     ddisplay_transform_coords(ddisp, update->left, pos, &x, &y);
   }
@@ -84,7 +104,7 @@ grid_draw_vertical_lines(DDisplay *ddisp, Rectangle *update, real length)
   int x = 0, y = 0;
   real pos;
   int height, width;
-  guint major_lines = ddisp->diagram->data->grid.major_lines;
+  guint major_lines = ddisp->diagram->grid.major_lines;
   DiaRenderer *renderer = ddisp->renderer;
   DiaInteractiveRendererInterface *irenderer;
   int major_count = 0;
@@ -95,7 +115,8 @@ grid_draw_vertical_lines(DDisplay *ddisp, Rectangle *update, real length)
   ddisplay_transform_coords(ddisp, update->right, update->bottom, &width, &height);
 
   if (major_lines) {
-    major_count = pos/length;
+    major_count = (int)ceil(pos/length);
+    if(major_count < 0) major_count -= major_lines * major_count;
     major_count %= major_lines;
   }
 
@@ -109,7 +130,7 @@ grid_draw_vertical_lines(DDisplay *ddisp, Rectangle *update, real length)
       major_count = (major_count+1)%major_lines;
     }
     irenderer->draw_pixel_line(renderer, x, y, x, height,
-			       &ddisp->diagram->data->grid.colour);
+			       &ddisp->diagram->grid.colour);
     pos += length;
   }
 }
@@ -134,7 +155,7 @@ grid_draw_hex(DDisplay *ddisp, Rectangle *update, real length)
 	  
       irenderer->draw_pixel_line(renderer,
 				 x, y, to_x, y,
-				 &ddisp->diagram->data->grid.colour);
+				 &ddisp->diagram->grid.colour);
       horiz_pos += 3 * length;
     }
 	
@@ -151,7 +172,7 @@ grid_draw_hex(DDisplay *ddisp, Rectangle *update, real length)
 	  
       irenderer->draw_pixel_line(renderer,
 				 x, y, to_x, y,
-				 &ddisp->diagram->data->grid.colour);
+				 &ddisp->diagram->grid.colour);
       horiz_pos += 3 * length;
     }
 	
@@ -168,14 +189,14 @@ grid_draw_hex(DDisplay *ddisp, Rectangle *update, real length)
 	  
       irenderer->draw_pixel_line(renderer,
 				 x, y, to_x, to_y,
-				 &ddisp->diagram->data->grid.colour);
+				 &ddisp->diagram->grid.colour);
 
       ddisplay_transform_coords(ddisp, horiz_pos, vert_pos, &x, &y);
       ddisplay_transform_coords(ddisp, horiz_pos - 0.5 * length, vert_pos + length * sqrt(3) * 0.5, &to_x, &to_y);
 	  
       irenderer->draw_pixel_line(renderer,
 				 x, y, to_x, to_y,
-				 &ddisp->diagram->data->grid.colour);
+				 &ddisp->diagram->grid.colour);
       horiz_pos += 3 * length;
     }
 	
@@ -192,14 +213,14 @@ grid_draw_hex(DDisplay *ddisp, Rectangle *update, real length)
 	  
       irenderer->draw_pixel_line(renderer,
 				 x, y, to_x, to_y,
-				 &ddisp->diagram->data->grid.colour);
+				 &ddisp->diagram->grid.colour);
 
       ddisplay_transform_coords(ddisp, horiz_pos + length, vert_pos, &x, &y);
       ddisplay_transform_coords(ddisp, horiz_pos + 1.5 * length, vert_pos + 0.5 * sqrt(3) * length, &to_x, &to_y);
 	  
       irenderer->draw_pixel_line(renderer,
 				 x, y, to_x, to_y,
-				 &ddisp->diagram->data->grid.colour);
+				 &ddisp->diagram->grid.colour);
       horiz_pos += 3 * length;
     }
 	
@@ -213,29 +234,26 @@ grid_draw(DDisplay *ddisp, Rectangle *update)
 {
   Grid *grid = &ddisp->grid;
   DiaRenderer *renderer = ddisp->renderer;
-  int major_lines;
 
   if (grid->visible) {
-    int width = dia_renderer_get_width_pixels(ddisp->renderer);
-    int height = dia_renderer_get_height_pixels(ddisp->renderer);
     /* distance between visible grid lines */
-    real width_x = ddisp->diagram->data->grid.width_x;
-    real width_y = ddisp->diagram->data->grid.width_y;
-    real width_w = ddisp->diagram->data->grid.width_w;
-    if (ddisp->diagram->data->grid.dynamic) {
-      width_x = width_y = calculate_dynamic_grid(ddisp, &width_x, &width_y);
+    real width_x = ddisp->diagram->grid.width_x;
+    real width_y = ddisp->diagram->grid.width_y;
+    real width_w = ddisp->diagram->grid.width_w;
+    if (ddisp->diagram->grid.dynamic) {
+      calculate_dynamic_grid(ddisp, &width_x, &width_y);
     } else {
-      width_x = ddisp->diagram->data->grid.width_x *
-	ddisp->diagram->data->grid.visible_x;
-      width_y = ddisp->diagram->data->grid.width_y *
-	ddisp->diagram->data->grid.visible_y;
+      width_x = ddisp->diagram->grid.width_x *
+	ddisp->diagram->grid.visible_x;
+      width_y = ddisp->diagram->grid.width_y *
+	ddisp->diagram->grid.visible_y;
     }
 
     DIA_RENDERER_GET_CLASS(renderer)->set_linewidth(renderer, 0.0);
     DIA_RENDERER_GET_CLASS(renderer)->set_dashlength(renderer,
 						     ddisplay_untransform_length(ddisp, 31));
     
-    if (ddisp->diagram->data->grid.hex) {
+    if (ddisp->diagram->grid.hex) {
       grid_draw_hex(ddisp, update, width_w);
     } else {
       if (ddisplay_transform_length(ddisp, width_y) >= 2.0 &&
@@ -285,7 +303,7 @@ pagebreak_draw(DDisplay *ddisp, Rectangle *update)
       ddisplay_transform_coords(ddisp, pos,0,&x,&y);
       irenderer->draw_pixel_line(renderer,
                                  x, 0, x, height,
-				 &dia->data->pagebreak_color);
+				 &dia->pagebreak_color);
       pos += pwidth;
     }
     /* Horizontal lines: */
@@ -294,7 +312,7 @@ pagebreak_draw(DDisplay *ddisp, Rectangle *update)
       ddisplay_transform_coords(ddisp, 0,pos,&x,&y);
       irenderer->draw_pixel_line(renderer,
 				 0, y, width, y,
-				 &dia->data->pagebreak_color);
+				 &dia->pagebreak_color);
       pos += pheight;
     }
   }
@@ -304,8 +322,8 @@ void
 snap_to_grid(DDisplay *ddisp, coord *x, coord *y)
 {
   if (ddisp->grid.snap) {
-    if (ddisp->diagram->data->grid.hex) {
-      real width_x = ddisp->diagram->data->grid.width_w;
+    if (ddisp->diagram->grid.hex) {
+      real width_x = ddisp->diagram->grid.width_w;
       real x_mod = (*x - 1*width_x) - floor((*x - 1*width_x) / (3*width_x)) * 3 * width_x;
       real y_mod = (*y - 0.25*sqrt(3) * width_x) -
 	floor((*y - 0.25 * sqrt(3) * width_x) / (sqrt(3)*width_x)) * sqrt(3) * width_x;
@@ -328,9 +346,9 @@ snap_to_grid(DDisplay *ddisp, coord *x, coord *y)
 	}
       }
     } else {
-      real width_x = ddisp->diagram->data->grid.width_x;
-      real width_y = ddisp->diagram->data->grid.width_y;
-      if (prefs.grid.dynamic) {
+      real width_x = ddisp->diagram->grid.width_x;
+      real width_y = ddisp->diagram->grid.width_y;
+      if (ddisp->diagram->grid.dynamic) {
 	calculate_dynamic_grid(ddisp, &width_x, &width_y);
       }
       *x = ROUND((*x) / width_x) * width_x;
