@@ -24,6 +24,7 @@
 #include <string.h>
 
 #include "diagram.h"
+#include "object.h"
 #include "layer_dialog.h"
 #include "interface.h"
 #include "display.h"
@@ -38,85 +39,186 @@
 #include "diaarrowchooser.h"
 #include "dialinechooser.h"
 #include "widgets.h"
+#include "message.h"
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include "dia-app-icons.h"
 #include "diacanvas.h"
 
-static const GtkTargetEntry create_object_targets[] = {
-  { "application/x-dia-object", 0, 0 },
+#include "pixmaps/missing.xpm"
+
+/* HB: file dnd stuff lent by The Gimp, not fully understood but working ...
+ */
+enum
+{
+  DIA_DND_TYPE_URI_LIST,
+  DIA_DND_TYPE_TEXT_PLAIN,
 };
 
+static GtkTargetEntry toolbox_target_table[] =
+{
+  { "text/uri-list", 0, DIA_DND_TYPE_URI_LIST },
+  { "text/plain", 0, DIA_DND_TYPE_TEXT_PLAIN }
+};
+
+static guint toolbox_n_targets = (sizeof (toolbox_target_table) /
+                                 sizeof (toolbox_target_table[0]));
+
+static void
+dia_dnd_file_drag_data_received (GtkWidget        *widget,
+                                 GdkDragContext   *context,
+                                 gint              x,
+                                 gint              y,
+                                 GtkSelectionData *data,
+                                 guint             info,
+                                 guint             time,
+				 DDisplay         *ddisp)
+{
+  switch (context->action)
+    {
+    case GDK_ACTION_DEFAULT:
+    case GDK_ACTION_COPY:
+    case GDK_ACTION_MOVE:
+    case GDK_ACTION_LINK:
+    case GDK_ACTION_ASK:
+    default:
+      {
+        Diagram *diagram = NULL;
+        gchar *sPath = NULL, *pFrom, *pTo; 
+
+        pFrom = strstr((gchar *) data->data, "file:");
+        while (pFrom) {
+          GError *error = NULL;
+
+          pTo = pFrom;
+          while (*pTo != 0 && *pTo != 0xd && *pTo != 0xa) pTo ++;
+          sPath = g_strndup(pFrom, pTo - pFrom);
+
+          /* format changed with Gtk+2.0, use conversion */
+          pFrom = g_filename_from_uri (sPath, NULL, &error);
+	  if (!ddisp)
+            diagram = diagram_load (pFrom, NULL);
+	  else {
+	    diagram = ddisp->diagram;
+	    if (!diagram_load_into (diagram, pFrom, NULL)) {
+	      /* the import filter is supposed to show the error message */
+              gtk_drag_finish (context, TRUE, FALSE, time);
+	      break;
+	    }
+	  }
+
+          g_free (pFrom);
+          g_free(sPath);
+
+          if (diagram != NULL) {
+            diagram_update_extents(diagram);
+            layer_dialog_set_diagram(diagram);
+            
+	    if (diagram->displays == NULL) {
+	      new_display(diagram);
+	    }
+          }
+
+          pFrom = strstr(pTo, "file:");
+        } /* while */
+        gtk_drag_finish (context, TRUE, FALSE, time);
+      }
+      break;
+    }
+  return;
+}
+
+static const GtkTargetEntry display_target_table[] = {
+  { "application/x-dia-object", 0, 0 },
+  { "text/uri-list", 0, DIA_DND_TYPE_URI_LIST },
+  { "text/plain", 0, DIA_DND_TYPE_TEXT_PLAIN }
+};
+static int display_n_targets = sizeof(display_target_table)/sizeof(display_target_table[0]);
+
 ToolButton tool_data[] =
 {
-  { (char **) arrow_xpm,
-    N_("Modify object(s)"),
-    N_("Modify"),
+  { (char **) dia_modify_tool_icon,
+    N_("Modify object(s)\nUse <Space> to toggle between this and other tools"),
+    NULL,
+    "ToolsModify",
     { MODIFY_TOOL, NULL, NULL}
   },
-  { (char **) magnify_xpm,
-    N_("Magnify"),
+  { (char **) dia_zoom_tool_icon,
     N_("Magnify"),
+    "<alt>M",
+    "ToolsMagnify",
     { MAGNIFY_TOOL, NULL, NULL}
   },
-  { (char **) scroll_xpm,
+  { (char **) dia_scroll_tool_icon,
     N_("Scroll around the diagram"),
-    N_("Scroll"),
+    "<alt>S",
+    "ToolsScroll",
     { SCROLL_TOOL, NULL, NULL}
   },
   { NULL,
     N_("Text"),
-    N_("Text"),
+    "<alt>T",
+    "ToolsText",
     { CREATE_OBJECT_TOOL, "Standard - Text", NULL }
   },
   { NULL,
     N_("Box"),
-    N_("Box"),
+    "<alt>R",
+    "ToolsBox",
     { CREATE_OBJECT_TOOL, "Standard - Box", NULL }
   },
   { NULL,
     N_("Ellipse"),
-    N_("Ellipse"),
+    "<alt>E",
+    "ToolsEllipse",
     { CREATE_OBJECT_TOOL, "Standard - Ellipse", NULL }
   },
   { NULL,
     N_("Polygon"),
-    N_("Polygon"),
+    "<alt>P",
+    "ToolsPolygon",
     { CREATE_OBJECT_TOOL, "Standard - Polygon", NULL }
   },
   { NULL,
     N_("Beziergon"),
-    N_("Beziergon"),
+    "<alt>B",
+    "ToolsBeziergon",
     { CREATE_OBJECT_TOOL, "Standard - Beziergon", NULL }
   },
   { NULL,
     N_("Line"),
-    N_("Line"),
+    "<alt>L",
+    "ToolsLine",
     { CREATE_OBJECT_TOOL, "Standard - Line", NULL }
   },
   { NULL,
     N_("Arc"),
-    N_("Arc"),
+    "<alt>A",
+    "ToolsArc",
     { CREATE_OBJECT_TOOL, "Standard - Arc", NULL }
   },
   { NULL,
     N_("Zigzagline"),
-    N_("Zigzagline"),
+    "<alt>Z",
+    "ToolsZigzagline",
     { CREATE_OBJECT_TOOL, "Standard - ZigZagLine", NULL }
   },
   { NULL,
     N_("Polyline"),
-    N_("Polyline"),
+    NULL,
+    "ToolsPolyline",
     { CREATE_OBJECT_TOOL, "Standard - PolyLine", NULL }
   },
   { NULL,
     N_("Bezierline"),
-    N_("Bezierline"),
+    "<alt>C",
+    "ToolsBezierline",
     { CREATE_OBJECT_TOOL, "Standard - BezierLine", NULL }
   },
   { NULL,
     N_("Image"),
-    N_("Image"),
+    "<alt>I",
+    "ToolsImage",
     { CREATE_OBJECT_TOOL, "Standard - Image", NULL }
   }
 };
@@ -142,11 +244,15 @@ grid_toggle_snap(GtkWidget *widget, gpointer data)
 			    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)));
 }
 
-/*  The popup shell is a pointer to the display shell that posted the latest
- *  popup menu.  When this is null, and a command is invoked, then the
- *  assumption is that the command was a result of a keyboard accelerator
- */
-GtkWidget *popup_shell = NULL;
+static void
+interface_toggle_mainpoint_magnetism(GtkWidget *widget, gpointer data)
+{
+  DDisplay *ddisp = (DDisplay *)data;
+  ddisplay_set_snap_to_objects(ddisp,
+				   gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)));
+  ddisplay_add_update_all(ddisp);
+  ddisplay_flush(ddisp);
+}
 
 static gint
 origin_button_press(GtkWidget *widget, GdkEventButton *event, gpointer data)
@@ -239,16 +345,13 @@ create_zoom_widget(DDisplay *ddisp) {
   gtk_widget_show_all(button);
 
   menu = gtk_menu_new();
+  zoom_add_zoom_amount(menu, "800%", ddisp);
   zoom_add_zoom_amount(menu, "400%", ddisp);
-  zoom_add_zoom_amount(menu, "283%", ddisp);
   zoom_add_zoom_amount(menu, "200%", ddisp);
-  zoom_add_zoom_amount(menu, "141%", ddisp);
   zoom_add_zoom_amount(menu, "100%", ddisp);
-  zoom_add_zoom_amount(menu, "85%", ddisp);
-  zoom_add_zoom_amount(menu, "70.7%", ddisp);
   zoom_add_zoom_amount(menu, "50%", ddisp);
-  zoom_add_zoom_amount(menu, "35.4%", ddisp);
   zoom_add_zoom_amount(menu, "25%", ddisp);
+  zoom_add_zoom_amount(menu, "12%", ddisp);
 
   gtk_widget_show_all(menu);
 
@@ -273,9 +376,14 @@ display_drop_callback(GtkWidget *widget, GdkDragContext *context,
 }
 
 static void
-display_data_received_callback (GtkWidget *widget, GdkDragContext *context,
-				gint x, gint y, GtkSelectionData *data,
-				guint info, guint time, DDisplay *ddisp)
+display_data_received_callback (GtkWidget *widget, 
+				GdkDragContext *context,
+				gint x, 
+				gint y, 
+				GtkSelectionData *data,
+				guint info, 
+				guint time, 
+				DDisplay *ddisp)
 {
   if (data->format == 8 && data->length == sizeof(ToolButtonData *) &&
       gtk_drag_get_source_widget(context) != NULL) {
@@ -288,7 +396,7 @@ display_data_received_callback (GtkWidget *widget, GdkDragContext *context,
 
     gtk_drag_finish (context, TRUE, FALSE, time);
   } else
-    gtk_drag_finish (context, FALSE, FALSE, time);
+    dia_dnd_file_drag_data_received (widget, context, x, y, data, info, time, ddisp);
 }
 
 void
@@ -303,12 +411,15 @@ create_display_shell(DDisplay *ddisp,
   GtkWidget *zoom_hbox, *zoom_label;
   int s_width, s_height;
 
+  if (!tool_tips) /* needed here if we dont create_toolbox() */
+    tool_tips = gtk_tooltips_new ();
+    
   s_width = gdk_screen_width ();
   s_height = gdk_screen_height ();
   if (width > s_width)
     width = s_width;
   if (height > s_height)
-    width = s_width;
+    height = s_height;
 
   /*  The adjustment datums  */
   ddisp->hsbdata = GTK_ADJUSTMENT (gtk_adjustment_new (0, 0, width, 1, (width-1)/4, width-1));
@@ -327,7 +438,7 @@ create_display_shell(DDisplay *ddisp,
       if (pixbuf)
         gtk_window_set_icon (GTK_WINDOW (ddisp->shell), pixbuf);
     }
-    gtk_window_set_default_size(ddisp->shell, width, height);
+    gtk_window_set_default_size(GTK_WINDOW (ddisp->shell), width, height);
   } else {
     ddisp->shell = gtk_event_box_new ();
   }
@@ -434,15 +545,17 @@ create_display_shell(DDisplay *ddisp,
   ddisp->canvas = dia_canvas_new();
   /* Dia's canvas does it' double buffering alone so switch off GTK's */
   gtk_widget_set_double_buffered (ddisp->canvas, FALSE);
-  dia_canvas_set_size(ddisp->canvas, width, height);
+#if 0 /* the following call forces the minimum diagram window size. But it seems to be superfluous otherwise. */
+  dia_canvas_set_size(DIA_CANVAS (ddisp->canvas), width, height);
+#endif
   gtk_widget_set_events (ddisp->canvas, CANVAS_EVENT_MASK);
   GTK_WIDGET_SET_FLAGS (ddisp->canvas, GTK_CAN_FOCUS);
   g_signal_connect (GTK_OBJECT (ddisp->canvas), "event",
 		    G_CALLBACK(ddisplay_canvas_events),
 		      ddisp);
-
+  
   gtk_drag_dest_set(ddisp->canvas, GTK_DEST_DEFAULT_ALL,
-		    create_object_targets, 1, GDK_ACTION_COPY);
+		    display_target_table, display_n_targets, GDK_ACTION_COPY);
   g_signal_connect (GTK_OBJECT (ddisp->canvas), "drag_drop",
 		    G_CALLBACK(display_drop_callback), NULL);
   g_signal_connect (GTK_OBJECT (ddisp->canvas), "drag_data_received",
@@ -467,34 +580,16 @@ create_display_shell(DDisplay *ddisp,
   gtk_table_attach (GTK_TABLE (table), navigation_button, 2, 3, 2, 3,
                     GTK_FILL, GTK_FILL, 0, 0);
 
-  menus_get_image_menu (NULL, &ddisp->accel_group);
+  /* TODO rob use per window accel */
+  ddisp->accel_group = menus_get_display_accels ();
   if (top_level_window)
     gtk_window_add_accel_group(GTK_WINDOW(ddisp->shell), ddisp->accel_group);
   if (use_mbar) 
   {
-    GString *path;
-    char *display = "<DisplayMBar>";
-
-    menus_get_image_menubar(&ddisp->menu_bar, &ddisp->mbar_item_factory);
+    ddisp->menu_bar = menus_create_display_menubar (&ddisp->ui_manager, &ddisp->actions);
+    g_assert (ddisp->menu_bar);
     gtk_box_pack_start (GTK_BOX (root_vbox), ddisp->menu_bar, FALSE, TRUE, 0);
-
-    path = g_string_new (display);
-    g_string_append (path,"/View/Show Rulers");
-    ddisp->rulers       = menus_get_item_from_path(path->str, ddisp->mbar_item_factory);
-    g_string_append (g_string_assign(path, display),"/View/Show Grid");
-    ddisp->visible_grid = menus_get_item_from_path(path->str, ddisp->mbar_item_factory);
-    g_string_append (g_string_assign(path, display),"/View/Snap To Grid");
-    ddisp->snap_to_grid = menus_get_item_from_path(path->str, ddisp->mbar_item_factory);
-    g_string_append (g_string_assign(path, display),"/View/Show Connection Points");
-    ddisp->show_cx_pts_mitem  = menus_get_item_from_path(path->str, ddisp->mbar_item_factory);
-
-#ifdef HAVE_LIBART
-    g_string_append (g_string_assign(path, display),"/View/AntiAliased");
-    ddisp->antialiased  = menus_get_item_from_path(path->str, ddisp->mbar_item_factory);
-#endif
-
-    menus_initialize_updatable_items (&ddisp->updatable_menu_items, ddisp->mbar_item_factory, display);
-    g_string_free (path,FALSE);
+    menus_initialize_updatable_items (&ddisp->updatable_menu_items, ddisp->actions);
   }
 
   /* the statusbars */
@@ -517,15 +612,28 @@ create_display_shell(DDisplay *ddisp,
   gtk_box_pack_start (GTK_BOX (status_hbox), zoom_hbox, FALSE, FALSE, 0);
 
   /* Grid on/off button */
-  ddisp->grid_status = dia_toggle_button_new_with_images("on-grid.png", "off-grid.png");
+  ddisp->grid_status = dia_toggle_button_new_with_icons(dia_on_grid_icon,
+							dia_off_grid_icon);
   
   g_signal_connect(G_OBJECT(ddisp->grid_status), "toggled",
-		   grid_toggle_snap, ddisp);
+		   G_CALLBACK (grid_toggle_snap), ddisp);
   gtk_tooltips_set_tip(tool_tips, ddisp->grid_status,
 		       _("Toggles snap-to-grid for this window."), NULL);
   gtk_box_pack_start (GTK_BOX (status_hbox), ddisp->grid_status,
 		      FALSE, FALSE, 0);
 
+
+  ddisp->mainpoint_status = dia_toggle_button_new_with_icons(dia_mainpoints_on_icon,
+							dia_mainpoints_off_icon);
+  
+  g_signal_connect(G_OBJECT(ddisp->mainpoint_status), "toggled",
+		   G_CALLBACK (interface_toggle_mainpoint_magnetism), ddisp);
+  gtk_tooltips_set_tip(tool_tips, ddisp->mainpoint_status,
+		       _("Toggles object snapping for this window."), NULL);
+  gtk_box_pack_start (GTK_BOX (status_hbox), ddisp->mainpoint_status,
+		      FALSE, FALSE, 0);
+
+
   /* Statusbar */
   ddisp->modified_status = gtk_statusbar_new ();
 
@@ -544,6 +652,7 @@ create_display_shell(DDisplay *ddisp,
   gtk_widget_show (zoom_hbox);
   gtk_widget_show (zoom_label);
   gtk_widget_show (ddisp->grid_status);
+  gtk_widget_show (ddisp->mainpoint_status);
   gtk_widget_show (ddisp->modified_status);
   gtk_widget_show (status_hbox);
   gtk_widget_show (table);
@@ -634,7 +743,7 @@ tool_setup_drag_source(GtkWidget *button, ToolButtonData *tooldata,
   g_return_if_fail(tooldata->type == CREATE_OBJECT_TOOL);
 
   gtk_drag_source_set(button, GDK_BUTTON1_MASK,
-		      create_object_targets, 1,
+		      display_target_table, display_n_targets,
 		      GDK_ACTION_DEFAULT|GDK_ACTION_COPY);
   g_signal_connect(GTK_OBJECT(button), "drag_data_get",
 		   G_CALLBACK(tool_drag_data_get), tooldata);
@@ -660,13 +769,43 @@ tool_select_callback(GtkWidget *widget, gpointer data) {
 }
 */
 
+/*
+ * Don't look too deep into this function. It is doing bad things
+ * with casts to conform to the historically interface. We know
+ * the difference between char* and char** - most of the time ;)
+ */
+static GtkWidget *
+create_widget_from_xpm_or_gdkp(gchar **icon_data, GtkWidget *button) 
+{
+  GtkWidget *pixmapwidget;
+
+  if (strncmp((char*)icon_data, "GdkP", 4) == 0) {
+    GdkPixbuf *p;
+    p = gdk_pixbuf_new_from_inline(-1, (char*)icon_data, TRUE, NULL);
+    pixmapwidget = gtk_image_new_from_pixbuf(p);
+  } else {
+    GdkBitmap *mask = NULL;
+    GtkStyle *style;
+    char **pixmap_data;
+    GdkPixmap *pixmap = NULL;
+
+    pixmap_data = icon_data;
+    style = gtk_widget_get_style(button);
+    pixmap = gdk_pixmap_colormap_create_from_xpm_d(NULL,
+						   gtk_widget_get_colormap(button), &mask,
+						   &style->bg[GTK_STATE_NORMAL], pixmap_data);
+    pixmapwidget = gtk_pixmap_new(pixmap, mask);
+  }
+  return pixmapwidget;
+}
+
 static void
 create_tools(GtkWidget *parent)
 {
   GtkWidget *button;
   GtkWidget *pixmapwidget;
-  GdkPixmap *pixmap;
-  GdkBitmap *mask;
+  GdkPixmap *pixmap = NULL;
+  GdkBitmap *mask = NULL;
   GtkStyle *style;
   char **pixmap_data;
   int i;
@@ -693,17 +832,11 @@ create_tools(GtkWidget *parent)
 	pixmap_data = tool_data[0].icon_data;
       else
 	pixmap_data = type->pixmap;
+      pixmapwidget = create_widget_from_xpm_or_gdkp(pixmap_data, button);
     } else {
-      pixmap_data = tool_data[i].icon_data;
+      pixmapwidget = create_widget_from_xpm_or_gdkp(tool_data[i].icon_data, button);
     }
     
-    style = gtk_widget_get_style(button);
-    pixmap = gdk_pixmap_colormap_create_from_xpm_d(NULL,
-		gtk_widget_get_colormap(button), &mask,
-		&style->bg[GTK_STATE_NORMAL], pixmap_data);
-    
-    pixmapwidget = gtk_pixmap_new(pixmap, mask);
-    
     /* GTKBUG:? padding changes */
     gtk_misc_set_padding(GTK_MISC(pixmapwidget), 2, 2);
     
@@ -721,13 +854,28 @@ create_tools(GtkWidget *parent)
       tool_setup_drag_source(button, &tool_data[i].callback_data,
 			     pixmap, mask);
 
-    gdk_pixmap_unref(pixmap);
+    if (pixmap) gdk_pixmap_unref(pixmap);
     if (mask) gdk_bitmap_unref(mask);
 
     tool_data[i].callback_data.widget = button;
 
-    gtk_tooltips_set_tip (tool_tips, button,
-			  gettext(tool_data[i].tool_desc), NULL);
+    if (tool_data[i].tool_accelerator) {
+	guint key;
+	GdkModifierType mods;
+	gchar *alabel, *atip;
+
+	gtk_accelerator_parse (tool_data[i].tool_accelerator, &key, &mods);
+
+	alabel = gtk_accelerator_get_label(key, mods);
+	atip = g_strconcat(gettext(tool_data[i].tool_desc), " (", alabel, ")", NULL);
+	gtk_tooltips_set_tip (tool_tips, button, atip, NULL);
+	g_free (atip);
+	g_free (alabel);
+
+    } else {
+	gtk_tooltips_set_tip (tool_tips, button,
+				gettext(tool_data[i].tool_desc), NULL);
+    }
     
     gtk_widget_show (pixmapwidget);
     gtk_widget_show (button);
@@ -735,112 +883,27 @@ create_tools(GtkWidget *parent)
 }
 
 static GtkWidget *sheet_option_menu;
-static GtkWidget *sheet_menu;
 static GtkWidget *sheet_wbox;
 
-#if 0
-static GtkWidget *
-create_sheet_page(GtkWidget *parent, Sheet *sheet)
-{
-  GSList *list;
-  SheetObject *sheet_obj;
-  GtkWidget *table;
-  GtkWidget *button;
-  GtkWidget *pixmapwidget;
-  GdkPixmap *pixmap;
-  GdkBitmap *mask;
-  ToolButtonData *data;
-  GtkStyle *style;
-  GtkWidget *scrolled_window;
-  int i;
-  int rows;
-
-  scrolled_window = gtk_scrolled_window_new(NULL, NULL);
-  gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW(scrolled_window),
-				  GTK_POLICY_AUTOMATIC,
-				  GTK_POLICY_AUTOMATIC);
-
-  rows = ceil(g_slist_length(sheet->objects) / (double)COLUMNS);
-  if (rows<1)
-    rows = 1;
-  
-  table = gtk_hwrap_box_new(FALSE);
-  gtk_wrap_box_set_aspect_ratio(GTK_WRAP_BOX(table), COLUMNS * 1.0 / rows);
-  gtk_wrap_box_set_justify(GTK_WRAP_BOX(table), GTK_JUSTIFY_TOP);
-  gtk_wrap_box_set_line_justify(GTK_WRAP_BOX(table), GTK_JUSTIFY_LEFT);
-
-  style = gtk_widget_get_style(parent);
-
-  i = 0;
-  list = sheet->objects;
-  while (list != NULL) {
-    sheet_obj = (SheetObject *) list->data;
-    
-    
-    if (sheet_obj->pixmap != NULL) {
-      pixmap = gdk_pixmap_colormap_create_from_xpm_d(NULL,
-			gtk_widget_get_colormap(parent), &mask, 
-			&style->bg[GTK_STATE_NORMAL], sheet_obj->pixmap);
-    } else if (sheet_obj->pixmap_file != NULL) {
-      pixmap = gdk_pixmap_colormap_create_from_xpm(NULL,
-			gtk_widget_get_colormap(parent), &mask,
-			&style->bg[GTK_STATE_NORMAL], sheet_obj->pixmap_file);
-    } else {
-      DiaObjectType *type;
-      type = object_get_type(sheet_obj->object_type);
-      pixmap = gdk_pixmap_colormap_create_from_xpm_d(NULL,
-			gtk_widget_get_colormap(parent), &mask, 
-			&style->bg[GTK_STATE_NORMAL], type->pixmap);
-    }
-
-    pixmapwidget = gtk_pixmap_new(pixmap, mask);
-
-    
-    button = gtk_radio_button_new (tool_group);
-    gtk_container_set_border_width (GTK_CONTAINER (button), 0);
-    tool_group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
-    
-    gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (button), FALSE);
-
-
-    gtk_container_add (GTK_CONTAINER (button), pixmapwidget);
-    gtk_wrap_box_pack_wrapped(GTK_WRAP_BOX(table), button,
-		      FALSE, TRUE, FALSE, sheet_obj->line_break);
-
-    /* This is a Memory leak, these can't be freed.
-       Doesn't matter much anyway... */
-
-    data = g_new(ToolButtonData, 1);
-    data->type = CREATE_OBJECT_TOOL;
-    data->extra_data = sheet_obj->object_type;
-    data->user_data = sheet_obj->user_data;
-    
-    g_signal_connect (GTK_OBJECT (button), "clicked",
-		      G_CALLBACK (tool_select_update),
-			data);
-    
-    g_signal_connect (GTK_OBJECT (button), "button_press_event",
-		      G_CALLBACK (tool_button_press),
-			data);
-
-    gtk_tooltips_set_tip (tool_tips, button,
-			  gettext(sheet_obj->description), NULL);
-
-    list = g_slist_next(list);
-    i++;
-  }
-
-  gtk_widget_show(table);
-  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled_window), table);
-  
-  return scrolled_window;
-}
-#endif
-
 gchar *interface_current_sheet_name;
 
+static Sheet *
+get_sheet_by_name(const gchar *name)
+{
+  GSList *tmp;
+  for (tmp = get_sheets_list(); tmp != NULL; tmp = tmp->next) {
+    Sheet *sheet = tmp->data;
+    /* There is something fishy with comparing both forms: the english and the localized one.
+     * But we should be on the safe side here, especially when bug #328570 gets tackled.
+     */
+    if (0 == g_strcasecmp(name, sheet->name) || 0 == g_strcasecmp(name, gettext(sheet->name)))
+      return sheet;
+  }
+  return NULL;
+}
+
 static void
-fill_sheet_wbox(GtkWidget *menu_item, Sheet *sheet)
+fill_sheet_wbox(Sheet *sheet)
 {
   int rows;
   GtkStyle *style;
@@ -878,12 +941,15 @@ fill_sheet_wbox(GtkWidget *menu_item, Sheet *sheet)
       
       pixbuf = gdk_pixbuf_new_from_file(sheet_obj->pixmap_file, &gerror);
       if (pixbuf != NULL) {
-          gdk_pixbuf_render_pixmap_and_mask(pixbuf, &pixmap, &mask, 1.0);
+          gdk_pixbuf_render_pixmap_and_mask_for_colormap(pixbuf, gtk_widget_get_colormap(sheet_wbox), &pixmap, &mask, 1.0);
           gdk_pixbuf_unref(pixbuf);
       } else {
-          
-          g_warning("failed to load pixbuf for file %s; cause=%s",
-                    sheet_obj->pixmap_file,gerror?gerror->message:"[NULL]");
+          pixmap = gdk_pixmap_colormap_create_from_xpm_d(NULL,
+			gtk_widget_get_colormap(sheet_wbox), &mask, 
+			&style->bg[GTK_STATE_NORMAL], missing);
+
+          message_warning("failed to load icon for file\n %s\n cause=%s",
+                          sheet_obj->pixmap_file,gerror?gerror->message:"[NULL]");
       }
     } else {
       DiaObjectType *type;
@@ -938,47 +1004,72 @@ fill_sheet_wbox(GtkWidget *menu_item, Sheet *sheet)
 			    GTK_BUTTON(first_button), NULL);
 }
 
+static void
+sheet_menu_callback(DiaDynamicMenu *menu, const gchar *string, void *user_data)
+{
+  Sheet *sheet = get_sheet_by_name(string);
+  if (sheet == NULL) {
+    message_warning(_("No sheet named %s"), string);
+  } else {
+    persistence_set_string("last-sheet-selected", string);
+    fill_sheet_wbox(sheet);
+  }
+}
+
+static int
+cmp_names (const void *a, const void *b)
+{
+  return g_utf8_collate(gettext( (gchar *)a ), gettext( (gchar *)b ));
+}
+
+static GList *
+get_sheet_names()
+{
+  GSList *tmp;
+  GList *names = NULL;
+  for (tmp = get_sheets_list(); tmp != NULL; tmp = tmp->next) {
+    Sheet *sheet = tmp->data;
+    names = g_list_append(names, sheet->name);
+  }
+  /* Already sorted in lib/ but here we sort by the localized (display-)name */
+  return g_list_sort (names, cmp_names);
+}
+
+static void
+create_sheet_dropdown_menu(GtkWidget *parent)
+{
+  GList *sheet_names = get_sheet_names();
+
+  if (sheet_option_menu != NULL) {
+    gtk_container_remove(GTK_CONTAINER(parent), sheet_option_menu);
+    sheet_option_menu = NULL;
+  }
+
+  sheet_option_menu =
+    dia_dynamic_menu_new_stringlistbased(_("Other sheets"), sheet_names, 
+					 sheet_menu_callback,
+					 NULL, "sheets");
+  dia_dynamic_menu_add_default_entry(DIA_DYNAMIC_MENU(sheet_option_menu),
+				     "Assorted");
+  dia_dynamic_menu_add_default_entry(DIA_DYNAMIC_MENU(sheet_option_menu),
+				     "UML");
+  /*    gtk_widget_set_size_request(sheet_option_menu, 20, -1);*/
+  gtk_wrap_box_pack_wrapped(GTK_WRAP_BOX(parent), sheet_option_menu,
+			    TRUE, TRUE, FALSE, FALSE, TRUE);    
+  /* 15 is a magic number that goes beyond the standard objects and
+   * the divider. */
+  gtk_wrap_box_reorder_child(GTK_WRAP_BOX(parent),
+			     sheet_option_menu, 15);
+  gtk_widget_show(sheet_option_menu);
+}
+
 void
 fill_sheet_menu(void)
 {
-  GSList *tmp;
-  GtkWidget *activate = NULL;
-  int index = 0;
-
-  gtk_container_foreach(GTK_CONTAINER(sheet_menu),
-			(GtkCallback)gtk_widget_destroy, NULL);
-  gtk_widget_ref(sheet_menu);
-  gtk_option_menu_remove_menu(GTK_OPTION_MENU(sheet_option_menu));
-  gtk_container_add(GTK_CONTAINER(sheet_menu), gtk_tearoff_menu_item_new());
-  for (tmp = get_sheets_list(); tmp != NULL; tmp = tmp->next) {
-    Sheet *sheet = tmp->data;
-    GtkWidget *menuitem = NULL;
-
-    if (sheet->objects == NULL)
-      continue;
-
-    menuitem = gtk_menu_item_new_with_label(gettext(sheet->name));
-    g_signal_connect(GTK_OBJECT(menuitem), "activate",
-		     G_CALLBACK(fill_sheet_wbox), sheet);
-    gtk_container_add(GTK_CONTAINER(sheet_menu), menuitem);
-  
-    /* off by one with gtk_option_menu_<get/set>_history should start
-     * with 0 but it counts from one, gtk- or my-error ? --hb */
-    index++;
-    if (prefs.recent_sheet == index)
-      activate = menuitem;
-  }
-  gtk_option_menu_set_menu(GTK_OPTION_MENU(sheet_option_menu), sheet_menu);
-  gtk_widget_unref(sheet_menu);
-
-  if (activate)
-    {
-      gtk_option_menu_set_history(GTK_OPTION_MENU(sheet_option_menu), prefs.recent_sheet);
-      /* simple gtk_option_menu_set_history() isn't enough,
-       * cause we need fill_sheet_wbox to be called */
-      gtk_menu_item_activate(GTK_MENU_ITEM(activate));
-    }
-  gtk_widget_show_all(sheet_menu);
+  gchar *selection = dia_dynamic_menu_get_entry(DIA_DYNAMIC_MENU(sheet_option_menu));
+  create_sheet_dropdown_menu(gtk_widget_get_parent(sheet_option_menu));
+  dia_dynamic_menu_select_entry(DIA_DYNAMIC_MENU(sheet_option_menu), selection);
+  g_free(selection);
 }
 
 void
@@ -987,6 +1078,8 @@ create_sheets(GtkWidget *parent)
   GtkWidget *separator;
   GtkWidget *label;
   GtkWidget *swin;
+  gchar *sheetname;
+  Sheet *sheet;
   
   separator = gtk_hseparator_new ();
   /* add a bit of padding around the separator */
@@ -997,17 +1090,11 @@ create_sheets(GtkWidget *parent)
   gtk_wrap_box_pack_wrapped (GTK_WRAP_BOX(parent), label, TRUE,TRUE, FALSE,FALSE, TRUE);
   gtk_widget_show(separator);
 
-  sheet_option_menu = gtk_option_menu_new();
-  gtk_widget_set_size_request(sheet_option_menu, 20, -1);
-  sheet_menu = gtk_menu_new();
-  gtk_option_menu_set_menu(GTK_OPTION_MENU(sheet_option_menu), sheet_menu);
-  gtk_wrap_box_pack_wrapped(GTK_WRAP_BOX(parent), sheet_option_menu,
-		    TRUE, TRUE, FALSE, FALSE, TRUE);
-  gtk_widget_show(sheet_option_menu);
+  create_sheet_dropdown_menu(parent);
 
   swin = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(swin),
-				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+				 GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
   gtk_wrap_box_pack_wrapped(GTK_WRAP_BOX(parent), swin, TRUE, TRUE, TRUE, TRUE, TRUE);
   gtk_widget_show(swin);
 
@@ -1017,7 +1104,17 @@ create_sheets(GtkWidget *parent)
   gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(swin), sheet_wbox);
   gtk_widget_show(sheet_wbox);
 
-  fill_sheet_menu();
+  persistence_register_string("last-sheet-selected", _("Misc"));
+  sheetname = persistence_get_string("last-sheet-selected");
+  sheet = get_sheet_by_name(sheetname);
+  if (sheet == NULL) {
+    /* Couldn't find it */
+  } else {
+    fill_sheet_wbox(sheet);
+    dia_dynamic_menu_select_entry(DIA_DYNAMIC_MENU(sheet_option_menu),
+				  sheetname);
+  }
+  g_free(sheetname);
 }
 
 static void
@@ -1149,8 +1246,6 @@ create_lineprops_area(GtkWidget *parent)
 static void
 toolbox_destroy (GtkWidget *widget, gpointer data)
 {
-  prefs.recent_sheet = gtk_option_menu_get_history(GTK_OPTION_MENU(sheet_option_menu));
-
   app_exit();
 }
 
@@ -1164,83 +1259,14 @@ toolbox_delete (GtkWidget *widget, GdkEvent *event, gpointer data)
 				      0, 0, NULL, toolbox_destroy, NULL);
     if (handlerid != 0)
       g_signal_handler_disconnect (GTK_OBJECT (widget), handlerid);
-    prefs.recent_sheet = gtk_option_menu_get_history(GTK_OPTION_MENU(sheet_option_menu));
 
     /** If the app didn't exit, don't close the window */
     return (!app_exit());
   }
+  return FALSE;
 }
 
 
-/* HB: file dnd stuff lent by The Gimp, not fully understood but working ...
- */
-enum
-{
-  DIA_DND_TYPE_URI_LIST,
-  DIA_DND_TYPE_TEXT_PLAIN,
-};
-
-static GtkTargetEntry toolbox_target_table[] =
-{
-  { "text/uri-list", 0, DIA_DND_TYPE_URI_LIST },
-  { "text/plain", 0, DIA_DND_TYPE_TEXT_PLAIN }
-};
-
-static guint toolbox_n_targets = (sizeof (toolbox_target_table) /
-                                 sizeof (toolbox_target_table[0]));
-
-static void
-dia_dnd_file_drag_data_received (GtkWidget        *widget,
-                                 GdkDragContext   *context,
-                                 gint              x,
-                                 gint              y,
-                                 GtkSelectionData *data,
-                                 guint             info,
-                                 guint             time)
-{
-  switch (context->action)
-    {
-    case GDK_ACTION_DEFAULT:
-    case GDK_ACTION_COPY:
-    case GDK_ACTION_MOVE:
-    case GDK_ACTION_LINK:
-    case GDK_ACTION_ASK:
-    default:
-      {
-        Diagram *diagram = NULL;
-        DDisplay *ddisp;
-        gchar *sPath = NULL, *pFrom, *pTo; 
-
-        pFrom = strstr((gchar *) data->data, "file:");
-        while (pFrom) {
-          GError *error = NULL;
-
-          pTo = pFrom;
-          while (*pTo != 0 && *pTo != 0xd && *pTo != 0xa) pTo ++;
-          sPath = g_strndup(pFrom, pTo - pFrom);
-
-          /* format changed with Gtk+2.0, use conversion */
-          pFrom = g_filename_from_uri (sPath, NULL, &error);
-          diagram = diagram_load (pFrom, NULL);
-          g_free (pFrom);
-          g_free(sPath);
-
-          if (diagram != NULL) {
-            diagram_update_extents(diagram);
-            layer_dialog_set_diagram(diagram);
-            
-            ddisp = new_display(diagram);
-          }
-
-          pFrom = strstr(pTo, "file:");
-        } /* while */
-        gtk_drag_finish (context, TRUE, FALSE, time);
-      }
-      break;
-    }
-  return;
-}
-
 void
 create_toolbox ()
 {
@@ -1314,7 +1340,7 @@ create_toolbox ()
 		     GDK_ACTION_COPY);
   g_signal_connect (GTK_OBJECT (wrapbox), "drag_data_received",
 		    G_CALLBACK (dia_dnd_file_drag_data_received),
-                      wrapbox);
+                    NULL); /* userdata == NULL here intentionally */
 
   /* menus -- initialised afterwards, because initing the display menus
    * uses the tool buttons*/
@@ -1346,9 +1372,8 @@ toolbox_hide(void)
 void
 create_tree_window(void)
 {
-  GtkCheckMenuItem *item = GTK_CHECK_MENU_ITEM
-    (menus_get_item_from_path("<Toolbox>/File/Diagram tree", NULL));
-  create_diagram_tree_window(&prefs.dia_tree, GTK_WIDGET(item));
+  GtkAction *action = menus_get_action ("FileTree");
+  create_diagram_tree_window(&prefs.dia_tree, GTK_TOGGLE_ACTION(action));
 }
 
 GtkWidget *
