@@ -22,27 +22,30 @@
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <assert.h>
 #include <string.h>
 #include <gdk/gdkkeysyms.h>
+#include <gtk/gtk.h>
 
-#include "config.h"
 #include "intl.h"
 
 #include "layer_dialog.h"
+#include "persistence.h"
+#include "widgets.h"
+#include "interface.h"
 
-#include "pixmaps/eye.xbm"
-#include "pixmaps/new.xpm"
-#include "pixmaps/lower.xpm"
-#include "pixmaps/raise.xpm"
-#include "pixmaps/delete.xpm"
+#include "dia-app-icons.h"
 
 static struct LayerDialog *layer_dialog = NULL;
 
 typedef struct _ButtonData ButtonData;
 
 struct _ButtonData {
-  gchar **xpm_data;
+  gchar *stock_name;
   gpointer callback;
   char *tooltip;
 };
@@ -51,10 +54,42 @@ enum LayerChangeType {
   TYPE_DELETE_LAYER,
   TYPE_ADD_LAYER,
   TYPE_RAISE_LAYER,
-  TYPE_LOWER_LAYER
+  TYPE_LOWER_LAYER,
 };
+
+struct LayerChange {
+  Change change;
+
+  enum LayerChangeType type;
+  Layer *layer;
+  int index;
+  int applied;
+};
+
+struct LayerVisibilityChange {
+  Change change;
+
+  GList *original_visibility;
+  Layer *layer;
+  gboolean is_exclusive;
+  int applied;
+};
+
+/** If TRUE, we're in the middle of a internal call to 
+ * dia_layer_widget_*_toggled and should not make undo, update diagram etc.
+ *
+ * If these calls were not done by simulating button presses, we could avoid
+ * this hack.
+ */
+static gboolean internal_call = FALSE;
+
 static Change *
 undo_layer(Diagram *dia, Layer *layer, enum LayerChangeType, int index);
+static struct LayerVisibilityChange *
+undo_layer_visibility(Diagram *dia, Layer *layer, gboolean exclusive);
+static void
+layer_visibility_change_apply(struct LayerVisibilityChange *change, 
+			      Diagram *dia);
 
 static void layer_dialog_new_callback(GtkWidget *widget, gpointer gdata);
 static void layer_dialog_raise_callback(GtkWidget *widget, gpointer gdata);
@@ -63,10 +98,10 @@ static void layer_dialog_delete_callback(GtkWidget *widget, gpointer gdata);
 static void layer_dialog_edit_layer(DiaLayerWidget *layer_widget);
 
 static ButtonData buttons[] = {
-  { new_xpm, layer_dialog_new_callback, N_("New Layer") },
-  { raise_xpm, layer_dialog_raise_callback, N_("Raise Layer") },
-  { lower_xpm, layer_dialog_lower_callback, N_("Lower Layer") },
-  { delete_xpm, layer_dialog_delete_callback, N_("Delete Layer") },
+  { GTK_STOCK_ADD, layer_dialog_new_callback, N_("New Layer") },
+  { GTK_STOCK_GO_UP, layer_dialog_raise_callback, N_("Raise Layer") },
+  { GTK_STOCK_GO_DOWN, layer_dialog_lower_callback, N_("Lower Layer") },
+  { GTK_STOCK_DELETE, layer_dialog_delete_callback, N_("Delete Layer") },
 };
 
 enum {
@@ -85,39 +120,19 @@ static int num_buttons = sizeof(buttons)/sizeof(ButtonData);
 #define SELECTED 1
 #define INSENSITIVE 2
 
-static GdkPixmap *eye_pixmap[3] = {NULL, NULL, NULL};
-
 static GtkWidget *
 create_button_box(GtkWidget *parent)
 {
   GtkWidget *button;
   GtkWidget *button_box;
-  GtkWidget *pixmapwid;
-  GdkPixmap *pixmap;
-  GdkBitmap *mask;
-  GdkColormap *cmap;
-  GtkStyle *style;
   int i;
   
-  GtkTooltips *tool_tips = NULL; /* ARG */
-  
-  gtk_widget_ensure_style(parent);
-  style = gtk_widget_get_style(parent);
-  cmap = gtk_widget_get_colormap(parent);
-
   button_box = gtk_hbox_new (TRUE, 1);
 
   for (i=0;i<num_buttons;i++) {
-    pixmap = gdk_pixmap_colormap_create_from_xpm_d (NULL, cmap,
-		&mask, &style->bg[GTK_STATE_NORMAL], buttons[i].xpm_data);
-      
-    pixmapwid =  gtk_pixmap_new (pixmap, mask);
-    gtk_widget_show(pixmapwid);
-
-    button = gtk_button_new ();
-    gtk_container_add (GTK_CONTAINER (button), pixmapwid);
-    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
-			       (GtkSignalFunc) buttons[i].callback,
+    button = gtk_button_new_from_stock(buttons[i].stock_name);
+    g_signal_connect_swapped (GTK_OBJECT (button), "clicked",
+			     G_CALLBACK(buttons[i].callback),
 			       GTK_OBJECT (parent));
           
     if (tool_tips != NULL)
@@ -179,6 +194,13 @@ layer_list_events (GtkWidget *widget,
   return FALSE;
 }
 
+static gboolean
+layer_dialog_delete(GtkWidget *widget, gpointer data)
+{
+  gtk_widget_hide(widget);
+  /* We're caching, so don't destroy */
+  return TRUE;
+}
 
 void
 create_layer_dialog(void)
@@ -201,19 +223,20 @@ create_layer_dialog(void)
   
   layer_dialog->dialog = dialog = gtk_dialog_new ();
   gtk_window_set_title (GTK_WINDOW (dialog), _("Layers"));
-  gtk_window_set_wmclass (GTK_WINDOW (dialog),
-			  "layer_window", "Dia");
-  gtk_window_set_policy (GTK_WINDOW (dialog), FALSE, TRUE, TRUE);
+  gtk_window_set_role (GTK_WINDOW (dialog), "layer_window");
+  gtk_window_set_resizable (GTK_WINDOW (dialog), TRUE);
 
-  gtk_signal_connect (GTK_OBJECT (dialog), "delete_event",
-                      GTK_SIGNAL_FUNC(gtk_widget_hide), NULL);
+  g_signal_connect (GTK_OBJECT (dialog), "delete_event",
+                      G_CALLBACK(layer_dialog_delete), NULL);
+  g_signal_connect (GTK_OBJECT (dialog), "destroy",
+                      G_CALLBACK(gtk_widget_destroyed), 
+		      &(layer_dialog->dialog));
 
-  
   vbox = GTK_DIALOG(dialog)->vbox;
 
   hbox = gtk_hbox_new(FALSE, 1);
   
-  label = gtk_label_new(_("Diagrams:"));
+  label = gtk_label_new(_("Diagram:"));
   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
   gtk_widget_show (label);
   
@@ -246,7 +269,7 @@ create_layer_dialog(void)
   gtk_widget_show (scrolled_win);
   gtk_widget_show (list);
 
-  gtk_signal_connect (GTK_OBJECT (list), "event",
+  g_signal_connect (GTK_OBJECT (list), "event",
 		      (GtkSignalFunc) layer_list_events,
 		      NULL);
 
@@ -258,16 +281,17 @@ create_layer_dialog(void)
   gtk_container_set_border_width(GTK_CONTAINER(GTK_DIALOG(dialog)->action_area),
 				 2);
 
-  button = gtk_button_new_with_label (_("Close"));
+  button = gtk_button_new_from_stock (GTK_STOCK_CLOSE);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), 
 		      button, TRUE, TRUE, 0);
-  gtk_signal_connect_object(GTK_OBJECT (button), "clicked",
-			    GTK_SIGNAL_FUNC(gtk_widget_hide),
+  g_signal_connect_swapped(GTK_OBJECT (button), "clicked",
+			   G_CALLBACK(gtk_widget_hide),
 			    GTK_OBJECT(dialog));
 
   gtk_widget_show (button);
 
-  
+  persistence_register_window(GTK_WINDOW(dialog));
+
   layer_dialog_update_diagram_list();
 }
 
@@ -282,11 +306,29 @@ dia_layer_select_callback(GtkWidget *widget, gpointer data)
   data_set_active_layer(lw->dia->data, lw->layer);
   diagram_add_update_all(lw->dia);
   diagram_flush(lw->dia);
+
+  internal_call = TRUE;
+  if (lw->connect_off) { /* If the user wants this off, it becomes so */
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lw->connectable), FALSE);
+  } else {
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lw->connectable), TRUE);
+  }
+  internal_call = FALSE;
 }
 
 static void
 dia_layer_deselect_callback(GtkWidget *widget, gpointer data)
 {
+  DiaLayerWidget *lw = DIA_LAYER_WIDGET(widget);
+
+  /** If layer dialog or diagram is missing, we are so dead. */
+  if (layer_dialog == NULL || layer_dialog->diagram == NULL) return;
+
+  internal_call = TRUE;
+  /** Set to on if the user has requested so. */
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lw->connectable), 
+			       lw->connect_on);
+  internal_call = FALSE;
 }
 
 
@@ -299,11 +341,14 @@ layer_dialog_new_callback(GtkWidget *widget, gpointer gdata)
   GList *list = NULL;
   GtkWidget *layer_widget;
   int pos;
+  static int next_layer_num = 1;
 
   dia = layer_dialog->diagram;
 
   if (dia != NULL) {
-    layer = new_layer(g_strdup(_("New layer")));
+    gchar* new_layer_name = g_strdup_printf(_("New layer %d"),
+					    next_layer_num++);
+    layer = new_layer(new_layer_name, dia->data);
 
     assert(GTK_LIST(layer_dialog->layer_list)->selection != NULL);
     selected = GTK_LIST(layer_dialog->layer_list)->selection->data;
@@ -471,13 +516,20 @@ layer_dialog_update_diagram_list(void)
   char *filename;
   int i;
   int current_nr;
-    
+
+  if (layer_dialog == NULL || layer_dialog->dialog == NULL) {
+    if (!dia_open_diagrams())
+      return; /* shortcut; maybe session end w/o this dialog */
+    else
+      create_layer_dialog();
+  }
+        
   new_menu = gtk_menu_new();
 
   current_nr = -1;
   
   i = 0;
-  dia_list = open_diagrams;
+  dia_list = dia_open_diagrams();
   while (dia_list != NULL) {
     dia = (Diagram *) dia_list->data;
 
@@ -485,7 +537,7 @@ layer_dialog_update_diagram_list(void)
       current_nr = i;
     }
     
-    filename = strrchr(dia->filename, '/');
+    filename = strrchr(dia->filename, G_DIR_SEPARATOR);
     if (filename==NULL) {
       filename = dia->filename;
     } else {
@@ -494,7 +546,7 @@ layer_dialog_update_diagram_list(void)
 
     menu_item = gtk_menu_item_new_with_label(filename);
 
-    gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
+    g_signal_connect (GTK_OBJECT (menu_item), "activate",
 			(GtkSignalFunc) layer_dialog_select_diagram_callback,
 			(gpointer) dia);
 
@@ -505,9 +557,9 @@ layer_dialog_update_diagram_list(void)
     i++;
   }
 
-  if (open_diagrams==NULL) {
+  if (dia_open_diagrams()==NULL) {
     menu_item = gtk_menu_item_new_with_label (_("none"));
-    gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
+    g_signal_connect (GTK_OBJECT (menu_item), "activate",
 			(GtkSignalFunc) layer_dialog_select_diagram_callback,
 			(gpointer) NULL);
     gtk_menu_append( GTK_MENU(new_menu), menu_item);
@@ -525,8 +577,8 @@ layer_dialog_update_diagram_list(void)
 
   if (current_nr == -1) {
     dia = NULL;
-    if (open_diagrams!=NULL) {
-      dia = (Diagram *) open_diagrams->data;
+    if (dia_open_diagrams()!=NULL) {
+      dia = (Diagram *) dia_open_diagrams()->data;
     }
     layer_dialog_set_diagram(dia);
   }
@@ -535,7 +587,22 @@ layer_dialog_update_diagram_list(void)
 void
 layer_dialog_show()
 {
-  gtk_widget_show(layer_dialog->dialog);
+  if (layer_dialog == NULL || layer_dialog->dialog == NULL)
+    create_layer_dialog();
+  gtk_window_present(GTK_WINDOW(layer_dialog->dialog));
+}
+
+/*
+ * Used to avoid writing to possibly already deleted layer in
+ * dia_layer_widget_connectable_toggled(). Must be called before
+ * e.g. gtk_list_clear_items() cause that will emit the toggled
+ * signal to last focus widget. See bug #329096
+ */
+static void
+_layer_widget_clear_layer (GtkWidget *widget, gpointer user_data)
+{
+  DiaLayerWidget *lw = DIA_LAYER_WIDGET(widget);
+  lw->layer = NULL;
 }
 
 void
@@ -548,14 +615,18 @@ layer_dialog_set_diagram(Diagram *dia)
   int sel_pos;
   int i,j;
 
-  
   if (dia!=NULL)
     active_layer = dia->data->active_layer;
-  
+
+  if (layer_dialog == NULL || layer_dialog->dialog == NULL) 
+    create_layer_dialog(); /* May have been destroyed */
+
+  gtk_container_foreach (GTK_LIST(layer_dialog->layer_list),
+                         _layer_widget_clear_layer, NULL);
   gtk_list_clear_items(GTK_LIST(layer_dialog->layer_list), 0, -1);
   layer_dialog->diagram = dia;
   if (dia != NULL) {
-    i = g_list_index(open_diagrams, dia);
+    i = g_list_index(dia_open_diagrams(), dia);
     if (i >= 0)
       gtk_option_menu_set_history(GTK_OPTION_MENU(layer_dialog->diagram_omenu),
 				  i);
@@ -581,6 +652,14 @@ layer_dialog_set_diagram(Diagram *dia)
 
 /******* DiaLayerWidget: *****/
 
+/* The connectability buttons don't quite behave the way they should.
+ * The shift-click behaviour messes up the active layer.
+ * To fix this, we need to rework the code so that the setting of
+ * connect_on and connect_off is not tied to the button toggling,
+ * but determined by what caused it (creation, user selection,
+ * shift-selection).
+ */
+
 static void
 dia_layer_widget_unrealize(GtkWidget *widget)
 {
@@ -608,184 +687,105 @@ dia_layer_widget_class_init(DiaLayerWidgetClass *klass)
 }
 
 static void
-dia_layer_widget_visible_redraw(DiaLayerWidget *layer_widget)
+dia_layer_widget_set_connectable(DiaLayerWidget *widget, gboolean on)
 {
-  GdkPixmap *pixmap;
-  GdkColor *color;
-  GtkStateType state;
-
-  state = GTK_WIDGET(layer_widget)->state;
-
-  if (GTK_WIDGET_IS_SENSITIVE (layer_widget))
-    {
-      if (state == GTK_STATE_SELECTED)
-	color = &layer_widget->visible->style->bg[GTK_STATE_SELECTED];
-      else
-	color = &layer_widget->visible->style->white;
-    }
-  else
-    color = &layer_widget->visible->style->bg[GTK_STATE_INSENSITIVE];
-
-  gdk_window_set_background (layer_widget->visible->window, color);
-
-  if (layer_widget->layer->visible) {
-    if (!eye_pixmap[NORMAL]) {
-      eye_pixmap[NORMAL] =
-	gdk_pixmap_create_from_data (layer_widget->visible->window,
-				     (gchar*) eye_bits, eye_width, eye_height, -1,
-				     &layer_widget->visible->style->fg[GTK_STATE_NORMAL],
-				     &layer_widget->visible->style->white);
-      eye_pixmap[SELECTED] =
-	gdk_pixmap_create_from_data (layer_widget->visible->window,
-				     (gchar*) eye_bits, eye_width, eye_height, -1,
-				     &layer_widget->visible->style->fg[GTK_STATE_SELECTED],
-				     &layer_widget->visible->style->bg[GTK_STATE_SELECTED]);
-      eye_pixmap[INSENSITIVE] =
-	gdk_pixmap_create_from_data (layer_widget->visible->window,
-				     (gchar*) eye_bits, eye_width, eye_height, -1,
-				     &layer_widget->visible->style->fg[GTK_STATE_INSENSITIVE],
-				     &layer_widget->visible->style->bg[GTK_STATE_INSENSITIVE]);
-    }
-
-    if (GTK_WIDGET_IS_SENSITIVE (GTK_WIDGET(layer_widget))) {
-      if (state == GTK_STATE_SELECTED)
-	pixmap = eye_pixmap[SELECTED];
-      else
-	pixmap = eye_pixmap[NORMAL];
-    } else {
-      pixmap = eye_pixmap[INSENSITIVE];
-    }
-
-    gdk_draw_pixmap (layer_widget->visible->window,
-		     layer_widget->visible->style->black_gc,
-		     pixmap, 0, 0, 0, 0, eye_width, eye_height);
-  } else {
-    gdk_window_clear (layer_widget->visible->window);
-  }
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget->connectable), on);
 }
 
 static void
-dia_layer_widget_exclusive_visible(DiaLayerWidget *layer_widget)
+dia_layer_widget_exclusive_connectable(DiaLayerWidget *layer_widget)
 {
   GList *list;
   DiaLayerWidget *lw;
   Layer *layer;
-  int visible = FALSE;
+  int connectable = FALSE;
   int i;
 
-  /*  First determine if _any_ other layer widgets are set to visible  */
+  /*  First determine if _any_ other layer widgets are set to connectable  */
   for (i=0;i<layer_widget->dia->data->layers->len;i++) {
     layer = g_ptr_array_index(layer_widget->dia->data->layers, i);
     if (layer_widget->layer != layer) {
-	visible |= layer->visible;
-    }
-  }
-
-  /*  Now, toggle the visibility for all layers except the specified one  */
-  for (i=0;i<layer_widget->dia->data->layers->len;i++) {
-    layer = g_ptr_array_index(layer_widget->dia->data->layers, i);
-    if (layer_widget->layer != layer) {
-	visible |= layer->visible;
+	connectable |= layer->connectable;
     }
   }
 
+  /*  Now, toggle the connectability for all layers except the specified one  */
   list = GTK_LIST(layer_dialog->layer_list)->children;
   while (list) {
     lw = DIA_LAYER_WIDGET(list->data);
-    if (lw != layer_widget)
-      lw->layer->visible = !visible;
-    else
-      lw->layer->visible = TRUE;
-    
-    dia_layer_widget_visible_redraw (lw);
+    if (lw != layer_widget) 
+      dia_layer_widget_set_connectable(lw, !connectable);
+    else {
+      dia_layer_widget_set_connectable(lw, TRUE);
+    }
+    gtk_widget_queue_draw(GTK_WIDGET(lw));
     
     list = g_list_next(list);
   }
 }
 
-static gint
-dia_layer_widget_button_events(GtkWidget *widget,
-			       GdkEvent  *event,
-			       gpointer data)
+static gboolean shifted = FALSE;
+
+static gboolean
+dia_layer_widget_button_event(GtkWidget *widget,
+			      GdkEventButton *event,
+			      gpointer userdata)
 {
-  static GtkWidget *click_widget = NULL;
-  static int button_down = 0;
-  static int old_state;
-  static int exclusive;
-  GtkWidget *event_widget;
-  gint return_val;
-  GdkEventButton *bevent;
-  DiaLayerWidget *lw;
-  
-  lw = DIA_LAYER_WIDGET(data);
+  DiaLayerWidget *lw = DIA_LAYER_WIDGET(userdata);
 
+  shifted = event->state & GDK_SHIFT_MASK;
+  internal_call = FALSE;
+  /* Redraw the label? */
+  gtk_widget_queue_draw(GTK_WIDGET(lw));
+  return FALSE;
+}
 
-  return_val = FALSE;
-  switch (event->type) {
-  case GDK_EXPOSE:
-    if (widget == lw->visible)
-      dia_layer_widget_visible_redraw (lw);
-    break;
-    
-  case GDK_BUTTON_PRESS:
-    return_val = TRUE;
-    
-    bevent = (GdkEventButton *) event;
-    
-    button_down = 1;
-    click_widget = widget;
-    gtk_grab_add (click_widget);
-    
-    if (widget == lw->visible) {
-      old_state = lw->layer->visible;
-      
-      /*  If this was a shift-click, make all/none visible  */
-      if (event->button.state & GDK_SHIFT_MASK) {
-	exclusive = TRUE;
-	dia_layer_widget_exclusive_visible (lw);
-      } else {
-	exclusive = FALSE;
-	lw->layer->visible = !lw->layer->visible;
-	dia_layer_widget_visible_redraw (lw);
-      }
-    }
-    break;
-    
-  case GDK_BUTTON_RELEASE:
-    return_val = TRUE;
-    
-    button_down = 0;
-    gtk_grab_remove (click_widget);
-    
-    if (widget == lw->visible) {
-      if ((exclusive) || (old_state != lw->layer->visible)) {
-	diagram_add_update_all(lw->dia);
-	diagram_flush(lw->dia);
-      }
-    } 
-    break;
-    
-  case GDK_ENTER_NOTIFY:
-  case GDK_LEAVE_NOTIFY:
-    event_widget = gtk_get_event_widget (event);
-    
-    if (button_down && (event_widget == click_widget)) {
-      if (widget == lw->visible) {
-	if (exclusive) {
-	  dia_layer_widget_exclusive_visible (lw);
-	} else {
-	  lw->layer->visible = !lw->layer->visible;
-	  dia_layer_widget_visible_redraw (lw);
-	}
-      } 
-    }
-    break;
-    
-  default:
-    break;
+static void
+dia_layer_widget_connectable_toggled(GtkToggleButton *widget,
+				     gpointer userdata)
+{
+  DiaLayerWidget *lw = DIA_LAYER_WIDGET(userdata);
+  if (!lw->layer)
+    return;
+  if (shifted) {
+    shifted = FALSE;
+    internal_call = TRUE;
+    dia_layer_widget_exclusive_connectable(lw);
+    internal_call = FALSE;
+  } else {
+    lw->layer->connectable = gtk_toggle_button_get_active(widget);
+  }
+  if (lw->layer == lw->dia->data->active_layer) {
+    lw->connect_off = !gtk_toggle_button_get_active(widget);
+    if (lw->connect_off) lw->connect_on = FALSE;
+  } else {
+    lw->connect_on = gtk_toggle_button_get_active(widget);
+    if (lw->connect_on) lw->connect_off = FALSE;
+  }
+  gtk_widget_queue_draw(GTK_WIDGET(lw));
+  if (!internal_call) {
+    diagram_add_update_all(lw->dia);
+    diagram_flush(lw->dia);
+  }
+}
+
+static void
+dia_layer_widget_visible_clicked(GtkToggleButton *widget,
+				 gpointer userdata)
+{
+  DiaLayerWidget *lw = DIA_LAYER_WIDGET(userdata);
+  struct LayerVisibilityChange *change;
+
+  /* Have to use this internal_call hack 'cause there's no way to switch
+   * a toggle button without causing the 'clicked' event:(
+   */
+  if (!internal_call) {
+    Diagram *dia = lw->dia;
+    change = undo_layer_visibility(dia, lw->layer, shifted);
+    /** This apply kills 'lw', thus we have to hold onto 'lw->dia' */
+    layer_visibility_change_apply(change, dia);
+    undo_set_transactionpoint(dia->undo);   
   }
-  
-  return return_val;
 }
 
 static void
@@ -793,27 +793,45 @@ dia_layer_widget_init(DiaLayerWidget *lw)
 {
   GtkWidget *hbox;
   GtkWidget *visible;
+  GtkWidget *connectable;
   GtkWidget *label;
-  GtkWidget *alignment;
 
   hbox = gtk_hbox_new(FALSE, 0);
 
   lw->dia = NULL;
   lw->layer = NULL;
   lw->edit_dialog = NULL;
-  
-  alignment = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
-  lw->visible = visible = gtk_drawing_area_new ();
-  gtk_drawing_area_size (GTK_DRAWING_AREA (visible), eye_width, eye_height);
-  gtk_widget_set_events (visible, BUTTON_EVENT_MASK);
-  gtk_signal_connect (GTK_OBJECT (visible), "event",
-                      (GtkSignalFunc) dia_layer_widget_button_events,
-                      lw);
-  gtk_object_set_user_data (GTK_OBJECT (visible), lw);
+
+  lw->connect_on = FALSE;
+  lw->connect_off = FALSE;
+ 
+  lw->visible = visible = 
+    dia_toggle_button_new_with_icons(dia_visible_icon, dia_visible_empty_icon);
+
+  g_signal_connect(G_OBJECT(visible), "button-release-event",
+		   G_CALLBACK(dia_layer_widget_button_event), lw);
+  g_signal_connect(G_OBJECT(visible), "button-press-event",
+		   G_CALLBACK(dia_layer_widget_button_event), lw);
+  g_signal_connect(G_OBJECT(visible), "clicked",
+		   G_CALLBACK(dia_layer_widget_visible_clicked), lw);
+  gtk_box_pack_start (GTK_BOX (hbox), visible, FALSE, TRUE, 2);
   gtk_widget_show(visible);
-  gtk_container_add (GTK_CONTAINER (alignment), visible);
-  gtk_box_pack_start (GTK_BOX (hbox), alignment, FALSE, TRUE, 2);
-  gtk_widget_show(alignment);
+
+  /*gtk_image_new_from_stock(GTK_STOCK_CONNECT, 
+			    GTK_ICON_SIZE_BUTTON), */
+  lw->connectable = connectable = 
+    dia_toggle_button_new_with_icons(dia_connectable_icon,
+				     dia_connectable_empty_icon);
+
+  g_signal_connect(G_OBJECT(connectable), "button-release-event",
+		   G_CALLBACK(dia_layer_widget_button_event), lw);
+  g_signal_connect(G_OBJECT(connectable), "button-press-event",
+		   G_CALLBACK(dia_layer_widget_button_event), lw);
+  g_signal_connect(G_OBJECT(connectable), "clicked",
+		   G_CALLBACK(dia_layer_widget_connectable_toggled), lw);
+
+  gtk_box_pack_start (GTK_BOX (hbox), connectable, FALSE, TRUE, 2);
+  gtk_widget_show(connectable);
   
   lw->label = label = gtk_label_new("layer_default_label");
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
@@ -824,28 +842,29 @@ dia_layer_widget_init(DiaLayerWidget *lw)
 
   gtk_container_add(GTK_CONTAINER(lw), hbox);
 
-  gtk_signal_connect (GTK_OBJECT (lw), "select",
+  g_signal_connect (GTK_OBJECT (lw), "select",
 		      (GtkSignalFunc) dia_layer_select_callback,
 		      (gpointer) NULL);
-  gtk_signal_connect (GTK_OBJECT (lw), "deselect",
+  g_signal_connect (GTK_OBJECT (lw), "deselect",
 		      (GtkSignalFunc) dia_layer_deselect_callback, 
 		      (gpointer) NULL);
 }
 
-guint
+GtkType
 dia_layer_widget_get_type(void)
 {
-  static guint dlw_type = 0;
+  static GtkType dlw_type = 0;
 
   if (!dlw_type) {
-    GtkTypeInfo dlw_info = {
+    static const GtkTypeInfo dlw_info = {
       "DiaLayerWidget",
       sizeof (DiaLayerWidget),
       sizeof (DiaLayerWidgetClass),
       (GtkClassInitFunc) dia_layer_widget_class_init,
       (GtkObjectInitFunc) dia_layer_widget_init,
-      (GtkArgSetFunc) NULL,
-      (GtkArgGetFunc) NULL
+      NULL,
+      NULL,
+      (GtkClassInitFunc) NULL,
     };
     
     dlw_type = gtk_type_unique (gtk_list_item_get_type (), &dlw_info);
@@ -861,10 +880,16 @@ dia_layer_widget_new(Diagram *dia, Layer *layer)
   
   widget = GTK_WIDGET ( gtk_type_new (dia_layer_widget_get_type ()));
   dia_layer_set_layer(DIA_LAYER_WIDGET(widget), dia, layer);
-  
+
+  /* These may get toggled when the button is set without the widget being
+   * selected first. */
+  DIA_LAYER_WIDGET(widget)->connect_on = FALSE;
+  DIA_LAYER_WIDGET(widget)->connect_off = FALSE;
+
   return widget;
 }
 
+/** Layer has either been selected or created */
 void
 dia_layer_set_layer(DiaLayerWidget *widget, Diagram *dia, Layer *layer)
 {
@@ -875,9 +900,17 @@ dia_layer_set_layer(DiaLayerWidget *widget, Diagram *dia, Layer *layer)
 }
 
 void
-dia_layer_update_from_layer(DiaLayerWidget *widget)
+dia_layer_update_from_layer (DiaLayerWidget *widget)
 {
-  gtk_label_set_text(GTK_LABEL(widget->label), widget->layer->name);
+  internal_call = TRUE;
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget->visible),
+			       widget->layer->visible);
+  
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget->connectable),
+			       widget->layer->connectable);
+  internal_call = FALSE;
+
+  gtk_label_set_text (GTK_LABEL (widget->label), widget->layer->name);
 }
 
 
@@ -886,8 +919,7 @@ dia_layer_update_from_layer(DiaLayerWidget *widget)
  */
 
 static void
-edit_layer_ok_callback (GtkWidget *w,
-			gpointer   client_data)
+edit_layer_ok_callback (GtkWidget *w, gpointer client_data)
 {
   EditLayerDialog *dialog;
   Layer *layer;
@@ -898,10 +930,10 @@ edit_layer_ok_callback (GtkWidget *w,
   g_free (layer->name);
   layer->name = g_strdup (gtk_entry_get_text (GTK_ENTRY (dialog->name_entry)));
   
-  diagram_add_update_all(dialog->layer_widget->dia);
-  diagram_flush(dialog->layer_widget->dia);
+  diagram_add_update_all (dialog->layer_widget->dia);
+  diagram_flush (dialog->layer_widget->dia);
 
-  dia_layer_update_from_layer(dialog->layer_widget);
+  dia_layer_update_from_layer (dialog->layer_widget);
 
   dialog->layer_widget->edit_dialog = NULL;
   gtk_widget_destroy (dialog->dialog);
@@ -917,8 +949,8 @@ edit_layer_cancel_callback (GtkWidget *w,
   dialog = (EditLayerDialog *) client_data;
 
   dialog->layer_widget->edit_dialog = NULL;
-  dialog = (EditLayerDialog *) client_data;
-  gtk_widget_destroy (dialog->dialog);
+  if (dialog->dialog != NULL)
+    gtk_widget_destroy (dialog->dialog);
   g_free (dialog);
 }
 
@@ -933,7 +965,7 @@ edit_layer_delete_callback (GtkWidget *w,
 }
 
 static void
-layer_dialog_edit_layer(DiaLayerWidget *layer_widget)
+layer_dialog_edit_layer (DiaLayerWidget *layer_widget)
 {
   EditLayerDialog *dialog;
   GtkWidget *vbox;
@@ -947,15 +979,18 @@ layer_dialog_edit_layer(DiaLayerWidget *layer_widget)
 
   /*  the dialog  */
   dialog->dialog = gtk_dialog_new ();
-  gtk_window_set_wmclass (GTK_WINDOW (dialog->dialog), "edit_layer_attrributes", "Dia");
+  gtk_window_set_role (GTK_WINDOW (dialog->dialog), "edit_layer_attrributes");
   gtk_window_set_title (GTK_WINDOW (dialog->dialog), _("Edit Layer Attributes"));
   gtk_window_set_position (GTK_WINDOW (dialog->dialog), GTK_WIN_POS_MOUSE);
 
   /*  handle the wm close signal */
-  gtk_signal_connect (GTK_OBJECT (dialog->dialog), "delete_event",
-		      GTK_SIGNAL_FUNC (edit_layer_delete_callback),
+  g_signal_connect (GTK_OBJECT (dialog->dialog), "delete_event",
+		      G_CALLBACK (edit_layer_delete_callback),
 		      dialog);
-
+  g_signal_connect (GTK_OBJECT (dialog->dialog), "destroy",
+		      G_CALLBACK (gtk_widget_destroy),
+		      &dialog->dialog);
+  
   /*  the main vbox  */
   vbox = gtk_vbox_new (FALSE, 1);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 2);
@@ -973,25 +1008,25 @@ layer_dialog_edit_layer(DiaLayerWidget *layer_widget)
   gtk_widget_show (dialog->name_entry);
   gtk_widget_show (hbox);
 
-  button = gtk_button_new_with_label (_("OK"));
+  button = gtk_button_new_from_stock (GTK_STOCK_OK);
   GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog->dialog)->action_area), 
 		      button, TRUE, TRUE, 0);
-  gtk_signal_connect (GTK_OBJECT (button), "clicked",
-		      GTK_SIGNAL_FUNC(edit_layer_ok_callback),
+  g_signal_connect (GTK_OBJECT (button), "clicked",
+		      G_CALLBACK(edit_layer_ok_callback),
 		      dialog);
   gtk_widget_grab_default (button);
   gtk_widget_show (button);
 
-  button = gtk_button_new_with_label (_("Cancel"));
+  button = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
   GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog->dialog)->action_area), 
 		      button, TRUE, TRUE, 0);
-  gtk_signal_connect (GTK_OBJECT (button), "clicked",
-		      GTK_SIGNAL_FUNC(edit_layer_cancel_callback),
-		      dialog);
+  g_signal_connect (GTK_OBJECT (button), "clicked",
+		      G_CALLBACK(edit_layer_cancel_callback),
+			    dialog);
   gtk_widget_show (button);
-  
+
   gtk_widget_show (vbox);
   gtk_widget_show (dialog->dialog);
 
@@ -1001,15 +1036,6 @@ layer_dialog_edit_layer(DiaLayerWidget *layer_widget)
 
 /******** layer changes: */
 
-struct LayerChange {
-  Change change;
-
-  enum LayerChangeType type;
-  Layer *layer;
-  int index;
-  int applied;
-};
-
 static void
 layer_change_apply(struct LayerChange *change, Diagram *dia)
 {
@@ -1090,7 +1116,7 @@ undo_layer(Diagram *dia, Layer *layer, enum LayerChangeType type, int index)
 {
   struct LayerChange *change;
 
-  change = g_new(struct LayerChange, 1);
+  change = g_new0(struct LayerChange, 1);
   
   change->change.apply = (UndoApplyFunc) layer_change_apply;
   change->change.revert = (UndoRevertFunc) layer_change_revert;
@@ -1104,3 +1130,102 @@ undo_layer(Diagram *dia, Layer *layer, enum LayerChangeType type, int index)
   undo_push_change(dia->undo, (Change *) change);
   return (Change *)change;
 }
+
+static void
+layer_visibility_change_apply(struct LayerVisibilityChange *change, 
+			      Diagram *dia)
+{
+  GPtrArray *layers;
+  Layer *layer = change->layer;
+  int visible = FALSE;
+  int i;
+
+  if (change->is_exclusive) {
+    /*  First determine if _any_ other layer widgets are set to visible.
+     *  If there is, exclusive switching turns all off.  */
+    for (i=0;i<dia->data->layers->len;i++) {
+      Layer *temp_layer = g_ptr_array_index(dia->data->layers, i);
+      if (temp_layer != layer) {
+	visible |= temp_layer->visible;
+      }
+    }
+
+    /*  Now, toggle the visibility for all layers except the specified one  */
+    layers = dia->data->layers;
+    for (i = 0; i < layers->len; i++) {
+      Layer *temp_layer = (Layer *) g_ptr_array_index(layers, i);
+      if (temp_layer == layer) {
+	temp_layer->visible = TRUE;
+      } else {
+	temp_layer->visible = !visible;
+      }
+    }
+  } else {
+    layer->visible = !layer->visible;
+  }
+  diagram_add_update_all(dia);
+
+  if (layer_dialog->diagram == dia) {
+    layer_dialog_set_diagram(dia);
+  }
+}
+
+/** Revert to the visibility before this change was applied.
+ */
+static void
+layer_visibility_change_revert(struct LayerVisibilityChange *change,
+			       Diagram *dia)
+{
+  GList *vis = change->original_visibility;
+  GPtrArray *layers = dia->data->layers;
+  int i;
+
+  for (i = 0; vis != NULL && i < layers->len; vis = g_list_next(vis), i++) {
+    Layer *layer = (Layer*) g_ptr_array_index(layers, i);
+    layer->visible = (gboolean)vis->data;
+  }
+
+  if (vis != NULL || i < layers->len) {
+    printf("Internal error: visibility undo has %d visibilities, but %d layers\n",
+	   g_list_length(change->original_visibility), layers->len);
+  }
+
+  diagram_add_update_all(dia);
+
+  if (layer_dialog->diagram == dia) {
+    layer_dialog_set_diagram(dia);
+  }
+}
+
+static void
+layer_visibility_change_free(struct LayerVisibilityChange *change)
+{
+  g_list_free(change->original_visibility);
+}
+
+static struct LayerVisibilityChange *
+undo_layer_visibility(Diagram *dia, Layer *layer, gboolean exclusive)
+{
+  struct LayerVisibilityChange *change;
+  GList *visibilities = NULL;
+  int i;
+  GPtrArray *layers = dia->data->layers;
+
+  change = g_new0(struct LayerVisibilityChange, 1);
+  
+  change->change.apply = (UndoApplyFunc) layer_visibility_change_apply;
+  change->change.revert = (UndoRevertFunc) layer_visibility_change_revert;
+  change->change.free = (UndoFreeFunc) layer_visibility_change_free;
+
+  for (i = 0; i < layers->len; i++) {
+    Layer *temp_layer = (Layer *) g_ptr_array_index(layers, i);
+    visibilities = g_list_append(visibilities, (gpointer)temp_layer->visible);
+  }
+
+  change->original_visibility = visibilities;
+  change->layer = layer;
+  change->is_exclusive = exclusive;
+
+  undo_push_change(dia->undo, (Change *) change);
+  return change;
+}
