@@ -27,8 +27,13 @@
 #include <fcntl.h>
 #include <string.h>
 #include <glib.h>
+#include <glib/gstdio.h> /* g_access() and friends */
 #include <errno.h>
 
+#ifndef W_OK
+#define W_OK 2
+#endif
+
 #include "intl.h"
 
 #include <libxml/tree.h>
@@ -40,6 +45,7 @@
 #include "load_save.h"
 #include "group.h"
 #include "diagramdata.h"
+#include "object.h"
 #include "message.h"
 #include "preferences.h"
 #include "diapagelayout.h"
@@ -47,12 +53,6 @@
 
 #ifdef G_OS_WIN32
 #include <io.h>
-#define mkstemp(s) _open(_mktemp(s), O_CREAT | O_TRUNC | O_WRONLY | _O_BINARY, 0644)
-#define fchmod(f,m) (0)
-#endif
-#ifdef __EMX__
-#define mkstemp(s) _open(_mktemp(s), O_CREAT | O_TRUNC | O_WRONLY | O_BINARY, 0644)
-#define fchmod(f,m) (0)
 #endif
 
 static void read_connections(GList *objects, xmlNodePtr layer_node,
@@ -199,7 +199,7 @@ read_objects(xmlNodePtr objects,
     g_hash_table_foreach(unknown_hash,
 			 GHFuncUnknownObjects,
 			 unknown_str);
-    message_error(unknown_str->str);
+    message_error("%s", unknown_str->str);
   }
   g_hash_table_destroy(unknown_hash);
   g_string_free(unknown_str, TRUE);
@@ -345,16 +345,18 @@ diagram_data_load(const char *filename, DiagramData *data, void* user_data)
   Layer *layer;
   xmlNsPtr namespace;
   gchar firstchar;
+  Diagram *diagram = DIA_IS_DIAGRAM (data) ? DIA_DIAGRAM (data) : NULL;
 
   if (g_file_test (filename, G_FILE_TEST_IS_DIR)) {
     message_error(_("You must specify a file, not a directory.\n"));
     return FALSE;
   }
 
-  fd = open(filename, O_RDONLY);
+  fd = g_open(filename, O_RDONLY, 0);
 
   if (fd==-1) {
-    message_error(_("Couldn't open: '%s' for reading.\n"), filename);
+    message_error(_("Couldn't open: '%s' for reading.\n"),
+		  dia_message_filename(filename));
     return FALSE;
   }
 
@@ -371,19 +373,22 @@ diagram_data_load(const char *filename, DiagramData *data, void* user_data)
   doc = xmlDiaParseFile(filename);
 
   if (doc == NULL){
-    message_error(_("Error loading diagram %s.\nUnknown file type."),filename);
+    message_error(_("Error loading diagram %s.\nUnknown file type."),
+		  dia_message_filename(filename));
     return FALSE;
   }
   
   if (doc->xmlRootNode == NULL) {
-    message_error(_("Error loading diagram %s.\nUnknown file type."),filename);
+    message_error(_("Error loading diagram %s.\nUnknown file type."),
+		  dia_message_filename(filename));
     xmlFreeDoc (doc);
     return FALSE;
   }
 
   namespace = xmlSearchNs(doc, doc->xmlRootNode, "dia");
   if (strcmp (doc->xmlRootNode->name, "diagram") || (namespace == NULL)){
-    message_error(_("Error loading diagram %s.\nNot a Dia file."), filename);
+    message_error(_("Error loading diagram %s.\nNot a Dia file."), 
+		  dia_message_filename(filename));
     xmlFreeDoc (doc);
     return FALSE;
   }
@@ -401,11 +406,12 @@ diagram_data_load(const char *filename, DiagramData *data, void* user_data)
   if (attr != NULL)
     data_color(attribute_first_data(attr), &data->bg_color);
 
-  data->pagebreak_color = prefs.new_diagram.pagebreak_color;
-  attr = composite_find_attribute(diagramdata, "pagebreak");
-  if (attr != NULL)
-    data_color(attribute_first_data(attr), &data->pagebreak_color);
-
+  if (diagram) {
+    diagram->pagebreak_color = prefs.new_diagram.pagebreak_color;
+    attr = composite_find_attribute(diagramdata, "pagebreak");
+    if (attr != NULL)
+      data_color(attribute_first_data(attr), &diagram->pagebreak_color);
+  }
   /* load paper information from diagramdata section */
   attr = composite_find_attribute(diagramdata, "paper");
   if (attr != NULL) {
@@ -481,55 +487,59 @@ diagram_data_load(const char *filename, DiagramData *data, void* user_data)
     data->paper.width /= data->paper.scaling;
     data->paper.height /= data->paper.scaling;
   }
-  attr = composite_find_attribute(diagramdata, "grid");
-  if (attr != NULL) {
-    gridinfo = attribute_first_data(attr);
 
-    attr = composite_find_attribute(gridinfo, "width_x");
-    if (attr != NULL)
-      data->grid.width_x = data_real(attribute_first_data(attr));
-    attr = composite_find_attribute(gridinfo, "width_y");
-    if (attr != NULL)
-      data->grid.width_y = data_real(attribute_first_data(attr));
-    attr = composite_find_attribute(gridinfo, "visible_x");
-    if (attr != NULL)
-      data->grid.visible_x = data_int(attribute_first_data(attr));
-    attr = composite_find_attribute(gridinfo, "visible_y");
-    if (attr != NULL)
-      data->grid.visible_y = data_int(attribute_first_data(attr));
-
-    data->grid.colour = prefs.new_diagram.grid_color;
-    attr = composite_find_attribute(diagramdata, "color");
-    if (attr != NULL)
-      data_color(attribute_first_data(attr), &data->grid.colour);
-
-  }
-  attr = composite_find_attribute(diagramdata, "guides");
-  if (attr != NULL) {
-    guint i;
-    DataNode guide;
-
-    guideinfo = attribute_first_data(attr);
-
-    attr = composite_find_attribute(guideinfo, "hguides");
+  if (diagram) {
+    attr = composite_find_attribute(diagramdata, "grid");
     if (attr != NULL) {
-      data->guides.nhguides = attribute_num_data(attr);
-      g_free(data->guides.hguides);
-      data->guides.hguides = g_new(real, data->guides.nhguides);
-    
-      guide = attribute_first_data(attr);
-      for (i = 0; i < data->guides.nhguides; i++, guide = data_next(guide))
-	data->guides.hguides[i] = data_real(guide);
+      gridinfo = attribute_first_data(attr);
+
+      attr = composite_find_attribute(gridinfo, "width_x");
+      if (attr != NULL)
+        diagram->grid.width_x = data_real(attribute_first_data(attr));
+      attr = composite_find_attribute(gridinfo, "width_y");
+      if (attr != NULL)
+        diagram->grid.width_y = data_real(attribute_first_data(attr));
+      attr = composite_find_attribute(gridinfo, "visible_x");
+      if (attr != NULL)
+        diagram->grid.visible_x = data_int(attribute_first_data(attr));
+      attr = composite_find_attribute(gridinfo, "visible_y");
+      if (attr != NULL)
+        diagram->grid.visible_y = data_int(attribute_first_data(attr));
+
+      diagram->grid.colour = prefs.new_diagram.grid_color;
+      attr = composite_find_attribute(diagramdata, "color");
+      if (attr != NULL)
+        data_color(attribute_first_data(attr), &diagram->grid.colour);
     }
-    attr = composite_find_attribute(guideinfo, "vguides");
+  }
+  if (diagram) {
+    attr = composite_find_attribute(diagramdata, "guides");
     if (attr != NULL) {
-      data->guides.nvguides = attribute_num_data(attr);
-      g_free(data->guides.vguides);
-      data->guides.vguides = g_new(real, data->guides.nvguides);
+      guint i;
+      DataNode guide;
+
+      guideinfo = attribute_first_data(attr);
+
+      attr = composite_find_attribute(guideinfo, "hguides");
+      if (attr != NULL) {
+        diagram->guides.nhguides = attribute_num_data(attr);
+        g_free(diagram->guides.hguides);
+        diagram->guides.hguides = g_new(real, diagram->guides.nhguides);
     
-      guide = attribute_first_data(attr);
-      for (i = 0; i < data->guides.nvguides; i++, guide = data_next(guide))
-	data->guides.vguides[i] = data_real(guide);
+        guide = attribute_first_data(attr);
+        for (i = 0; i < diagram->guides.nhguides; i++, guide = data_next(guide))
+	  diagram->guides.hguides[i] = data_real(guide);
+      }
+      attr = composite_find_attribute(guideinfo, "vguides");
+      if (attr != NULL) {
+        diagram->guides.nvguides = attribute_num_data(attr);
+        g_free(diagram->guides.vguides);
+        diagram->guides.vguides = g_new(real, diagram->guides.nvguides);
+    
+        guide = attribute_first_data(attr);
+        for (i = 0; i < diagram->guides.nvguides; i++, guide = data_next(guide))
+	  diagram->guides.vguides[i] = data_real(guide);
+      }
     }
   }
 
@@ -585,7 +595,7 @@ diagram_data_load(const char *filename, DiagramData *data, void* user_data)
   if (data->layers->len < 1) {
     message_error (_("Error loading diagram:\n%s.\n"
                      "A valid Dia file defines at least one layer."),
-		     filename);
+		     dia_message_filename(filename));
     return FALSE;
   }
 
@@ -632,7 +642,7 @@ write_objects(GList *objects, xmlNodePtr objects_node,
       (*obj_nr)++;
       
       /*
-      if (obj->can_parent && obj->children)
+      if (object_flags_set(obj, DIA_OBJECT_CAN_PARENT) && obj->children)
       {
 	int res;
 	xmlNodePtr parent_node;
@@ -752,6 +762,7 @@ diagram_data_write_doc(DiagramData *data, const char *filename)
   Layer *layer;
   AttributeNode attr;
   xmlNs *name_space;
+  Diagram *diagram = DIA_IS_DIAGRAM (data) ? DIA_DIAGRAM (data) : NULL;
 
   doc = xmlNewDoc("1.0");
   doc->encoding = xmlStrdup("UTF-8");
@@ -767,9 +778,10 @@ diagram_data_write_doc(DiagramData *data, const char *filename)
   attr = new_attribute((ObjectNode)tree, "background");
   data_add_color(attr, &data->bg_color);
 
-  attr = new_attribute((ObjectNode)tree, "pagebreak");
-  data_add_color(attr, &data->pagebreak_color);
-
+  if (diagram) {
+    attr = new_attribute((ObjectNode)tree, "pagebreak");
+    data_add_color(attr, &diagram->pagebreak_color);
+  }
   attr = new_attribute((ObjectNode)tree, "paper");
   pageinfo = data_add_composite(attr, "paper");
   data_add_string(composite_add_attribute(pageinfo, "name"),
@@ -795,29 +807,30 @@ diagram_data_write_doc(DiagramData *data, const char *filename)
 		 data->paper.fitheight);
   }
 
-  attr = new_attribute((ObjectNode)tree, "grid");
-  gridinfo = data_add_composite(attr, "grid");
-  data_add_real(composite_add_attribute(gridinfo, "width_x"),
-		data->grid.width_x);
-  data_add_real(composite_add_attribute(gridinfo, "width_y"),
-		data->grid.width_y);
-  data_add_int(composite_add_attribute(gridinfo, "visible_x"),
-	       data->grid.visible_x);
-  data_add_int(composite_add_attribute(gridinfo, "visible_y"),
-	       data->grid.visible_y);
-  attr = new_attribute((ObjectNode)tree, "color");
-  data_add_composite(gridinfo, "color");
-  data_add_color(attr, &data->grid.colour);
+  if (diagram) {
+    attr = new_attribute((ObjectNode)tree, "grid");
+    gridinfo = data_add_composite(attr, "grid");
+    data_add_real(composite_add_attribute(gridinfo, "width_x"),
+		  diagram->grid.width_x);
+    data_add_real(composite_add_attribute(gridinfo, "width_y"),
+		  diagram->grid.width_y);
+    data_add_int(composite_add_attribute(gridinfo, "visible_x"),
+	         diagram->grid.visible_x);
+    data_add_int(composite_add_attribute(gridinfo, "visible_y"),
+	         diagram->grid.visible_y);
+    attr = new_attribute((ObjectNode)tree, "color");
+    data_add_composite(gridinfo, "color");
+    data_add_color(attr, &diagram->grid.colour);
   
-
-  attr = new_attribute((ObjectNode)tree, "guides");
-  guideinfo = data_add_composite(attr, "guides");
-  attr = composite_add_attribute(guideinfo, "hguides");
-  for (i = 0; i < data->guides.nhguides; i++)
-    data_add_real(attr, data->guides.hguides[i]);
-  attr = composite_add_attribute(guideinfo, "vguides");
-  for (i = 0; i < data->guides.nvguides; i++)
-    data_add_real(attr, data->guides.vguides[i]);
+    attr = new_attribute((ObjectNode)tree, "guides");
+    guideinfo = data_add_composite(attr, "guides");
+    attr = composite_add_attribute(guideinfo, "hguides");
+    for (i = 0; i < diagram->guides.nhguides; i++)
+      data_add_real(attr, diagram->guides.hguides[i]);
+    attr = composite_add_attribute(guideinfo, "vguides");
+    for (i = 0; i < diagram->guides.nvguides; i++)
+    data_add_real(attr, diagram->guides.vguides[i]);
+  }
 
   objects_hash = g_hash_table_new(g_direct_hash, g_direct_equal);
 
@@ -868,7 +881,12 @@ diagram_data_raw_save(DiagramData *data, const char *filename)
   return ret;
 }
 
-/** This saves the diagram, using a backup in case of failure */
+/** This saves the diagram, using a backup in case of failure.
+ * @param data
+ * @param filename
+ * @returns TRUE on successfull save, FALSE otherwise.  If a failure is
+ * indicated, an error message will already have been given to the user.
+ */
 static int
 diagram_data_save(DiagramData *data, const char *filename)
 {
@@ -877,7 +895,18 @@ diagram_data_save(DiagramData *data, const char *filename)
   int mode,_umask;
   int fildes;
   int ret;
-   
+
+  /* Once we depend on GTK 2.8+, we can use these tests. */
+#if GLIB_CHECK_VERSION(2,8,0)
+  /* Check that we're allowed to write to the target file at all. */
+  if (g_file_test(filename, G_FILE_TEST_EXISTS) &&
+      g_access(filename, W_OK) != 0) {
+    message_error(_("No allowed to write to output file %s\n"), 
+		  dia_message_filename(filename));
+    return FALSE;
+  }
+#endif
+
   /* build the temporary and backup file names */
   dirname = g_strdup(filename);
   p = strrchr((char *)dirname,G_DIR_SEPARATOR);
@@ -890,18 +919,34 @@ diagram_data_save(DiagramData *data, const char *filename)
   tmpname = g_strconcat(dirname,"__diaXXXXXX",NULL);
   bakname = g_strconcat(filename,"~",NULL);
 
+#if GLIB_CHECK_VERSION(2,8,0)
+  /* Check that we can create the other files */
+  if (g_file_test(dirname, G_FILE_TEST_EXISTS) &&
+      g_access(dirname, W_OK) != 0) {
+    message_error(_("No allowed to write temporary files in %s\n"), 
+		  dia_message_filename(dirname));
+    return FALSE;
+  }
+#endif
+
   /* open a temporary name, and fix the modes to match what fopen() would have
-     done (mkstemp() is (rightly so) a bit paranoid for what we do). */
-  fildes = mkstemp(tmpname);
+     done (mkstemp() is (rightly so) a bit paranoid for what we do).  */
+  fildes = g_mkstemp(tmpname);
+  /* should not be necessary anymore on *NIXas well, because we are using g_mkstemp ? */
   _umask = umask(0); umask(_umask);
   mode = 0666 & ~_umask;
+#ifndef G_OS_WIN32
   ret = fchmod(fildes,mode);
+#else
+  ret = 0; /* less paranoia on windoze */ 
+#endif
   file = fdopen(fildes,"wb");
 
   /* Now write the data in the temporary file name. */
 
   if (file==NULL) {
-    message_error(_("Can't open output file %s: %s\n"), tmpname, strerror(errno));
+    message_error(_("Can't open output file %s: %s\n"), 
+		  dia_message_filename(tmpname), strerror(errno));
     return FALSE;
   }
   fclose(file);
@@ -911,7 +956,9 @@ diagram_data_save(DiagramData *data, const char *filename)
   if (ret < 0) {
     /* Save failed; we clean our stuff up, without touching the file named
        "filename" if it existed. */
-    unlink(tmpname);
+    message_error(_("Internal error %d writing file %s\n"), 
+		  ret, dia_message_filename(tmpname));
+    g_unlink(tmpname);
     g_free(tmpname);
     g_free(dirname);
     g_free(bakname);
@@ -919,9 +966,14 @@ diagram_data_save(DiagramData *data, const char *filename)
   }
   /* save succeeded. We kill the old backup file, move the old file into 
      backup, and the temp file into the new saved file. */
-  unlink(bakname);
-  rename(filename,bakname);
-  ret = rename(tmpname,filename);
+  g_unlink(bakname);
+  g_rename(filename,bakname);
+  ret = g_rename(tmpname,filename);
+  if (ret < 0) {
+    message_error(_("Can't rename %s to final output file %s: %s\n"), 
+		  dia_message_filename(filename), 
+		  dia_message_filename(filename), strerror(errno));
+  }
   g_free(tmpname);
   g_free(dirname);
   g_free(bakname);
@@ -934,7 +986,6 @@ diagram_save(Diagram *dia, const char *filename)
   gboolean res = diagram_data_save(dia->data, filename);
 
   if (!res) {
-    message_error(_("Failed to save file '%s'.\n"), filename);    
     return res;
   }
 
@@ -956,10 +1007,12 @@ diagram_cleanup_autosave(Diagram *dia)
 
   savefile = dia->autosavefilename;
   if (savefile == NULL) return;
-  printf("Cleaning up autosave %s for %s\n", savefile, dia->filename);
-
-  if (stat(savefile, &statbuf) == 0) { /* Success */
-    unlink(savefile);
+#ifdef TRACES
+  g_print("Cleaning up autosave %s for %s\n", 
+          savefile, dia->filename ? dia->filename : "<no name>");
+#endif
+  if (g_stat(savefile, &statbuf) == 0) { /* Success */
+    g_unlink(savefile);
   }
   g_free(savefile);
   dia->autosavefilename = NULL;
