@@ -34,7 +34,7 @@
 static GtkWidget *dialog = NULL;
 static GtkWidget *dialog_vbox = NULL;
 static GtkWidget *object_part = NULL;
-static Object *current_obj = NULL;
+static DiaObject *current_obj = NULL;
 static Diagram *current_dia = NULL;
 
 static GtkWidget *no_properties_dialog = NULL;
@@ -60,7 +60,6 @@ static void create_dialog()
              GTK_STOCK_OK, GTK_RESPONSE_OK,
              NULL);
 
-  //GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
   gtk_dialog_set_default_response (GTK_DIALOG(dialog), GTK_RESPONSE_OK);
 
   dialog_vbox = GTK_DIALOG(dialog)->vbox;
@@ -165,8 +164,23 @@ properties_respond(GtkWidget *widget,
   return 0;
 }
 
+/** Give focus to the first focusable widget found in `widget'.
+ * @param widget Some (possibly composite) widget.
+ */
+static void
+properties_give_focus(GtkWidget *widget, gpointer data)
+{
+  if (GTK_WIDGET_CAN_FOCUS(widget)) {
+    gtk_widget_grab_focus(widget);
+  } else {
+    if (GTK_IS_CONTAINER(widget)) {
+      gtk_container_foreach(GTK_CONTAINER(widget), properties_give_focus, data);
+    }
+  }
+}
+
 void
-properties_show(Diagram *dia, Object *obj)
+properties_show(Diagram *dia, DiaObject *obj)
 {
   GtkWidget *properties = NULL;
 
@@ -196,7 +210,7 @@ properties_show(Diagram *dia, Object *obj)
   }
 
   if (obj != NULL) {
-    ObjectType *otype;
+    DiaObjectType *otype;
     gchar *buf;
 
     otype = obj->type;
@@ -216,6 +230,8 @@ properties_show(Diagram *dia, Object *obj)
 
   gtk_widget_show (properties);
 
+  properties_give_focus(properties, NULL);
+
   if (obj != current_obj)
     gtk_window_resize (GTK_WINDOW(dialog), 1, 1); /* resize to minimum */
   gtk_window_set_transient_for(GTK_WINDOW(dialog),
@@ -227,7 +243,7 @@ properties_show(Diagram *dia, Object *obj)
 }
 
 void
-properties_hide_if_shown(Diagram *dia, Object *obj)
+properties_hide_if_shown(Diagram *dia, DiaObject *obj)
 {
   if (current_obj == obj) {
     properties_show(dia, NULL);
@@ -235,7 +251,7 @@ properties_hide_if_shown(Diagram *dia, Object *obj)
 }
 
 void
-properties_update_if_shown(Diagram *dia, Object *obj)
+properties_update_if_shown(Diagram *dia, DiaObject *obj)
 {
   if (current_obj == obj) {
     properties_show(dia, obj);
