@@ -23,12 +23,12 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <assert.h>
 #include <math.h>
 #include <string.h> /* memcpy() */
 
 #include "beziershape.h"
 #include "message.h"
+#include "diarenderer.h"
 
 #define HANDLE_BEZMAJOR  (HANDLE_CUSTOM1)
 #define HANDLE_LEFTCTRL  (HANDLE_CUSTOM2)
@@ -39,6 +39,11 @@ enum change_type {
   TYPE_REMOVE_POINT
 };
 
+/* Invariant:
+   # of handles = 3*(numpoints-1)
+   # of connections = 2*(numpoints-1) + 1 (main point)
+   For historical reasons, the main point is the last cp.
+ */
 struct PointChange {
   ObjectChange obj_change;
 
@@ -78,6 +83,8 @@ beziershape_create_corner_change(BezierShape *bezier, Handle *handle,
 			BezCornerType old_corner_type,
 			BezCornerType new_corner_type);
 
+static void new_handles_and_connections(BezierShape *bezier, int num_points);
+
 static void setup_handle(Handle *handle, int handle_id)
 {
   handle->id = handle_id;
@@ -103,9 +110,10 @@ static int get_handle_nr(BezierShape *bezier, Handle *handle)
 #define get_comp_nr(hnum) ((int)(hnum)/3+1)
 #define get_major_nr(hnum) (((int)(hnum)+2)/3)
 
-void
+ObjectChange *
 beziershape_move_handle(BezierShape *bezier, Handle *handle,
-			Point *to, HandleMoveReason reason)
+			Point *to, ConnectionPoint *cp,
+			HandleMoveReason reason, ModifierKeys modifiers)
 {
   int handle_nr, comp_nr, next_nr, prev_nr;
   Point delta, pt;
@@ -205,9 +213,10 @@ beziershape_move_handle(BezierShape *bezier, Handle *handle,
     message_error("Internal error in beziershape_move_handle.");
     break;
   }
+  return NULL;
 }
 
-void
+ObjectChange*
 beziershape_move(BezierShape *bezier, Point *to)
 {
   Point p;
@@ -222,6 +231,8 @@ beziershape_move(BezierShape *bezier, Point *to)
     point_add(&bezier->points[i].p2, &p);
     point_add(&bezier->points[i].p3, &p);
   }
+
+  return NULL;
 }
 
 int
@@ -305,12 +316,12 @@ add_handles(BezierShape *bezier, int pos, BezPoint *point,
 	    ConnectionPoint *cp1, ConnectionPoint *cp2)
 {
   int i, next;
-  Object *obj;
+  DiaObject *obj;
 
   g_assert(pos >= 1);
   g_assert(pos <= bezier->numpoints);
 
-  obj = (Object *)bezier;
+  obj = (DiaObject *)bezier;
   bezier->numpoints++;
   next = pos + 1;
   if (pos == bezier->numpoints - 1)
@@ -330,35 +341,47 @@ add_handles(BezierShape *bezier, int pos, BezPoint *point,
   if (pos == bezier->numpoints - 1)
     bezier->points[0].p1 = bezier->points[0].p3 = bezier->points[pos].p3;
   bezier->corner_types[pos] = corner_type;
-  object_add_handle_at((Object*)bezier, handle1, 3*pos-3);
-  object_add_handle_at((Object*)bezier, handle2, 3*pos-2);
-  object_add_handle_at((Object*)bezier, handle3, 3*pos-1);
-  object_add_connectionpoint_at((Object *)bezier, cp1, 2*pos-1);
-  object_add_connectionpoint_at((Object *)bezier, cp2, 2*pos);
+  object_add_handle_at((DiaObject*)bezier, handle1, 3*pos-3);
+  object_add_handle_at((DiaObject*)bezier, handle2, 3*pos-2);
+  object_add_handle_at((DiaObject*)bezier, handle3, 3*pos-1);
+  object_add_connectionpoint_at((DiaObject *)bezier, cp1, 2*pos-2);
+  object_add_connectionpoint_at((DiaObject *)bezier, cp2, 2*pos-1);
 }
 
 static void
 remove_handles(BezierShape *bezier, int pos)
 {
   int i;
-  Object *obj;
+  DiaObject *obj;
   Handle *old_handle1, *old_handle2, *old_handle3;
   ConnectionPoint *old_cp1, *old_cp2;
   Point tmppoint;
+  Point controlvector;
 
   g_assert(pos > 0);
   g_assert(pos < bezier->numpoints);
 
-  obj = (Object *)bezier;
+  obj = (DiaObject *)bezier;
 
   /* delete the points */
   bezier->numpoints--;
   tmppoint = bezier->points[pos].p1;
+  if (pos == bezier->numpoints) {
+    controlvector = bezier->points[pos-1].p3;
+    point_sub(&controlvector, &bezier->points[pos].p1);
+  }
   for (i = pos; i < bezier->numpoints; i++) {
     bezier->points[i] = bezier->points[i+1];
     bezier->corner_types[i] = bezier->corner_types[i+1];
   }
   bezier->points[pos].p1 = tmppoint;
+  if (pos == bezier->numpoints) {
+    /* If this was the last point, we also need to move points[0] and
+       the control point in points[1]. */
+    bezier->points[0].p1 = bezier->points[bezier->numpoints-1].p3;
+    bezier->points[1].p1 = bezier->points[0].p1;
+    point_sub(&bezier->points[1].p1, &controlvector);
+  }
   bezier->points = g_realloc(bezier->points,
 			     bezier->numpoints * sizeof(BezPoint));
   bezier->corner_types = g_realloc(bezier->corner_types,
@@ -370,8 +393,8 @@ remove_handles(BezierShape *bezier, int pos)
   object_remove_handle(&bezier->object, old_handle1);
   object_remove_handle(&bezier->object, old_handle2);
   object_remove_handle(&bezier->object, old_handle3);
-  old_cp1 = obj->connections[2*pos-1];
-  old_cp2 = obj->connections[2*pos];
+  old_cp1 = obj->connections[2*pos-2];
+  old_cp2 = obj->connections[2*pos-1];
   object_remove_connectionpoint(&bezier->object, old_cp1);
   object_remove_connectionpoint(&bezier->object, old_cp2);
 }
@@ -387,22 +410,28 @@ beziershape_add_segment(BezierShape *bezier, int segment, Point *point)
   Handle *new_handle1, *new_handle2, *new_handle3;
   ConnectionPoint *new_cp1, *new_cp2;
   Point startpoint;
+  Point other;
 
-  startpoint = bezier->points[segment].p3;
+  if (segment != 1)
+    startpoint = bezier->points[segment-1].p3;
+  else 
+    startpoint = bezier->points[0].p1;
+  other = bezier->points[segment].p3;
   if (point == NULL) {
-    realpoint.p1.x = (startpoint.x + bezier->points[segment+1].p3.x)/6;
-    realpoint.p1.y = (startpoint.y + bezier->points[segment+1].p3.y)/6;
-    realpoint.p2.x = (startpoint.x + bezier->points[segment+1].p3.x)/3;
-    realpoint.p2.y = (startpoint.y + bezier->points[segment+1].p3.y)/3;
-    realpoint.p3.x = (startpoint.x + bezier->points[segment+1].p3.x)/2;
-    realpoint.p3.y = (startpoint.y + bezier->points[segment+1].p3.y)/2;
+    realpoint.p1.x = (startpoint.x + other.x)/6;
+    realpoint.p1.y = (startpoint.y + other.y)/6;
+    realpoint.p2.x = (startpoint.x + other.x)/3;
+    realpoint.p2.y = (startpoint.y + other.y)/3;
+    realpoint.p3.x = (startpoint.x + other.x)/2;
+    realpoint.p3.y = (startpoint.y + other.y)/2;
   } else {
-    realpoint.p2.x = point->x+(startpoint.x-bezier->points[segment+1].p3.x)/6;
-    realpoint.p2.y = point->y+(startpoint.y-bezier->points[segment+1].p3.y)/6;
+    realpoint.p2.x = point->x+(startpoint.x-other.x)/6;
+    realpoint.p2.y = point->y+(startpoint.y-other.y)/6;
+
     realpoint.p3 = *point;
     /* this really goes into the next segment ... */
-    realpoint.p1.x = point->x-(startpoint.x-bezier->points[segment+1].p3.x)/6;
-    realpoint.p1.y = point->y-(startpoint.y-bezier->points[segment+1].p3.y)/6;
+    realpoint.p1.x = point->x-(startpoint.x-other.x)/6;
+    realpoint.p1.y = point->y-(startpoint.y-other.y)/6;
   }
   realpoint.type = BEZ_CURVE_TO;
 
@@ -434,6 +463,7 @@ beziershape_remove_segment(BezierShape *bezier, int pos)
 
   g_assert(pos > 0);
   g_assert(bezier->numpoints > 2);
+  g_assert(pos < bezier->numpoints);
 
   old_handle1 = bezier->object.handles[3*pos-3];
   old_handle2 = bezier->object.handles[3*pos-2];
@@ -441,12 +471,12 @@ beziershape_remove_segment(BezierShape *bezier, int pos)
   old_point = bezier->points[pos];
   old_ctype = bezier->corner_types[pos];
 
-  old_cp1 = bezier->object.connections[2*pos-1];
-  old_cp2 = bezier->object.connections[2*pos];
+  old_cp1 = bezier->object.connections[2*pos-2];
+  old_cp2 = bezier->object.connections[2*pos-1];
   
-  object_unconnect((Object *)bezier, old_handle1);
-  object_unconnect((Object *)bezier, old_handle2);
-  object_unconnect((Object *)bezier, old_handle3);
+  object_unconnect((DiaObject *)bezier, old_handle1);
+  object_unconnect((DiaObject *)bezier, old_handle2);
+  object_unconnect((DiaObject *)bezier, old_handle3);
 
   remove_handles(bezier, pos);
 
@@ -583,228 +613,239 @@ beziershape_update_data(BezierShape *bezier)
 {
   int i;
   Point last;
+  Point minp, maxp;
   
+  DiaObject *obj = &bezier->object;
+  
+  /* handle the case of whole points array update (via set_prop) */
+  if (3*(bezier->numpoints-1) != obj->num_handles ||
+      2*(bezier->numpoints-1) + 1 != obj->num_connections) {
+    object_unconnect_all(obj); /* too drastic ? */
+
+    /* delete the old ones */
+    for (i = 0; i < obj->num_handles; i++)
+      g_free(obj->handles[i]);
+    g_free(obj->handles);
+    for (i = 0; i < obj->num_connections; i++);
+      g_free(obj->connections[i]);
+    g_free(obj->connections);
+
+    obj->num_handles = 3*(bezier->numpoints-1);
+    obj->handles = g_new(Handle*, obj->num_handles);
+    obj->num_connections = 2*(bezier->numpoints-1) + 1;
+    obj->connections = g_new(ConnectionPoint *, obj->num_connections);
+
+    new_handles_and_connections(bezier, bezier->numpoints);
+
+    bezier->corner_types = g_realloc(bezier->corner_types, bezier->numpoints*sizeof(BezCornerType));
+    for (i = 0; i < bezier->numpoints; i++)
+      bezier->corner_types[i] = BEZ_CORNER_SYMMETRIC;
+  }
+
   /* Update handles: */
   bezier->points[0].p3 = bezier->points[0].p1;
   for (i = 1; i < bezier->numpoints; i++) {
-    bezier->object.handles[3*i-3]->pos = bezier->points[i].p1;
-    bezier->object.handles[3*i-2]->pos = bezier->points[i].p2;
-    bezier->object.handles[3*i-1]->pos = bezier->points[i].p3;
+    obj->handles[3*i-3]->pos = bezier->points[i].p1;
+    obj->handles[3*i-2]->pos = bezier->points[i].p2;
+    obj->handles[3*i-1]->pos = bezier->points[i].p3;
   }
 
   /* Update connection points: */
   last = bezier->points[0].p1;
   for (i = 1; i < bezier->numpoints; i++) {
-    bezier->object.connections[2*i-2]->pos = last;
-    bezier->object.connections[2*i-1]->pos.x =
+    Point slopepoint1, slopepoint2;
+    slopepoint1 = bezier->points[i].p1;
+    point_sub(&slopepoint1, &last);
+    point_scale(&slopepoint1, .5);
+    point_add(&slopepoint1, &last);
+    slopepoint2 = bezier->points[i].p2;
+    point_sub(&slopepoint2, &bezier->points[i].p3);
+    point_scale(&slopepoint2, .5);
+    point_add(&slopepoint2, &bezier->points[i].p3);
+
+    obj->connections[2*i-2]->pos = last;
+    obj->connections[2*i-2]->directions =
+      find_slope_directions(last, bezier->points[i].p1);
+    obj->connections[2*i-1]->pos.x =
       (last.x + 3*bezier->points[i].p1.x + 3*bezier->points[i].p2.x +
        bezier->points[i].p3.x)/8;
-    bezier->object.connections[2*i-1]->pos.y =
+    obj->connections[2*i-1]->pos.y =
       (last.y + 3*bezier->points[i].p1.y + 3*bezier->points[i].p2.y +
        bezier->points[i].p3.y)/8;
+    obj->connections[2*i-1]->directions = 
+      find_slope_directions(slopepoint1, slopepoint2);
     last = bezier->points[i].p3;
   }
+  
+  /* Find the middle of the object (or some approximation at least) */
+  minp = maxp = bezier->points[0].p1;
+  for (i = 1; i < bezier->numpoints; i++) {
+    Point p = bezier->points[i].p3;
+    if (p.x < minp.x) minp.x = p.x;
+    if (p.x > maxp.x) maxp.x = p.x;
+    if (p.y < minp.y) minp.y = p.y;
+    if (p.y > maxp.y) maxp.y = p.y;
+  }
+  obj->connections[obj->num_connections-1]->pos.x = (minp.x + maxp.x) / 2;
+  obj->connections[obj->num_connections-1]->pos.y = (minp.y + maxp.y) / 2;
+  obj->connections[obj->num_connections-1]->directions = DIR_ALL;
 }
 
 void
 beziershape_update_boundingbox(BezierShape *bezier)
 {
-  Rectangle *bb;
-  BezPoint *points;
-  Point pt1,pt0;
-  BezierShapeBBExtras *extra;
-  int i;
-  
-  assert(bezier != NULL);
+  ElementBBExtras *extra;
+  PolyBBExtras pextra;
+
+  g_assert(bezier != NULL);
 
   extra = &bezier->extra_spacing;
-  bb = &bezier->object.bounding_box;
-  points = &bezier->points[0];
+  pextra.start_trans = pextra.end_trans = 
+    pextra.start_long = pextra.end_long = 0;
+  pextra.middle_trans = extra->border_trans;
 
-  bb->right = bb->left = points[0].p1.x;
-  bb->top = bb->bottom = points[0].p1.y;
-
-
-  for (i=0;i<=(bezier->numpoints);i++) {
-    int imod = i % bezier->numpoints;
-    int ipomod = (i+1) % bezier->numpoints;
-    if (points[imod].type == BEZ_CURVE_TO) {
-      pt1 = points[imod].p3; 
-    } else {
-      pt1 = points[imod].p1;
-    }
-    if (points[ipomod].type == BEZ_CURVE_TO) {
-      point_sub(&pt1,&(points[ipomod].p3));
-    } else {
-      point_sub(&pt1,&(points[ipomod].p1));
-    }
-    point_normalize(&pt1);
-
-    if (i!=0) {
-      real co;
-
-      if (points[i].type == BEZ_CURVE_TO) {
-        /* FIXME : dig up the math, do something closer to the curve (CC) */        
-        check_bb_x(bb,points[i].p1.x + extra->border_trans * 1.41,1);
-        check_bb_x(bb,points[i].p1.x - extra->border_trans * 1.41,1);
-        check_bb_x(bb,points[i].p2.x + extra->border_trans * 1.41,1);
-        check_bb_x(bb,points[i].p2.x - extra->border_trans * 1.41,1);
-        check_bb_x(bb,points[i].p3.x + extra->border_trans * 1.41,1);
-        check_bb_x(bb,points[i].p3.x - extra->border_trans * 1.41,1);
-
-        check_bb_y(bb,points[i].p1.y + extra->border_trans * 1.41,1);
-        check_bb_y(bb,points[i].p1.y - extra->border_trans * 1.41,1);
-        check_bb_y(bb,points[i].p2.y + extra->border_trans * 1.41,1);
-        check_bb_y(bb,points[i].p2.y - extra->border_trans * 1.41,1);
-        check_bb_y(bb,points[i].p3.y + extra->border_trans * 1.41,1);
-        check_bb_y(bb,points[i].p3.y - extra->border_trans * 1.41,1);
-      } else {
-        check_bb_x(bb, points[imod].p1.x + (extra->border_trans * pt1.y),pt1.y);
-        check_bb_x(bb, points[imod].p1.x - (extra->border_trans * pt1.y),pt1.y);
-        check_bb_y(bb, points[imod].p1.y + (extra->border_trans * pt1.x),pt1.x);
-        check_bb_y(bb, points[imod].p1.y - (extra->border_trans * pt1.x),pt1.x);
-      
-        co = point_dot(&pt1,&pt0);
-        if (co > -0.9816) { /* 0.9816 == cos(11deg) */
-          real alpha = fabs(acos(-co));
-          real overshoot = extra->border_trans / (sin(alpha/2.0));
-          Point pts;
-          
-          pts = pt1; point_sub(&pts,&pt0);
-          point_normalize(&pts);
-
-          check_bb_x(bb,points[imod].p1.x + (overshoot * pts.x),pts.x);
-          check_bb_y(bb,points[imod].p1.y + (overshoot * pts.y),pts.y);
-        } else { 
-          check_bb_x(bb, points[imod].p1.x + (extra->border_trans * pt0.y),pt0.y);
-          check_bb_x(bb, points[imod].p1.x - (extra->border_trans * pt0.y),pt0.y);
-          check_bb_y(bb, points[imod].p1.y + (extra->border_trans * pt0.x),pt0.x);
-          check_bb_y(bb, points[imod].p1.y - (extra->border_trans * pt0.x),pt0.x);
-        } 
-      }
-    }
-    pt0=pt1;
-  }
+  polybezier_bbox(&bezier->points[0],
+                  bezier->numpoints,
+                  &pextra, TRUE,
+                  &bezier->object.bounding_box);
 }
 
 void
-beziershape_simple_draw(BezierShape *bezier, Renderer *renderer, real width)
+beziershape_simple_draw(BezierShape *bezier, DiaRenderer *renderer, real width)
 {
   BezPoint *points;
   
-  assert(bezier != NULL);
-  assert(renderer != NULL);
+  g_assert(bezier != NULL);
+  g_assert(renderer != NULL);
 
   points = &bezier->points[0];
   
-  renderer->ops->set_linewidth(renderer, width);
-  renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
-  renderer->ops->set_linejoin(renderer, LINEJOIN_ROUND);
-  renderer->ops->set_linecaps(renderer, LINECAPS_BUTT);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linewidth(renderer, width);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linejoin(renderer, LINEJOIN_ROUND);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linecaps(renderer, LINECAPS_BUTT);
 
-  renderer->ops->fill_bezier(renderer, points, bezier->numpoints,&color_white);
-  renderer->ops->draw_bezier(renderer, points, bezier->numpoints,&color_black);
+  DIA_RENDERER_GET_CLASS(renderer)->fill_bezier(renderer, points, bezier->numpoints,&color_white);
+  DIA_RENDERER_GET_CLASS(renderer)->draw_bezier(renderer, points, bezier->numpoints,&color_black);
 }
 
 void
-beziershape_draw_control_lines(BezierShape *bez, Renderer *renderer)
+beziershape_draw_control_lines(BezierShape *bez, DiaRenderer *renderer)
 {
   Color line_colour = {0.0, 0.0, 0.6};
   Point startpoint;
   int i;
   
   /* setup renderer ... */
-  renderer->ops->set_linewidth(renderer, 0);
-  renderer->ops->set_linestyle(renderer, LINESTYLE_DOTTED);
-  renderer->ops->set_dashlength(renderer, 1);
-  renderer->ops->set_linejoin(renderer, LINEJOIN_MITER);
-  renderer->ops->set_linecaps(renderer, LINECAPS_BUTT);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linewidth(renderer, 0);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_DOTTED);
+  DIA_RENDERER_GET_CLASS(renderer)->set_dashlength(renderer, 1);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linejoin(renderer, LINEJOIN_MITER);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linecaps(renderer, LINECAPS_BUTT);
 
   startpoint = bez->points[0].p1;
   for (i = 1; i < bez->numpoints; i++) {
-    renderer->ops->draw_line(renderer, &startpoint, &bez->points[i].p1,
+    DIA_RENDERER_GET_CLASS(renderer)->draw_line(renderer, &startpoint, &bez->points[i].p1,
                              &line_colour);
-    renderer->ops->draw_line(renderer, &bez->points[i].p2, &bez->points[i].p3,
+    DIA_RENDERER_GET_CLASS(renderer)->draw_line(renderer, &bez->points[i].p2, &bez->points[i].p3,
                              &line_colour);
     startpoint = bez->points[i].p3;
   }
 }
 
-void
-beziershape_init(BezierShape *bezier)
+static void
+new_handles_and_connections(BezierShape *bezier, int num_points)
 {
-  Object *obj;
+  DiaObject *obj;
+  int i;
 
   obj = &bezier->object;
 
-  object_init(obj, 6, 4);
+  for (i = 0; i < num_points-1; i++) {
+    obj->handles[3*i] = g_new(Handle, 1);
+    obj->handles[3*i+1] = g_new(Handle, 1);
+    obj->handles[3*i+2] = g_new(Handle, 1);
   
-  bezier->numpoints = 3;
+    obj->handles[3*i]->connect_type = HANDLE_NONCONNECTABLE;
+    obj->handles[3*i]->connected_to = NULL;
+    obj->handles[3*i]->type = HANDLE_MINOR_CONTROL;
+    obj->handles[3*i]->id = HANDLE_RIGHTCTRL;
 
-  bezier->points = g_new(BezPoint, 3);
+    obj->handles[3*i+1]->connect_type = HANDLE_NONCONNECTABLE;
+    obj->handles[3*i+1]->connected_to = NULL;
+    obj->handles[3*i+1]->type = HANDLE_MINOR_CONTROL;
+    obj->handles[3*i+1]->id = HANDLE_LEFTCTRL;
+  
+    obj->handles[3*i+2]->connect_type = HANDLE_NONCONNECTABLE;
+    obj->handles[3*i+2]->connected_to = NULL;
+    obj->handles[3*i+2]->type = HANDLE_MAJOR_CONTROL;
+    obj->handles[3*i+2]->id = HANDLE_BEZMAJOR;
+
+    obj->connections[2*i] = g_new0(ConnectionPoint, 1);
+    obj->connections[2*i+1] = g_new0(ConnectionPoint, 1);
+    obj->connections[2*i]->object = obj;
+    obj->connections[2*i+1]->object = obj;
+    obj->connections[2*i]->flags = 0;
+    obj->connections[2*i+1]->flags = 0;
+  }
+
+  /** Main point */
+  obj->connections[obj->num_connections-1] = g_new0(ConnectionPoint, 1);
+  obj->connections[obj->num_connections-1]->object = obj;
+  obj->connections[obj->num_connections-1]->flags = CP_FLAGS_MAIN;
+}
+
+void
+beziershape_init(BezierShape *bezier, int num_points)
+{
+  DiaObject *obj;
+  int i;
+
+  obj = &bezier->object;
+
+  object_init(obj, 3*(num_points-1), 2*(num_points-1) + 1);
+  
+  bezier->numpoints = num_points;
+
+  bezier->points = g_new(BezPoint, num_points);
   bezier->points[0].type = BEZ_MOVE_TO;
-  bezier->points[1].type = BEZ_CURVE_TO;
-  bezier->points[2].type = BEZ_CURVE_TO;
+  bezier->corner_types = g_new(BezCornerType, num_points);
+  for (i = 1; i < num_points; i++) {
+    bezier->points[i].type = BEZ_CURVE_TO;
+    bezier->corner_types[i] = BEZ_CORNER_SYMMETRIC;
+  }
 
-  bezier->corner_types = g_new(BezCornerType, 3);
-  bezier->corner_types[0] = BEZ_CORNER_SYMMETRIC;
-  bezier->corner_types[1] = BEZ_CORNER_SYMMETRIC;
-  bezier->corner_types[2] = BEZ_CORNER_SYMMETRIC;
+  new_handles_and_connections(bezier, num_points);
 
-  bezier->object.handles[0] = g_new(Handle, 1);
-  bezier->object.handles[1] = g_new(Handle, 1);
-  bezier->object.handles[2] = g_new(Handle, 1);
-  bezier->object.handles[3] = g_new(Handle, 1);
-  bezier->object.handles[4] = g_new(Handle, 1);
-  bezier->object.handles[5] = g_new(Handle, 1);
+  /* The points are not assigned at this point, so don't try to use
+     them */
+  /*  beziershape_update_data(bezier);*/
+}
 
-  obj->handles[0]->connect_type = HANDLE_NONCONNECTABLE;
-  obj->handles[0]->connected_to = NULL;
-  obj->handles[0]->type = HANDLE_MINOR_CONTROL;
-  obj->handles[0]->id = HANDLE_RIGHTCTRL;
-  
-  obj->handles[1]->connect_type = HANDLE_NONCONNECTABLE;
-  obj->handles[1]->connected_to = NULL;
-  obj->handles[1]->type = HANDLE_MINOR_CONTROL;
-  obj->handles[1]->id = HANDLE_LEFTCTRL;
 
-  obj->handles[2]->connect_type = HANDLE_NONCONNECTABLE;
-  obj->handles[2]->connected_to = NULL;
-  obj->handles[2]->type = HANDLE_MAJOR_CONTROL;
-  obj->handles[2]->id = HANDLE_BEZMAJOR;
+/** This function does *not* set up handles */
+void
+beziershape_set_points(BezierShape *bez, int num_points, BezPoint *points)
+{
+  int i;
 
-  obj->handles[3]->connect_type = HANDLE_NONCONNECTABLE;
-  obj->handles[3]->connected_to = NULL;
-  obj->handles[3]->type = HANDLE_MINOR_CONTROL;
-  obj->handles[3]->id = HANDLE_RIGHTCTRL;
-  
-  obj->handles[4]->connect_type = HANDLE_NONCONNECTABLE;
-  obj->handles[4]->connected_to = NULL;
-  obj->handles[4]->type = HANDLE_MINOR_CONTROL;
-  obj->handles[4]->id = HANDLE_LEFTCTRL;
+  bez->numpoints = num_points;
 
-  obj->handles[5]->connect_type = HANDLE_NONCONNECTABLE;
-  obj->handles[5]->connected_to = NULL;
-  obj->handles[5]->type = HANDLE_MAJOR_CONTROL;
-  obj->handles[5]->id = HANDLE_BEZMAJOR;
+  if (bez->points)
+    g_free(bez->points);
 
-  obj->connections[0] = g_new0(ConnectionPoint, 1);
-  obj->connections[1] = g_new0(ConnectionPoint, 1);
-  obj->connections[2] = g_new0(ConnectionPoint, 1);
-  obj->connections[3] = g_new0(ConnectionPoint, 1);
-  obj->connections[0]->object = obj;
-  obj->connections[1]->object = obj;
-  obj->connections[2]->object = obj;
-  obj->connections[3]->object = obj;
+  bez->points = g_malloc((bez->numpoints)*sizeof(BezPoint));
 
-  beziershape_update_data(bezier);
+  for (i=0;i<bez->numpoints;i++) {
+    bez->points[i] = points[i];
+  }
 }
 
 void
 beziershape_copy(BezierShape *from, BezierShape *to)
 {
   int i;
-  Object *toobj, *fromobj;
+  DiaObject *toobj, *fromobj;
 
   toobj = &to->object;
   fromobj = &from->object;
@@ -821,13 +862,14 @@ beziershape_copy(BezierShape *from, BezierShape *to)
     to->corner_types[i] = from->corner_types[i];
   }
 
-  for (i = 0; i < to->object.num_handles; i++) {
-    to->object.handles[i] = g_new(Handle, 1);
-    setup_handle(to->object.handles[i], from->object.handles[i]->id);
+  for (i = 0; i < toobj->num_handles; i++) {
+    toobj->handles[i] = g_new(Handle, 1);
+    setup_handle(toobj->handles[i], fromobj->handles[i]->id);
   }
-  for (i = 0; i < to->object.num_connections; i++) {
-    to->object.connections[i] = g_new0(ConnectionPoint, 1);
-    to->object.connections[i]->object = &to->object;
+  for (i = 0; i < toobj->num_connections; i++) {
+    toobj->connections[i] = g_new0(ConnectionPoint, 1);
+    toobj->connections[i]->object = &to->object;
+    toobj->connections[i]->flags = fromobj->connections[i]->flags;
   }
 
   memcpy(&to->extra_spacing,&from->extra_spacing,sizeof(to->extra_spacing));  
@@ -896,7 +938,7 @@ beziershape_load(BezierShape *bezier, ObjectNode obj_node)
   AttributeNode attr;
   DataNode data;
   
-  Object *obj = &bezier->object;
+  DiaObject *obj = &bezier->object;
 
   object_load(obj, obj_node);
 
@@ -907,7 +949,8 @@ beziershape_load(BezierShape *bezier, ObjectNode obj_node)
   else
     bezier->numpoints = 0;
 
-  object_init(obj, 3 * (bezier->numpoints - 1), 2 * (bezier->numpoints - 1));
+  object_init(obj, 3 * (bezier->numpoints - 1),
+	      2 * (bezier->numpoints - 1) + 1);
 
   data = attribute_first_data(attr);
   if (bezier->numpoints != 0) {
@@ -953,10 +996,11 @@ beziershape_load(BezierShape *bezier, ObjectNode obj_node)
     setup_handle(obj->handles[3*i+1], HANDLE_LEFTCTRL);
     setup_handle(obj->handles[3*i+2],   HANDLE_BEZMAJOR);
   }
-  for (i = 0; i < bezier->object.num_connections; i++) {
+  for (i = 0; i < obj->num_connections; i++) {
     obj->connections[i] = g_new0(ConnectionPoint, 1);
     obj->connections[i]->object = obj;
   }
+  obj->connections[obj->num_connections-1]->flags = CP_FLAGS_MAIN;
 
   beziershape_update_data(bezier);
 }
@@ -980,7 +1024,7 @@ beziershape_point_change_free(struct PointChange *change)
 }
 
 static void
-beziershape_point_change_apply(struct PointChange *change, Object *obj)
+beziershape_point_change_apply(struct PointChange *change, DiaObject *obj)
 {
   change->applied = 1;
   switch (change->type) {
@@ -1000,7 +1044,7 @@ beziershape_point_change_apply(struct PointChange *change, Object *obj)
 }
 
 static void
-beziershape_point_change_revert(struct PointChange *change, Object *obj)
+beziershape_point_change_revert(struct PointChange *change, DiaObject *obj)
 {
   switch (change->type) {
   case TYPE_ADD_POINT:
@@ -1050,7 +1094,7 @@ beziershape_create_point_change(BezierShape *bezier, enum change_type type,
 }
 
 static void
-beziershape_corner_change_apply(struct CornerChange *change, Object *obj)
+beziershape_corner_change_apply(struct CornerChange *change, DiaObject *obj)
 {
   BezierShape *bez = (BezierShape *)obj;
   int handle_nr = get_handle_nr(bez, change->handle);
@@ -1068,7 +1112,7 @@ beziershape_corner_change_apply(struct CornerChange *change, Object *obj)
 }
 
 static void
-beziershape_corner_change_revert(struct CornerChange *change, Object *obj)
+beziershape_corner_change_revert(struct CornerChange *change, DiaObject *obj)
 {
   BezierShape *bez = (BezierShape *)obj;
   int handle_nr = get_handle_nr(bez, change->handle);
