@@ -24,7 +24,7 @@
 #include <gtk/gtkwidget.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
-#include "pixmaps/broken.xpm"
+#include "dia-lib-icons.h"
 
 #define SCALING_CACHE
 
@@ -39,6 +39,9 @@ struct _DiaImage {
 
 gboolean _dia_image_initialized = FALSE;
 
+/** Perform required initialization to handle images with GDK.
+ *  Should not be called in non-interactive use.
+ */
 void 
 dia_image_init(void)
 {
@@ -48,6 +51,9 @@ dia_image_init(void)
   }
 }
 
+/** Get the image to put in place of a image that cannot be read.
+ * @returns A statically allocated image.
+ */
 DiaImage
 dia_image_get_broken(void)
 {
@@ -55,7 +61,7 @@ dia_image_get_broken(void)
 
   if (broken == NULL) {
     broken = g_new(struct _DiaImage, 1);
-    broken->image = gdk_pixbuf_new_from_xpm_data((const char **)broken_xpm);
+    broken->image = gdk_pixbuf_new_from_inline(-1, dia_broken_icon, FALSE, NULL);
   } else {
     gdk_pixbuf_ref(broken->image);
   }
@@ -67,15 +73,29 @@ dia_image_get_broken(void)
   return broken;
 }
 
-DiaImage 
-dia_image_load(gchar *filename) 
+/** Load an image from file.
+ * @param filename Name of the file to load.
+ * @returns An image loaded from file, or NULL if an error occurred.
+ *          Error messages will be displayed to the user.
+ */
+DiaImage
+dia_image_load(gchar *filename)
 {
   DiaImage dia_img;
   GdkPixbuf *image;
+  GError *error = NULL;
 
-  image = gdk_pixbuf_new_from_file(filename, NULL);
-  if (image == NULL)
+  image = gdk_pixbuf_new_from_file(filename, &error);
+  if (image == NULL) {
+    /* dia_image_load() function is also (mis)used to check file
+     * existance. Don't warn if the file is simply not there but
+     * only if there is something else wrong while loading it.
+     */
+    if (g_file_test(filename, G_FILE_TEST_EXISTS))
+      g_warning ("%s", error->message);
+    g_error_free (error);
     return NULL;
+  }
 
   dia_img = g_new(struct _DiaImage, 1);
   dia_img->image = image;
@@ -87,24 +107,40 @@ dia_image_load(gchar *filename)
   return dia_img;
 }
 
+/** Reference an image.
+ * @param image Image that we want a reference to.
+ */
 void
 dia_image_add_ref(DiaImage image)
 {
   gdk_pixbuf_ref(image->image);
 }
 
+/** Release a reference to an image.
+ * @param image Image to unreference.
+ */
 void
 dia_image_release(DiaImage image)
 {
   gdk_pixbuf_unref(image->image);
 }
 
+/** Render an image unto a window.
+ * @param image Image object to render.
+ * @param window The window to render the image into.
+ * @param x X position in window of place to render image.
+ * @param y Y position in window of place to render image.
+ * @param width Width in pixels of rendering in window.
+ * @param height Height in pixels of rendering in window.
+ */
 void 
-dia_image_draw(DiaImage image, GdkWindow *window,
+dia_image_draw(DiaImage image, GdkWindow *window, GdkGC *gc,
 	       int x, int y, int width, int height)
 {
   GdkPixbuf *scaled;
 
+  if (width < 1 || height < 1)
+    return;
   if (gdk_pixbuf_get_width(image->image) != width ||
       gdk_pixbuf_get_height(image->image) != height) {
     /* Using TILES to make it look more like PostScript */
@@ -128,33 +164,50 @@ dia_image_draw(DiaImage image, GdkWindow *window,
   }
 
   /* Once we can render Alpha, we'll do it! */
-  gdk_pixbuf_render_to_drawable_alpha(scaled, window,
-				      0, 0, x, y, width, height, 
-				      GDK_PIXBUF_ALPHA_BILEVEL, 128,
-				      GDK_RGB_DITHER_NORMAL, 0, 0);
+  gdk_draw_pixbuf(window, gc, scaled,
+		  0, 0, x, y, width, height, 
+		  GDK_RGB_DITHER_NORMAL, 0, 0);
+
 #ifndef SCALING_CACHE
   gdk_pixbuf_unref(scaled);
 #endif
 }
 
+/** Get the width of an image.
+ * @param image An image object
+ * @returns The natural width of the object in pixels.
+ */
 int 
 dia_image_width(DiaImage image)
 {
   return gdk_pixbuf_get_width(image->image);
 }
 
+/** Get the height of an image.
+ * @param image An image object
+ * @returns The natural height of the object in pixels.
+ */
 int 
 dia_image_height(DiaImage image)
 {
   return gdk_pixbuf_get_height(image->image);
 }
 
+/** Get the rowstride number of bytes per row, see gdk_pixbuf_get_rowstride.
+ * @param image An image object
+ * @returns The rowstride number of the image.
+ */
 int
 dia_image_rowstride(DiaImage image)
 {
   return gdk_pixbuf_get_rowstride(image->image);
 }
 
+/** Get the raw RGB data from an image.
+ * @param image An image object.
+ * @returns An array of bytes (height*rowstride) containing the RGB data
+ *          This array should be freed after use.
+ */
 guint8 *
 dia_image_rgb_data(DiaImage image)
 {
@@ -183,6 +236,11 @@ dia_image_rgb_data(DiaImage image)
   }
 }
 
+/** Get the mask data for an image.
+ * @param image An image object.
+ * @returns An array of bytes (width*height) with the alpha channel information
+ *          from the image.  This array should be freed after use.
+ */
 guint8 *
 dia_image_mask_data(DiaImage image)
 {
@@ -208,6 +266,11 @@ dia_image_mask_data(DiaImage image)
   return mask;
 }
 
+/** Get full RGBA data from an image.
+ * @param image An image object.
+ * @returns An array of pixels as delivered by gdk_pixbuf_get_pixels, or
+ *          NULL if the image has no alpha channel.
+ */
 const guint8 *
 dia_image_rgba_data(DiaImage image)
 {
@@ -220,8 +283,15 @@ dia_image_rgba_data(DiaImage image)
   }
 }
 
-char *
+/** Return the filename associated with an image.
+ * @param image An image object
+ * @returns The filename associated with an image, or "(null)" if the image
+ *          has no filename.  This string should *not* be freed by the caller.
+ */
+const char *
 dia_image_filename(DiaImage image)
 {
+  if (!image->filename)
+    return "(null)";
   return image->filename;
 }
