@@ -34,7 +34,6 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#include <locale.h>
 
 #include <libxml/entities.h>
 #include <libxml/tree.h>
@@ -47,6 +46,14 @@
 #include "dia_image.h"
 
 #include "diasvgrenderer.h"
+#include "textline.h"
+
+#define DTOSTR_BUF_SIZE G_ASCII_DTOSTR_BUF_SIZE
+#define dia_svg_dtostr(buf,d) \
+	g_ascii_formatd(buf,sizeof(buf),"%g",d)
+static void
+draw_text_line(DiaRenderer *self, TextLine *text_line,
+	       Point *pos, Color *colour);
 
 /* DiaSvgRenderer methods */
 static void
@@ -128,44 +135,58 @@ set_linestyle(DiaRenderer *self, LineStyle mode)
 {
   DiaSvgRenderer *renderer = DIA_SVG_RENDERER (self);
   real hole_width;
-  char *old_locale;
+  gchar dash_length_buf[DTOSTR_BUF_SIZE];
+  gchar dot_length_buf[DTOSTR_BUF_SIZE];
+  gchar hole_width_buf[DTOSTR_BUF_SIZE];
 
   renderer->saved_line_style = mode;
 
-  old_locale = setlocale(LC_NUMERIC, "C");
   g_free(renderer->linestyle);
   switch(mode) {
   case LINESTYLE_SOLID:
     renderer->linestyle = NULL;
     break;
   case LINESTYLE_DASHED:
-    renderer->linestyle = g_strdup_printf("%g", renderer->dash_length);
+    dia_svg_dtostr(dash_length_buf, renderer->dash_length);
+    renderer->linestyle = g_strdup_printf("%s", dash_length_buf);
     break;
   case LINESTYLE_DASH_DOT:
     hole_width = (renderer->dash_length - renderer->dot_length) / 2.0;
-    renderer->linestyle = g_strdup_printf("%g %g %g %g",
-					  renderer->dash_length,
-					  hole_width,
-					  renderer->dot_length,
-					  hole_width);
+
+    dia_svg_dtostr(dash_length_buf, renderer->dash_length);
+    dia_svg_dtostr(dot_length_buf, renderer->dot_length);
+    dia_svg_dtostr(hole_width_buf, hole_width);
+
+    renderer->linestyle = g_strdup_printf("%s %s %s %s",
+					  dash_length_buf,
+					  hole_width_buf,
+					  dot_length_buf,
+					  hole_width_buf );
     break;
   case LINESTYLE_DASH_DOT_DOT:
     hole_width = (renderer->dash_length - 2.0*renderer->dot_length) / 3.0;
-    renderer->linestyle = g_strdup_printf("%g %g %g %g %g %g",
-					  renderer->dash_length,
-					  hole_width,
-					  renderer->dot_length,
-					  hole_width,
-					  renderer->dot_length,
-					  hole_width );
+
+    dia_svg_dtostr(dash_length_buf, renderer->dash_length);
+    dia_svg_dtostr(dot_length_buf, renderer->dot_length);
+    dia_svg_dtostr(hole_width_buf, hole_width);
+
+    renderer->linestyle = g_strdup_printf("%s %s %s %s %s %s",
+					  dash_length_buf,
+					  hole_width_buf,
+					  dot_length_buf,
+					  hole_width_buf,
+					  dot_length_buf,
+					  hole_width_buf );
     break;
   case LINESTYLE_DOTTED:
-    renderer->linestyle = g_strdup_printf("%g", renderer->dot_length);
+
+    dia_svg_dtostr(dot_length_buf, renderer->dot_length);
+
+    renderer->linestyle = g_strdup_printf("%s", dot_length_buf);
     break;
   default:
     renderer->linestyle = NULL;
   }
-  setlocale(LC_NUMERIC, old_locale);
 }
 
 static void
@@ -199,15 +220,13 @@ get_draw_style(DiaSvgRenderer *renderer,
 	       Color *colour)
 {
   static GString *str = NULL;
-  char *old_locale;
+  gchar linewidth_buf[DTOSTR_BUF_SIZE];
 
   if (!str) str = g_string_new(NULL);
   g_string_truncate(str, 0);
 
   /* TODO(CHECK): the shape-export didn't have 'fill: none' here */
-  old_locale = setlocale(LC_NUMERIC, "C");
-  g_string_sprintf(str, "fill: none; fill-opacity:0; stroke-width: %g", renderer->linewidth);
-  setlocale(LC_NUMERIC, old_locale);
+  g_string_sprintf(str, "fill: none; fill-opacity:0; stroke-width: %s", dia_svg_dtostr(linewidth_buf, renderer->linewidth) );
   if (strcmp(renderer->linecap, "butt"))
     g_string_sprintfa(str, "; stroke-linecap: %s", renderer->linecap);
   if (strcmp(renderer->linejoin, "miter"))
@@ -246,23 +265,20 @@ draw_line(DiaRenderer *self,
 {
   DiaSvgRenderer *renderer = DIA_SVG_RENDERER (self);
   xmlNodePtr node;
-  char buf[512];
-  char *old_locale;
+  gchar d_buf[DTOSTR_BUF_SIZE];
 
   node = xmlNewChild(renderer->root, renderer->svg_name_space, "line", NULL);
 
   xmlSetProp(node, "style", get_draw_style(renderer, line_colour));
 
-  old_locale = setlocale(LC_NUMERIC, "C");
-  g_snprintf(buf, sizeof(buf), "%g", start->x);
-  xmlSetProp(node, "x1", buf);
-  g_snprintf(buf, sizeof(buf), "%g", start->y);
-  xmlSetProp(node, "y1", buf);
-  g_snprintf(buf, sizeof(buf), "%g", end->x);
-  xmlSetProp(node, "x2", buf);
-  g_snprintf(buf, sizeof(buf), "%g", end->y);
-  xmlSetProp(node, "y2", buf);
-  setlocale(LC_NUMERIC, old_locale);
+  dia_svg_dtostr(d_buf, start->x);
+  xmlSetProp(node, "x1", d_buf);
+  dia_svg_dtostr(d_buf, start->y);
+  xmlSetProp(node, "y1", d_buf);
+  dia_svg_dtostr(d_buf, end->x);
+  xmlSetProp(node, "x2", d_buf);
+  dia_svg_dtostr(d_buf, end->y);
+  xmlSetProp(node, "y2", d_buf);
 }
 
 static void
@@ -274,19 +290,20 @@ draw_polyline(DiaRenderer *self,
   int i;
   xmlNodePtr node;
   GString *str;
-  char *old_locale;
+  gchar px_buf[DTOSTR_BUF_SIZE];
+  gchar py_buf[DTOSTR_BUF_SIZE];
 
   node = xmlNewChild(renderer->root, renderer->svg_name_space, "polyline", NULL);
   
   xmlSetProp(node, "style", get_draw_style(renderer, line_colour));
 
-  old_locale = setlocale(LC_NUMERIC, "C");
   str = g_string_new(NULL);
   for (i = 0; i < num_points; i++)
-    g_string_sprintfa(str, "%g,%g ", points[i].x, points[i].y);
+    g_string_sprintfa(str, "%s,%s ",
+		      dia_svg_dtostr(px_buf, points[i].x),
+		      dia_svg_dtostr(py_buf, points[i].y) );
   xmlSetProp(node, "points", str->str);
   g_string_free(str, TRUE);
-  setlocale(LC_NUMERIC, old_locale);
 }
 
 static void
@@ -298,19 +315,20 @@ draw_polygon(DiaRenderer *self,
   int i;
   xmlNodePtr node;
   GString *str;
-  char *old_locale;
+  gchar px_buf[DTOSTR_BUF_SIZE];
+  gchar py_buf[DTOSTR_BUF_SIZE];
 
   node = xmlNewChild(renderer->root, renderer->svg_name_space, "polygon", NULL);
   
   xmlSetProp(node, "style", get_draw_style(renderer, line_colour));
 
-  old_locale = setlocale(LC_NUMERIC, "C");
   str = g_string_new(NULL);
   for (i = 0; i < num_points; i++)
-    g_string_sprintfa(str, "%g,%g ", points[i].x, points[i].y);
+    g_string_sprintfa(str, "%s,%s ",
+		      dia_svg_dtostr(px_buf, points[i].x),
+		      dia_svg_dtostr(py_buf, points[i].y) );
   xmlSetProp(node, "points", str->str);
   g_string_free(str, TRUE);
-  setlocale(LC_NUMERIC, old_locale);
 }
 
 static void
@@ -322,19 +340,20 @@ fill_polygon(DiaRenderer *self,
   int i;
   xmlNodePtr node;
   GString *str;
-  char *old_locale;
+  gchar px_buf[DTOSTR_BUF_SIZE];
+  gchar py_buf[DTOSTR_BUF_SIZE];
 
   node = xmlNewChild(renderer->root, renderer->svg_name_space, "polygon", NULL);
   
   xmlSetProp(node, "style", get_fill_style(renderer, colour));
 
-  old_locale = setlocale(LC_NUMERIC, "C");
   str = g_string_new(NULL);
   for (i = 0; i < num_points; i++)
-    g_string_sprintfa(str, "%g,%g ", points[i].x, points[i].y);
+    g_string_sprintfa(str, "%s,%s ",
+		      dia_svg_dtostr(px_buf, points[i].x),
+		      dia_svg_dtostr(py_buf, points[i].y) );
   xmlSetProp(node, "points", str->str);
   g_string_free(str, TRUE);
-  setlocale(LC_NUMERIC, old_locale);
 }
 
 static void
@@ -344,23 +363,20 @@ draw_rect(DiaRenderer *self,
 {
   DiaSvgRenderer *renderer = DIA_SVG_RENDERER (self);
   xmlNodePtr node;
-  char buf[512];
-  char *old_locale;
+  gchar d_buf[DTOSTR_BUF_SIZE];
 
   node = xmlNewChild(renderer->root, NULL, "rect", NULL);
 
   xmlSetProp(node, "style", get_draw_style(renderer, colour));
 
-  old_locale = setlocale(LC_NUMERIC, "C");
-  g_snprintf(buf, sizeof(buf), "%g", ul_corner->x);
-  xmlSetProp(node, "x", buf);
-  g_snprintf(buf, sizeof(buf), "%g", ul_corner->y);
-  xmlSetProp(node, "y", buf);
-  g_snprintf(buf, sizeof(buf), "%g", lr_corner->x - ul_corner->x);
-  xmlSetProp(node, "width", buf);
-  g_snprintf(buf, sizeof(buf), "%g", lr_corner->y - ul_corner->y);
-  xmlSetProp(node, "height", buf);
-  setlocale(LC_NUMERIC, old_locale);
+  dia_svg_dtostr(d_buf, ul_corner->x);
+  xmlSetProp(node, "x", d_buf);
+  dia_svg_dtostr(d_buf, ul_corner->y);
+  xmlSetProp(node, "y", d_buf);
+  dia_svg_dtostr(d_buf, lr_corner->x - ul_corner->x);
+  xmlSetProp(node, "width", d_buf);
+  dia_svg_dtostr(d_buf, lr_corner->y - ul_corner->y);
+  xmlSetProp(node, "height", d_buf);
 }
 
 static void
@@ -370,36 +386,20 @@ fill_rect(DiaRenderer *self,
 {
   DiaSvgRenderer *renderer = DIA_SVG_RENDERER (self);
   xmlNodePtr node;
-  char buf[512];
-  char *old_locale;
+  gchar d_buf[DTOSTR_BUF_SIZE];
 
   node = xmlNewChild(renderer->root, renderer->svg_name_space, "rect", NULL);
 
   xmlSetProp(node, "style", get_fill_style(renderer, colour));
 
-  old_locale = setlocale(LC_NUMERIC, "C");
-  g_snprintf(buf, sizeof(buf), "%g", ul_corner->x);
-  xmlSetProp(node, "x", buf);
-  g_snprintf(buf, sizeof(buf), "%g", ul_corner->y);
-  xmlSetProp(node, "y", buf);
-  g_snprintf(buf, sizeof(buf), "%g", lr_corner->x - ul_corner->x);
-  xmlSetProp(node, "width", buf);
-  g_snprintf(buf, sizeof(buf), "%g", lr_corner->y - ul_corner->y);
-  xmlSetProp(node, "height", buf);
-  setlocale(LC_NUMERIC, old_locale);
-}
-
-static int sweep (real x1,real y1,real x2,real y2,real x3,real y3)
-{
-  real X2=x2-x1;
-  real Y2=y2-y1;
-  real X3=x3-x1;
-  real Y3=y3-y1;
-  real L=sqrt(X2*X2+Y2*Y2);
-  real cost=X2/L;
-  real sint=Y2/L;
-  real res=-X3*sint+Y3*cost;
-  return res>0;
+  dia_svg_dtostr(d_buf, ul_corner->x);
+  xmlSetProp(node, "x", d_buf);
+  dia_svg_dtostr(d_buf, ul_corner->y);
+  xmlSetProp(node, "y", d_buf);
+  dia_svg_dtostr(d_buf, lr_corner->x - ul_corner->x);
+  xmlSetProp(node, "width", d_buf);
+  dia_svg_dtostr(d_buf, lr_corner->y - ul_corner->y);
+  xmlSetProp(node, "height", d_buf);
 }
 
 static void
@@ -413,36 +413,30 @@ draw_arc(DiaRenderer *self,
   xmlNodePtr node;
   char buf[512];
   real rx = width / 2, ry = height / 2;
-  real x1=center->x + rx*cos(angle1*G_PI/180);
-  real y1=center->y - ry*sin(angle1*G_PI/180);
-  real x2=center->x + rx*cos(angle2*G_PI/180);
-  real y2=center->y - ry*sin(angle2*G_PI/180);
-  int swp = sweep(x1,y1,x2,y2,center->x,center->y);
-  int l_arc;
-  char *old_locale;
-
-  if (angle2 > angle1) {
-    l_arc = (angle2 - angle1) > 180;
-  } else {
-    l_arc = (360 - angle2 + angle1) > 180;
-  }
-  
-  if (l_arc)
-      swp = !swp;
+  real sx=center->x + rx*cos(angle1*G_PI/180);
+  real sy=center->y - ry*sin(angle1*G_PI/180);
+  real ex=center->x + rx*cos(angle2*G_PI/180);
+  real ey=center->y - ry*sin(angle2*G_PI/180);
+  int swp = 0; /* always drawin negative direction */
+  int large_arc = (angle2 - angle1 >= 180);
+  gchar sx_buf[DTOSTR_BUF_SIZE];
+  gchar sy_buf[DTOSTR_BUF_SIZE];
+  gchar rx_buf[DTOSTR_BUF_SIZE];
+  gchar ry_buf[DTOSTR_BUF_SIZE];
+  gchar ex_buf[DTOSTR_BUF_SIZE];
+  gchar ey_buf[DTOSTR_BUF_SIZE];
 
   node = xmlNewChild(renderer->root, renderer->svg_name_space, "path", NULL);
   
   xmlSetProp(node, "style", get_draw_style(renderer, colour));
 
-  /* this path might be incorrect ... */
-  old_locale = setlocale(LC_NUMERIC, "C");
-  g_snprintf(buf, sizeof(buf), "M %g,%g A %g,%g 0 %d %d %g,%g",
-	     x1, y1,
-	     rx, ry,l_arc ,swp ,
-	     x2, y2);
+  g_snprintf(buf, sizeof(buf), "M %s,%s A %s,%s 0 %d %d %s,%s",
+	     dia_svg_dtostr(sx_buf, sx), dia_svg_dtostr(sy_buf, sy),
+	     dia_svg_dtostr(rx_buf, rx), dia_svg_dtostr(ry_buf, ry),
+	     large_arc, swp,
+	     dia_svg_dtostr(ex_buf, ex), dia_svg_dtostr(ey_buf, ey) );
 
   xmlSetProp(node, "d", buf);
-  setlocale(LC_NUMERIC, old_locale);
 }
 
 static void
@@ -456,37 +450,34 @@ fill_arc(DiaRenderer *self,
   xmlNodePtr node;
   char buf[512];
   real rx = width / 2, ry = height / 2;
-  real x1=center->x + rx*cos(angle1*G_PI/180);
-  real y1=center->y - ry*sin(angle1*G_PI/180);
-  real x2=center->x + rx*cos(angle2*G_PI/180);
-  real y2=center->y - ry*sin(angle2*G_PI/180);
-  int swp = sweep(x1,y1,x2,y2,center->x,center->y);
-  int l_arc;
-  char *old_locale;
-
-  if (angle2 > angle1) {
-    l_arc = (angle2 - angle1) > 180;
-  } else {
-    l_arc = (360 - angle2 + angle1) > 180;
-  }
-
-  if (l_arc)
-      swp = !swp;
+  real sx=center->x + rx*cos(angle1*G_PI/180);
+  real sy=center->y - ry*sin(angle1*G_PI/180);
+  real ex=center->x + rx*cos(angle2*G_PI/180);
+  real ey=center->y - ry*sin(angle2*G_PI/180);
+  int swp = 0; /* always drawin negative direction */
+  int large_arc = (angle2 - angle1 >= 180);
+  gchar sx_buf[DTOSTR_BUF_SIZE];
+  gchar sy_buf[DTOSTR_BUF_SIZE];
+  gchar rx_buf[DTOSTR_BUF_SIZE];
+  gchar ry_buf[DTOSTR_BUF_SIZE];
+  gchar ex_buf[DTOSTR_BUF_SIZE];
+  gchar ey_buf[DTOSTR_BUF_SIZE];
+  gchar cx_buf[DTOSTR_BUF_SIZE];
+  gchar cy_buf[DTOSTR_BUF_SIZE];
 
   node = xmlNewChild(renderer->root, NULL, "path", NULL);
   
   xmlSetProp(node, "style", get_fill_style(renderer, colour));
 
-  /* this path might be incorrect ... */
-  old_locale = setlocale(LC_NUMERIC, "C");
-  g_snprintf(buf, sizeof(buf), "M %g,%g A %g,%g 0 %d %d %g,%g L %g,%g z",
-	     x1, y1,
-	     rx, ry,l_arc ,swp ,
-	     x2, y2,
-	     center->x, center->y);
+  g_snprintf(buf, sizeof(buf), "M %s,%s A %s,%s 0 %d %d %s,%s L %s,%s z",
+	     dia_svg_dtostr(sx_buf, sx), dia_svg_dtostr(sy_buf, sy),
+	     dia_svg_dtostr(rx_buf, rx), dia_svg_dtostr(ry_buf, ry),
+	     large_arc, swp,
+	     dia_svg_dtostr(ex_buf, ex), dia_svg_dtostr(ey_buf, ey),
+	     dia_svg_dtostr(cx_buf, center->x),
+	     dia_svg_dtostr(cy_buf, center->y) );
 
   xmlSetProp(node, "d", buf);
-  setlocale(LC_NUMERIC, old_locale);
 }
 
 static void
@@ -497,23 +488,20 @@ draw_ellipse(DiaRenderer *self,
 {
   DiaSvgRenderer *renderer = DIA_SVG_RENDERER (self);
   xmlNodePtr node;
-  char buf[512];
-  char *old_locale;
+  gchar d_buf[DTOSTR_BUF_SIZE];
 
   node = xmlNewChild(renderer->root, renderer->svg_name_space, "ellipse", NULL);
 
   xmlSetProp(node, "style", get_draw_style(renderer, colour));
 
-  old_locale = setlocale(LC_NUMERIC, "C");
-  g_snprintf(buf, sizeof(buf), "%g", center->x);
-  xmlSetProp(node, "cx", buf);
-  g_snprintf(buf, sizeof(buf), "%g", center->y);
-  xmlSetProp(node, "cy", buf);
-  g_snprintf(buf, sizeof(buf), "%g", width / 2);
-  xmlSetProp(node, "rx", buf);
-  g_snprintf(buf, sizeof(buf), "%g", height / 2);
-  xmlSetProp(node, "ry", buf);
-  setlocale(LC_NUMERIC, old_locale);
+  dia_svg_dtostr(d_buf, center->x);
+  xmlSetProp(node, "cx", d_buf);
+  dia_svg_dtostr(d_buf, center->y);
+  xmlSetProp(node, "cy", d_buf);
+  dia_svg_dtostr(d_buf, width / 2);
+  xmlSetProp(node, "rx", d_buf);
+  dia_svg_dtostr(d_buf, height / 2);
+  xmlSetProp(node, "ry", d_buf);
 }
 
 static void
@@ -524,23 +512,20 @@ fill_ellipse(DiaRenderer *self,
 {
   DiaSvgRenderer *renderer = DIA_SVG_RENDERER (self);
   xmlNodePtr node;
-  char buf[512];
-  char *old_locale;
+  gchar d_buf[DTOSTR_BUF_SIZE];
 
   node = xmlNewChild(renderer->root, renderer->svg_name_space, "ellipse", NULL);
 
   xmlSetProp(node, "style", get_fill_style(renderer, colour));
 
-  old_locale = setlocale(LC_NUMERIC, "C");
-  g_snprintf(buf, sizeof(buf), "%g", center->x);
-  xmlSetProp(node, "cx", buf);
-  g_snprintf(buf, sizeof(buf), "%g", center->y);
-  xmlSetProp(node, "cy", buf);
-  g_snprintf(buf, sizeof(buf), "%g", width / 2);
-  xmlSetProp(node, "rx", buf);
-  g_snprintf(buf, sizeof(buf), "%g", height / 2);
-  xmlSetProp(node, "ry", buf);
-  setlocale(LC_NUMERIC, old_locale);
+  dia_svg_dtostr(d_buf, center->x);
+  xmlSetProp(node, "cx", d_buf);
+  dia_svg_dtostr(d_buf, center->y);
+  xmlSetProp(node, "cy", d_buf);
+  dia_svg_dtostr(d_buf, width / 2);
+  xmlSetProp(node, "rx", d_buf);
+  dia_svg_dtostr(d_buf, height / 2);
+  xmlSetProp(node, "ry", d_buf);
 }
 
 static void
@@ -553,7 +538,12 @@ draw_bezier(DiaRenderer *self,
   int i;
   xmlNodePtr node;
   GString *str;
-  char *old_locale;
+  gchar p1x_buf[DTOSTR_BUF_SIZE];
+  gchar p1y_buf[DTOSTR_BUF_SIZE];
+  gchar p2x_buf[DTOSTR_BUF_SIZE];
+  gchar p2y_buf[DTOSTR_BUF_SIZE];
+  gchar p3x_buf[DTOSTR_BUF_SIZE];
+  gchar p3y_buf[DTOSTR_BUF_SIZE];
 
   node = xmlNewChild(renderer->root, renderer->svg_name_space, "path", NULL);
   
@@ -561,12 +551,12 @@ draw_bezier(DiaRenderer *self,
 
   str = g_string_new(NULL);
 
-  old_locale = setlocale(LC_NUMERIC, "C");
   if (points[0].type != BEZ_MOVE_TO)
     g_warning("first BezPoint must be a BEZ_MOVE_TO");
 
-  g_string_sprintf(str, "M %g %g", (double)points[0].p1.x,
-		   (double)points[0].p1.y);
+  g_string_sprintf(str, "M %s %s",
+		   dia_svg_dtostr(p1x_buf, (gdouble) points[0].p1.x),
+		   dia_svg_dtostr(p1y_buf, (gdouble) points[0].p1.y) );
 
   for (i = 1; i < numpoints; i++)
     switch (points[i].type) {
@@ -574,19 +564,22 @@ draw_bezier(DiaRenderer *self,
       g_warning("only first BezPoint can be a BEZ_MOVE_TO");
       break;
     case BEZ_LINE_TO:
-      g_string_sprintfa(str, " L %g,%g",
-			(double) points[i].p1.x, (double) points[i].p1.y);
+      g_string_sprintfa(str, " L %s,%s",
+			dia_svg_dtostr(p1x_buf, (gdouble) points[i].p1.x),
+			dia_svg_dtostr(p1y_buf, (gdouble) points[i].p1.y) );
       break;
     case BEZ_CURVE_TO:
-      g_string_sprintfa(str, " C %g,%g %g,%g %g,%g",
-			(double) points[i].p1.x, (double) points[i].p1.y,
-			(double) points[i].p2.x, (double) points[i].p2.y,
-			(double) points[i].p3.x, (double) points[i].p3.y );
+      g_string_sprintfa(str, " C %s,%s %s,%s %s,%s",
+			dia_svg_dtostr(p1x_buf, (gdouble) points[i].p1.x),
+			dia_svg_dtostr(p1y_buf, (gdouble) points[i].p1.y),
+			dia_svg_dtostr(p2x_buf, (gdouble) points[i].p2.x),
+			dia_svg_dtostr(p2y_buf, (gdouble) points[i].p2.y),
+			dia_svg_dtostr(p3x_buf, (gdouble) points[i].p3.x),
+			dia_svg_dtostr(p3y_buf, (gdouble) points[i].p3.y) );
       break;
     }
   xmlSetProp(node, "d", str->str);
   g_string_free(str, TRUE);
-  setlocale(LC_NUMERIC, old_locale);
 }
 
 static void
@@ -599,7 +592,12 @@ fill_bezier(DiaRenderer *self,
   int i;
   xmlNodePtr node;
   GString *str;
-  char *old_locale;
+  gchar p1x_buf[DTOSTR_BUF_SIZE];
+  gchar p1y_buf[DTOSTR_BUF_SIZE];
+  gchar p2x_buf[DTOSTR_BUF_SIZE];
+  gchar p2y_buf[DTOSTR_BUF_SIZE];
+  gchar p3x_buf[DTOSTR_BUF_SIZE];
+  gchar p3y_buf[DTOSTR_BUF_SIZE];
 
   node = xmlNewChild(renderer->root, renderer->svg_name_space, "path", NULL);
   
@@ -607,12 +605,12 @@ fill_bezier(DiaRenderer *self,
 
   str = g_string_new(NULL);
 
-  old_locale = setlocale(LC_NUMERIC, "C");
   if (points[0].type != BEZ_MOVE_TO)
     g_warning("first BezPoint must be a BEZ_MOVE_TO");
 
-  g_string_sprintf(str, "M %g %g", (double)points[0].p1.x,
-		   (double)points[0].p1.y);
+  g_string_sprintf(str, "M %s %s",
+		   dia_svg_dtostr(p1x_buf, (gdouble) points[0].p1.x),
+		   dia_svg_dtostr(p1y_buf, (gdouble) points[0].p1.y) );
  
   for (i = 1; i < numpoints; i++)
     switch (points[i].type) {
@@ -620,20 +618,23 @@ fill_bezier(DiaRenderer *self,
       g_warning("only first BezPoint can be a BEZ_MOVE_TO");
       break;
     case BEZ_LINE_TO:
-      g_string_sprintfa(str, " L %g,%g",
-			(double) points[i].p1.x, (double) points[i].p1.y);
+      g_string_sprintfa(str, " L %s,%s",
+			dia_svg_dtostr(p1x_buf, (gdouble) points[i].p1.x),
+			dia_svg_dtostr(p1y_buf, (gdouble) points[i].p1.y) );
       break;
     case BEZ_CURVE_TO:
-      g_string_sprintfa(str, " C %g,%g %g,%g %g,%g",
-			(double) points[i].p1.x, (double) points[i].p1.y,
-			(double) points[i].p2.x, (double) points[i].p2.y,
-			(double) points[i].p3.x, (double) points[i].p3.y );
+      g_string_sprintfa(str, " C %s,%s %s,%s %s,%s",
+			dia_svg_dtostr(p1x_buf, (gdouble) points[i].p1.x),
+			dia_svg_dtostr(p1y_buf, (gdouble) points[i].p1.y),
+			dia_svg_dtostr(p2x_buf, (gdouble) points[i].p2.x),
+			dia_svg_dtostr(p2y_buf, (gdouble) points[i].p2.y),
+			dia_svg_dtostr(p3x_buf, (gdouble) points[i].p3.x),
+			dia_svg_dtostr(p3y_buf, (gdouble) points[i].p3.y) );
       break;
     }
   g_string_append(str, "z");
   xmlSetProp(node, "d", str->str);
   g_string_free(str, TRUE);
-  setlocale(LC_NUMERIC, old_locale);
 }
 
 static void
@@ -642,12 +643,17 @@ draw_string(DiaRenderer *self,
 	    Point *pos, Alignment alignment,
 	    Color *colour)
 {    
+  TextLine *text_line = text_line_new(text, self->font, self->font_height);
+  Point realigned_pos = *pos;
+  realigned_pos.x -= text_line_get_alignment_adjustment(text_line, alignment);
+  draw_text_line(self, text_line, &realigned_pos, colour);
+  return;
+#if 0
   DiaSvgRenderer *renderer = DIA_SVG_RENDERER (self);
   xmlNodePtr node;
-  char buf[512];
   char *style, *tmp;
   real saved_width;
-  char *old_locale;
+  gchar d_buf[DTOSTR_BUF_SIZE];
 
   node = xmlNewChild(renderer->root, renderer->svg_name_space, "text", text);
  
@@ -670,9 +676,8 @@ draw_string(DiaRenderer *self,
     style = g_strconcat(style, "; text-anchor:end", NULL);
     break;
   }
-  old_locale = setlocale(LC_NUMERIC, "C");
-  tmp = g_strdup_printf("%s; font-size: %g", style, self->font_height);
-  setlocale(LC_NUMERIC, old_locale);
+  tmp = g_strdup_printf("%s; font-size: %s cm", style,
+			dia_svg_dtostr(d_buf, self->font_height) );
   g_free (style);
   style = tmp;
 
@@ -691,12 +696,60 @@ draw_string(DiaRenderer *self,
   xmlSetProp(node, "style", style);
   g_free(style);
 
-  old_locale = setlocale(LC_NUMERIC, "C");
-  g_snprintf(buf, sizeof(buf), "%g", pos->x);
-  xmlSetProp(node, "x", buf);
-  g_snprintf(buf, sizeof(buf), "%g", pos->y);
-  xmlSetProp(node, "y", buf);
-  setlocale(LC_NUMERIC, old_locale);
+  dia_svg_dtostr(d_buf, pos->x);
+  xmlSetProp(node, "x", d_buf);
+  dia_svg_dtostr(d_buf, pos->y);
+  xmlSetProp(node, "y", d_buf);
+#endif
+}
+
+
+static void
+draw_text_line(DiaRenderer *self, TextLine *text_line,
+	       Point *pos, Color *colour)
+{    
+  DiaSvgRenderer *renderer = DIA_SVG_RENDERER (self);
+  xmlNodePtr node;
+  char *style, *tmp;
+  real saved_width;
+  gchar d_buf[DTOSTR_BUF_SIZE];
+  DiaFont *font;
+
+  node = xmlNewChild(renderer->root, renderer->svg_name_space, "text", 
+		     text_line_get_string(text_line));
+ 
+  saved_width = renderer->linewidth;
+  renderer->linewidth = 0.001;
+  style = (char*)get_fill_style(renderer, colour);
+  /* return value must not be freed */
+  renderer->linewidth = saved_width;
+  tmp = g_strdup_printf("%s; font-size: %s cm", style,
+			dia_svg_dtostr(d_buf, text_line_get_height(text_line)));
+  style = tmp;
+  
+  tmp = g_strdup_printf("%s; length: %s cm", style,
+			dia_svg_dtostr(d_buf, text_line_get_width(text_line)));
+  g_free (style);
+  style = tmp;
+
+  font = text_line_get_font(text_line);
+  tmp = g_strdup_printf("%s; font-family: %s; font-style: %s; "
+			"font-weight: %s",style,
+			dia_font_get_family(font),
+			dia_font_get_slant_string(font),
+			dia_font_get_weight_string(font));
+  g_free(style);
+  style = tmp;
+
+  /* have to do something about fonts here ... */
+
+  xmlSetProp(node, "style", style);
+  g_free(style);
+
+  dia_svg_dtostr(d_buf, pos->x);
+  xmlSetProp(node, "x", d_buf);
+  dia_svg_dtostr(d_buf, pos->y);
+  xmlSetProp(node, "y", d_buf);
 }
 
 static void
@@ -707,22 +760,19 @@ draw_image(DiaRenderer *self,
 {
   DiaSvgRenderer *renderer = DIA_SVG_RENDERER (self);
   xmlNodePtr node;
-  char buf[512];
-  char *old_locale;
+  gchar d_buf[DTOSTR_BUF_SIZE];
 
   node = xmlNewChild(renderer->root, NULL, "image", NULL);
 
-  old_locale = setlocale(LC_NUMERIC, "C");
-  g_snprintf(buf, sizeof(buf), "%g", point->x);
-  xmlSetProp(node, "x", buf);
-  g_snprintf(buf, sizeof(buf), "%g", point->y);
-  xmlSetProp(node, "y", buf);
-  g_snprintf(buf, sizeof(buf), "%g", width);
-  xmlSetProp(node, "width", buf);
-  g_snprintf(buf, sizeof(buf), "%g", height);
-  xmlSetProp(node, "height", buf);
+  dia_svg_dtostr(d_buf, point->x);
+  xmlSetProp(node, "x", d_buf);
+  dia_svg_dtostr(d_buf, point->y);
+  xmlSetProp(node, "y", d_buf);
+  dia_svg_dtostr(d_buf, width);
+  xmlSetProp(node, "width", d_buf);
+  dia_svg_dtostr(d_buf, height);
+  xmlSetProp(node, "height", d_buf);
   xmlSetProp(node, "xlink:href", dia_image_filename(image));
-  setlocale(LC_NUMERIC, old_locale);
 }
 
 /* constructor */
@@ -812,6 +862,7 @@ dia_svg_renderer_class_init (DiaSvgRendererClass *klass)
 
   renderer_class->draw_bezier   = draw_bezier;
   renderer_class->fill_bezier   = fill_bezier;
+  renderer_class->draw_text_line  = draw_text_line;
 
   /* svg specific */
   svg_renderer_class->get_draw_style = get_draw_style;
