@@ -19,12 +19,17 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "filter.h"
 #include "intl.h"
 #include <string.h>
 
 static GList *export_filters = NULL;
 static GList *import_filters = NULL;
+static GList *callback_filters = NULL;
 
 static gint
 export_filter_compare(gconstpointer a, gconstpointer b)
@@ -37,10 +42,19 @@ export_filter_compare(gconstpointer a, gconstpointer b)
 void
 filter_register_export(DiaExportFilter *efilter)
 {
+  if (efilter->description == NULL) {
+    return;
+  }
   export_filters = g_list_insert_sorted(export_filters, efilter,
 					export_filter_compare);
 }
 
+void
+filter_unregister_export(DiaExportFilter *efilter)
+{
+  export_filters = g_list_remove(export_filters, efilter);
+}
+
 /* returns a sorted list of the export filters. */
 GList *
 filter_get_export_filters(void)
@@ -70,7 +84,9 @@ filter_get_export_filter_label(DiaExportFilter *efilter)
   return ret;
 }
 
-/* guess the filter for a given filename. */
+/* Guess the filter for a given filename. 
+ * Returns the first filter found that matches the extension on the filename,
+ * or NULL if none such are found. */
 DiaExportFilter *
 filter_guess_export_filter(const gchar *filename)
 {
@@ -94,6 +110,27 @@ filter_guess_export_filter(const gchar *filename)
   return NULL;
 }
 
+/** Get an export filter by unique name.
+ */
+DiaExportFilter *
+filter_get_by_name(const gchar *name) 
+{
+  GList *tmp;
+  DiaExportFilter *filter = NULL;
+
+  for (tmp = export_filters; tmp != NULL; tmp = tmp->next) {
+    DiaExportFilter *ef = tmp->data;
+    if (ef->unique_name != NULL) {
+      if (!g_strcasecmp(ef->unique_name, name)) {
+	if (filter) 
+	  g_warning(_("Multiple export filters with unique name %s"), name);
+	filter = ef;
+      }
+    }
+  }
+  return filter;
+}
+
 static gint
 import_filter_compare(gconstpointer a, gconstpointer b)
 {
@@ -105,10 +142,19 @@ import_filter_compare(gconstpointer a, gconstpointer b)
 void
 filter_register_import(DiaImportFilter *ifilter)
 {
+  if (ifilter->description == NULL) {
+    return;
+  }
   import_filters = g_list_insert_sorted(import_filters, ifilter,
 					import_filter_compare);
 }
 
+void
+filter_unregister_import(DiaImportFilter *ifilter)
+{
+  import_filters = g_list_remove(import_filters, ifilter);
+}
+
 /* returns a sorted list of the export filters. */
 GList *
 filter_get_import_filters(void)
@@ -162,3 +208,24 @@ filter_guess_import_filter(const gchar *filename)
   return NULL;
 }
 
+/* register a new callback from a plug-in */
+void 
+filter_register_callback(DiaCallbackFilter *cbfilter)
+{
+  /* sanity check */
+  g_return_if_fail (cbfilter != NULL);
+  g_return_if_fail (cbfilter->callback != NULL);
+  g_return_if_fail (cbfilter->menupath != NULL);
+  g_return_if_fail (cbfilter->description != NULL);
+  g_return_if_fail (cbfilter->action != NULL);
+
+  /* callback_filters is always pointing to the last element */
+  callback_filters = g_list_append (callback_filters, (gpointer)cbfilter);
+}
+
+/* return the registered callbacks list, called once to build menus */
+GList *
+filter_get_callbacks(void)
+{
+  return g_list_first (callback_filters);
+}
