@@ -34,7 +34,7 @@
 
 
 /** 
- * In a goodly selection of fonts, 500 is very common, yet they don't name it.
+ * In a goodly selection of fonts, 500 is very common, yet Pango doesn't name it.
  * I am calling 500 'medium' and 600 'demibold'.
  * We should really have a more flexible system where 300 or 400 is normal, 
  * the next one up bold, or something.  But this will do for now.
@@ -99,14 +99,6 @@ struct _DiaFontClass {
     GObjectClass parent_class;
 };
 
-struct _DiaFont {
-    GObject parent_instance;
-
-        /* Do not access directly !*/
-    PangoFontDescription* pfd;    
-    /* mutable */ char* legacy_name;    
-};
-
 /* Set the PangoContext used to render text.
  */
 void dia_font_init(PangoContext* pcontext);
@@ -145,6 +137,8 @@ DiaFontStyle dia_font_get_style(const DiaFont* font);
 
     /* Retrieves the family of the font. Caller must NOT free. */
 G_CONST_RETURN char* dia_font_get_family(const DiaFont* font);
+/* Acessor for the PangoFontDescription */
+G_CONST_RETURN PangoFontDescription *dia_font_get_description (const DiaFont* font);
 
     /* Retrieves the height of the font */
 real dia_font_get_height(const DiaFont* font);
@@ -192,6 +186,15 @@ real dia_font_descent(const char* string, DiaFont* font, real height);
     /* prepares a layout of the text, in font 'font'. */
 PangoLayout* dia_font_build_layout(const char* string, DiaFont* font,
                                    real height);
+/* prepares a layout of the text, in font 'font' scaled by matrix. */
+PangoLayout* dia_font_build_layout_with_matrix(const char* string,
+					       DiaFont* font,
+					       real height,
+					       PangoMatrix *matrix);
+
+real* dia_font_get_sizes(const char* string, DiaFont *font, real height,
+			 real* width, real* ascent, real* descent, 
+			 int *n_offsets, PangoLayoutLine **layout_offsets);
 
 /* -------- Font and string functions - scaled versions.
    Use these version in Renderers, exclusively. */
@@ -200,27 +203,5 @@ PangoLayout* dia_font_build_layout(const char* string, DiaFont* font,
      what is 1:1 scale. zoom_factor will then be divided by size_one. */
 void dia_font_set_nominal_zoom_factor(real size_one);
 
-    /* Get the width of the string with the given font in cm */
-real dia_font_scaled_string_width(const char* string, DiaFont* font,
-                                  real height, real zoom_factor);
-    /* Get the max ascent of the font in cm (a positive number).
-
-    FIXME: may turn out that we need to pass a string here as well. */
-real dia_font_scaled_ascent(const char* string, DiaFont* font,
-                            real height, real zoom_factor);
-    /* Get the max descent of the font in cm (a positive number) 
-       FIXME: may turn out that we need to pass a string here as well. */
-real dia_font_scaled_descent(const char* string, DiaFont* font,
-                             real height, real zoom_factor);
-
-    /* prepares a layout of the text, in font 'font'.
-
-    When zoom_factor != 1.0, may tweak the font's size or stretch so that its
-    bounding box is actually linear with respect to the zoom factor (kerning,
-    ligaturing and other wild beasts usually get in the way of linear
-    scaling). */
-PangoLayout* dia_font_scaled_build_layout(const char *string, DiaFont* font,
-                                          real height, real zoom_factor);
-
 
 #endif /* FONT_H */
