@@ -18,9 +18,9 @@
 
 #include <config.h>
 
-#include "parent.h"
 #include "geometry.h"
 #include "object.h"
+#include "parent.h"
 #include <glib.h>
 
 
@@ -37,7 +37,7 @@ gboolean parent_list_expand(GList *obj_list)
   {
     DiaObject *obj = (DiaObject *) list->data;
 
-    if (obj->can_parent && obj->children)
+    if (object_flags_set(obj, DIA_OBJECT_CAN_PARENT) && obj->children)
     {
       obj_list = g_list_concat(obj_list, g_list_copy(obj->children));
       nothing_affected = FALSE;
@@ -49,14 +49,14 @@ gboolean parent_list_expand(GList *obj_list)
   return nothing_affected;
 }
 
-/*
-   Returns the original list minus any items that appear as children (at any depth) of the
-   objects in the original list
-   This is very different from the parent_list_affected function, which returns a list of ALL
-   objects affected.
+/**
+ * Returns the original list minus any items that appear as children
+ * (at any depth) of the objects in the original list.  This is very
+ * different from the parent_list_affected function, which returns a
+ * list of ALL objects affected.
 
-   The caller must call g_list_free() on the returned list
-   when the list is no longer needed
+ * The caller must call g_list_free() on the returned list
+ * when the list is no longer needed.
  */
 
 GList *parent_list_affected_hierarchy(GList *obj_list)
@@ -135,24 +135,21 @@ GList *parent_list_affected(GList *obj_list)
   going any further
 
   returns TRUE if resizing was obstructed*/
-gboolean parent_handle_move_out_check(DiaObject *object, Point *to)
+gboolean 
+parent_handle_move_out_check(DiaObject *object, Point *to)
 {
-  Rectangle *p_ext, *c_ext;
+  Rectangle p_ext, c_ext;
   Point new_delta;
 
   if (!object->parent)
     return FALSE;
 
-  p_ext = parent_handle_extents(object->parent);
-  c_ext = parent_point_extents(to);
+  parent_handle_extents(object->parent, &p_ext);
+  parent_point_extents(to, &c_ext);
 
-
-  new_delta = parent_move_child_delta(p_ext, c_ext, NULL);
+  new_delta = parent_move_child_delta(&p_ext, &c_ext, NULL);
   point_add(to, &new_delta);
 
-  g_free(p_ext);
-  g_free(c_ext);
-
   if (new_delta.x || new_delta.y)
     return TRUE;
 
@@ -167,61 +164,47 @@ gboolean parent_handle_move_out_check(DiaObject *object, Point *to)
 gboolean parent_handle_move_in_check(DiaObject *object, Point *to, Point *start_at)
 {
   GList *list = object->children;
-  Rectangle *common_ext = NULL;
-  Rectangle *p_ext;
-  Point new_delta;
+  gboolean once = TRUE;
+  Rectangle common_ext;
+  gboolean restricted = FALSE;
 
-  if (!object->can_parent || !object->children)
+  if (!object_flags_set(object, DIA_OBJECT_CAN_PARENT) || !object->children)
     return FALSE;
 
-  p_ext = parent_point_extents(to);
   while (list)
   {
-    if (!common_ext)
-      common_ext = g_memdup(parent_handle_extents(list->data), sizeof(Rectangle));
-    else
-      rectangle_union(common_ext, parent_handle_extents(list->data));
-
+    if (once) {
+      parent_handle_extents(list->data, &common_ext);
+      once = FALSE;
+    } else {
+      Rectangle c_ext;
+      parent_handle_extents (list->data, &c_ext);
+      rectangle_union(&common_ext, &c_ext);
+    }
     list = g_list_next(list);
   }
-  new_delta = parent_move_child_delta_out(p_ext, common_ext, start_at);
-  point_add(to, &new_delta);
-
-  if (new_delta.x || new_delta.y)
-    return TRUE;
-
-  return FALSE;
-}
-
-/* this function is the opposite of parent_move_child_delta()
-   this function makes sure that the parent is OUTSIDE the child's "extent"
-   and if it's inside, it returns a delta that moves it back out
-   Also the last parameter is used as a direction guide FIX ME FIX ME FIX ME FIX ME FIX ME FIX ME
-   (the move is performed in the opposite direction of the passed parameter)
+  /* The start point gives the decision were to clip to:
+   * if it is below the child rect we need to clip to it's bottom.
+   * The check has nothing to do with 'to' being in the rectangle,
+   * but instead having the right barrier
    */
-Point parent_move_child_delta_out(Rectangle *p_ext, Rectangle *c_ext, const Point *start)
-{
-  Point direction;
-  Point new_delta = {0, 0};
+  if (start_at->y >= common_ext.bottom) {
+	if (to->y < common_ext.bottom)
+	  to->y = common_ext.bottom, restricted = TRUE;
+  } else if (start_at->y <= common_ext.top) {
+	if (to->y > common_ext.top)
+	  to->y = common_ext.top, restricted = TRUE;
+  }
 
-  direction.x = p_ext->left - start->x;
-  direction.y = p_ext->top - start->y;
+  if (start_at->x >= common_ext.right) {
+	if (to->x < common_ext.right)
+	  to->x = common_ext.right, restricted = TRUE;
+  } else if (start_at->x <= common_ext.left) {
+	if (to->x > common_ext.left)
+	  to->x = common_ext.left, restricted = TRUE;
+  }
 
-  /* if the start point is to the left of child and we're moving to the right */
-  if (start->x <= c_ext->left && direction.x > 0 && p_ext->left > c_ext->left)
-     new_delta.x = c_ext->left - p_ext->left;
-   /* if the start point is to the right of the child and we're moving left */
-   else if (start->x >= c_ext->right && direction.x < 0 && p_ext->left < c_ext->right)
-     new_delta.x =  c_ext->right - p_ext->left;
-
-   /* if the start point is above the child and we're moving down */
-   if (start->y <= c_ext->top && direction.y > 0 && p_ext->top > c_ext->top)
-     new_delta.y = c_ext->top - p_ext->top;
-   /* if the start poit is below the child and we're moving up */
-   else if (start->y >= c_ext->bottom && direction.y < 0 && p_ext->bottom < c_ext->bottom)
-     new_delta.y =  c_ext->bottom - p_ext->bottom;
-
-   return new_delta;
+  return restricted;
 }
 
     /* p_ext are the "extents" of the parent and c_ext are the "extents" of the child
@@ -258,26 +241,28 @@ Point parent_move_child_delta(Rectangle *p_ext, Rectangle *c_ext, Point *delta)
 }
 
 /* the caller must free the returned rectangle */
-Rectangle *parent_point_extents(Point *point)
+void
+parent_point_extents(Point *point, Rectangle *extents)
 {
-  Rectangle *extents = g_new0(Rectangle, 1);
   extents->left = point->x;
   extents->right = point->x;
   extents->top = point->y;
   extents->bottom = point->y;
-
-  return extents;
 }
 
-/* the caller must free the returned rectangle */
-Rectangle *parent_handle_extents(DiaObject *obj)
+/**
+ * the caller must provide the 'returned' rectangle,
+ * which is initialized to the biggest rectangle containing
+ * all the objects handles
+ */
+gboolean
+parent_handle_extents(DiaObject *obj, Rectangle *extents)
 {
   int idx;
-  Rectangle *extents = g_new0(Rectangle, 1);
   coord *left_most = NULL, *top_most = NULL, *bottom_most = NULL, *right_most = NULL;
 
   if (obj->num_handles == 0)
-    return NULL;
+    return FALSE;
 
   for (idx = 0; idx < obj->num_handles; idx++)
   {
@@ -298,5 +283,21 @@ Rectangle *parent_handle_extents(DiaObject *obj)
   extents->top = *top_most;
   extents->bottom = *bottom_most;
 
-  return extents;
+  return TRUE;
+}
+
+/** Apply a function to all children of the given object (recursively, 
+ * depth-first).
+ * @param obj A parent object.
+ * @param function A function that takes a single DiaObject as an argument.
+ */
+void
+parent_apply_to_children(DiaObject *obj, DiaObjectFunc function)
+{
+  GList *children;
+  for (children = obj->children; children != NULL; children = g_list_next(children)) {
+    DiaObject *child = children->data;
+    (*function)(child);
+    parent_apply_to_children(child, function);
+  }
 }
