@@ -26,6 +26,8 @@
 #include <gmodule.h>
 #include "diatypes.h"
 
+G_BEGIN_DECLS
+
 /*
  * The api version to ensure dia core and the plug-ins agree on
  * talking about the same thing. If some incompatible change is 
@@ -46,7 +48,7 @@
  * The list is by no means complete. If in doubt about your change
  * please ask on dia-list or alternative increment ;-)      --hb
  */
-#define DIA_PLUGIN_API_VERSION 5
+#define DIA_PLUGIN_API_VERSION 8
 
 typedef enum {
   DIA_PLUGIN_INIT_OK,
@@ -103,4 +105,6 @@ g_module_check_init(GModule *gmodule) \
 /* prototype for plugin init function (should be implemented by plugin) */
 G_MODULE_EXPORT PluginInitResult dia_plugin_init(PluginInfo *info);
 
+G_END_DECLS
+
 #endif
