@@ -30,6 +30,8 @@
 #include "message.h"
 #include "diarenderer.h"
 #include "diagramdata.h"
+#include "objchange.h"
+#include "textline.h"
 
 static int text_key_event(Focus *focus, guint keysym,
 			  const gchar *str, int strlen,
@@ -57,6 +59,110 @@ struct TextObjectChange {
 
 #define CURSOR_HEIGHT_RATIO 20
 
+/* *** Encapsulation functions for transferring to text_line *** */
+gchar *
+text_get_line(Text *text, int line)
+{
+  return text_line_get_string(text->lines[line]);
+}
+
+/** Raw sets one line to a given text, not copying, not freeing.
+ */
+static void
+text_set_line(Text *text, int line_no, gchar *line)
+{
+  text_line_set_string(text->lines[line_no], line);
+}
+
+/** Set the text of a line, freeing, copying and mallocing as required.
+ * Updates strlen and row_width entries, but not max_width.
+ */
+static void
+text_set_line_text(Text *text, int line_no, gchar *line)
+{
+  text_set_line(text, line_no, line);
+}
+
+/** Delete the line, freeing appropriately and moving stuff up.
+ * This function circumvents the normal free/alloc cycle of 
+ * text_set_line_text. */
+static void
+text_delete_line(Text *text, int line_no)
+{
+  int i;
+
+  g_free(text->lines[line_no]);
+  for (i = line_no; i < text->numlines - 1; i++) {
+    text->lines[i] = text->lines[i+1];
+  }
+  text->numlines -= 1;
+  text->lines = g_realloc(text->lines, sizeof(TextLine *)*text->numlines);
+}
+
+/** Insert a new (empty) line at line_no.
+ * This function circumvents the normal free/alloc cycle of 
+ * text_set_line_text. */
+static void
+text_insert_line(Text *text, int line_no)
+{
+  int i;
+  text->numlines += 1;
+  text->lines = g_realloc(text->lines, sizeof(char *)*text->numlines);
+
+  for (i = text->numlines - 1; i > line_no; i--) {
+    text->lines[i] = text->lines[i - 1];
+  }
+  text->lines[line_no] = text_line_new("", text->font, text->height);;
+}
+
+/** Get the in-diagram width of the given line.
+ * @param text The text object;
+ * @param line_no The index of the line in the text object, starting at 0.
+ * @returns The width in cm of the indicated line.
+ */
+real
+text_get_line_width(Text *text, int line_no)
+{
+  return text_line_get_width(text->lines[line_no]);
+}
+
+/** Get the number of characters of the given line.
+ * @param text The text object;
+ * @param line_no The index of the line in the text object, starting at 0.
+ * @returns The number of UTF-8 characters of the indicated line.
+ */
+int
+text_get_line_strlen(Text *text, int line_no)
+{
+  return g_utf8_strlen(text_line_get_string(text->lines[line_no]), -1);
+}
+
+real
+text_get_max_width(Text *text)
+{
+  return text->max_width;
+}
+
+/** Get the *average* ascent of this Text object.
+ * @param a Text object
+ * @returns the average of the ascents of each line (height above baseline)
+ */
+real
+text_get_ascent(Text *text)
+{
+  return text->ascent;
+}
+
+/** Get the *average* descent of this Text object.
+ * @param a Text object
+ * @returns the average of the descents of each line (height below baseline)
+ */
+real
+text_get_descent(Text *text)
+{
+  return text->descent;
+}
+
 static ObjectChange *text_create_change(Text *text, enum change_type type,
 					gunichar ch, int pos, int row);
 
@@ -67,10 +173,8 @@ calc_width(Text *text)
   int i;
 
   width = 0.0;
-  for (i=0;i<text->numlines;i++) {
-    text->row_width[i] =
-      dia_font_string_width(text->line[i], text->font, text->height);
-    width = MAX(width, text->row_width[i]);
+  for (i = 0; i < text->numlines; i++) {
+    width = MAX(width, text_get_line_width(text, i));
   }
   
   text->max_width = width;
@@ -83,8 +187,8 @@ calc_ascent_descent(Text *text)
   guint i;
     
   for ( i = 0; i < text->numlines; i++) {
-      sig_a += dia_font_ascent(text->line[i], text->font, text->height);
-      sig_d += dia_font_descent(text->line[i], text->font, text->height);
+    sig_a += text_line_get_ascent(text->lines[i]);
+    sig_d += text_line_get_descent(text->lines[i]);
   }
   
   text->ascent = sig_a / (real)text->numlines;
@@ -97,88 +201,73 @@ free_string(Text *text)
   int i;
 
   for (i=0;i<text->numlines;i++) {
-    g_free(text->line[i]);
+    text_line_destroy(text->lines[i]);
   }
 
-  g_free(text->line);
-  text->line = NULL;
-  
-  g_free(text->strlen);
-  text->strlen = NULL;
-
-  g_free(text->alloclen);
-  text->alloclen = NULL;
-
-  g_free(text->row_width);
-  text->row_width = NULL;
+  g_free(text->lines);
+  text->lines = NULL;
 }
 
 static void
 set_string(Text *text, const char *string)
 {
   int numlines, i;
-  int alloclen;
   const char *s,*s2;
   
   s = string;
   
   numlines = 1;
   if (s != NULL) 
-    while ( (s = g_utf8_strchr(s,-1,'\n')) != NULL ) {
-      s++;
-      if ((*s) != 0) {
-        numlines++;
+    while ( (s = g_utf8_strchr(s, -1, '\n')) != NULL ) {
+      numlines++;
+      if (*s) {
+	s = g_utf8_next_char(s);
       }
     }
   text->numlines = numlines;
-  text->line = g_malloc(sizeof(char *)*numlines);
-  text->strlen = g_malloc(sizeof(int)*numlines);
-  text->alloclen = g_malloc(sizeof(int)*numlines);
-  text->row_width = g_malloc(sizeof(real)*numlines);
+  text->lines = g_new0(TextLine *, numlines);
+  for (i = 0; i < numlines; i++) {
+    text->lines[i] = text_line_new("", text->font, text->height);
+  }
 
   s = string;
 
-  if (string==NULL) {
-    text->line[0] = (char *)g_malloc(1);
-    text->line[0][0] = 0;
-    text->strlen[0] = 0;
-    text->alloclen[0] = 1;
+  if (string == NULL) {
+    text_set_line_text(text, 0, "");
     return;
   }
 
-  for (i=0;i<numlines;i++) {
-    s2 = g_utf8_strchr(s,-1,'\n');
-    if (s2==NULL) {
-      alloclen = strlen(s);
-    } else {
-      alloclen = s2 - s;
+  for (i = 0; i < numlines; i++) {
+    gchar *string_line;
+    s2 = g_utf8_strchr(s, -1, '\n');
+    if (s2 == NULL) { /* No newline */
+      s2 = s + strlen(s);
+    }
+    string_line = g_strndup(s, s2 - s);
+    text_set_line_text(text, i, string_line);
+    g_free(string_line);
+    s = s2;
+    if (*s) {
+      s = g_utf8_next_char(s);
     }
-    text->line[i] = (char *)g_malloc(alloclen+1);
-    text->alloclen[i] = alloclen+1;
-    strncpy (text->line[i], s, alloclen);
-    text->line[i][alloclen] = 0; /* end line with \0 */
-    text->strlen[i] = g_utf8_strlen(text->line[i], -1);;
-    s = s2+1;
   }
 
   if (text->cursor_row >= text->numlines) {
     text->cursor_row = text->numlines - 1;
   }
   
-  if (text->cursor_pos > text->strlen[text->cursor_row]) {
-    text->cursor_pos = text->strlen[text->cursor_row];
+  if (text->cursor_pos > text_get_line_strlen(text, text->cursor_row)) {
+    text->cursor_pos = text_get_line_strlen(text, text->cursor_row);
   }
 }
 
 void
 text_set_string(Text *text, const char *string)
 {
-  if (text->line != NULL)
+  if (text->lines != NULL)
     free_string(text);
   
   set_string(text, string);
-
-  calc_width(text);
 }
 
 Text *
@@ -206,7 +295,6 @@ new_text(const char *string, DiaFont *font, real height,
   
   set_string(text, string);
 
-  calc_width(text);
   calc_ascent_descent(text);
 
   return text;
@@ -220,17 +308,7 @@ text_copy(Text *text)
 
   copy = g_new(Text, 1);
   copy->numlines = text->numlines;
-  copy->line = g_malloc(sizeof(char *)*text->numlines);
-  copy->strlen = g_malloc(sizeof(int)*copy->numlines);
-  copy->alloclen = g_malloc(sizeof(int)*copy->numlines);
-  copy->row_width = g_malloc(sizeof(real)*copy->numlines);
-  
-  for (i=0;i<text->numlines;i++) {
-    copy->line[i] = (char *)g_malloc(text->alloclen[i]+1);
-    strcpy(copy->line[i], text->line[i]);
-    copy->strlen[i] = text->strlen[i];
-    copy->alloclen[i] = text->alloclen[i];
-  }
+  copy->lines = g_new(TextLine *, text->numlines);
   
   copy->font = dia_font_ref(text->font);
   copy->height = text->height;
@@ -238,6 +316,13 @@ text_copy(Text *text)
   copy->color = text->color;
   copy->alignment = text->alignment;
 
+  for (i=0;i<text->numlines;i++) {
+    TextLine *text_line = text->lines[i];
+    copy->lines[i] = text_line_new(text_line_get_string(text_line),
+				   text_line_get_font(text_line),
+				   text_line_get_height(text_line));
+  }
+
   copy->cursor_pos = 0;
   copy->cursor_row = 0;
   copy->focus.obj = NULL;
@@ -248,9 +333,6 @@ text_copy(Text *text)
   copy->ascent = text->ascent;
   copy->descent = text->descent;
   copy->max_width = text->max_width;
-  for (i=0;i<text->numlines;i++) {
-    copy->row_width[i] = text->row_width[i];
-  }
   
   return copy;
 }
@@ -266,8 +348,11 @@ text_destroy(Text *text)
 void
 text_set_height(Text *text, real height)
 {
+  int i;
   text->height = height;
-
+  for (i = 0; i < text->numlines; i++) {
+    text_line_set_height(text->lines[i], height);
+  }
   calc_width(text);
   calc_ascent_descent(text);
 }
@@ -276,8 +361,14 @@ void
 text_set_font(Text *text, DiaFont *font)
 {
   DiaFont *old_font = text->font;
+  int i;
+
   text->font = dia_font_ref(font);
   dia_font_unref(old_font);
+
+  for (i = 0; i < text->numlines; i++) {
+    text_line_set_font(text->lines[i], font);
+  }
   
   calc_width(text);
   calc_ascent_descent(text);
@@ -345,8 +436,6 @@ text_calc_boundingbox(Text *text, Rectangle *box)
   }
 }
 
-
-
 char *
 text_get_string_copy(Text *text)
 {
@@ -355,7 +444,8 @@ text_get_string_copy(Text *text)
   
   num = 0;
   for (i=0;i<text->numlines;i++) {
-    num += strlen(text->line[i])+1;
+    /* This is for allocation, so it should not use g_utf8_strlen() */
+    num += strlen(text_get_line(text, i))+1; 
   }
 
   str = g_malloc(num);
@@ -363,7 +453,7 @@ text_get_string_copy(Text *text)
   *str = 0;
   
   for (i=0;i<text->numlines;i++) {
-    strcat(str, text->line[i]);
+    strcat(str, text_get_line(text, i));
     if (i != (text->numlines-1)) {
       strcat(str, "\n");
     }
@@ -398,13 +488,13 @@ text_distance_from(Text *text, Point *point)
   case ALIGN_LEFT:
     break;
   case ALIGN_CENTER:
-    left -= text->row_width[line] / 2.0;
+    left -= text_get_line_width(text, line) / 2.0;
     break;
   case ALIGN_RIGHT:
-    left -= text->row_width[line];
+    left -= text_get_line_width(text, line);
     break;
   }
-  right = left + text->row_width[line];
+  right = left + text_get_line_width(text, line);
 
   if (point->x <= left) {
     dx = left - point->x;
@@ -430,14 +520,16 @@ text_draw(Text *text, DiaRenderer *renderer)
     real height = text->ascent+text->descent;
     curs_y = text->position.y - text->ascent + text->cursor_row*text->height; 
 
+    DIA_RENDERER_GET_CLASS(renderer)->set_font(renderer, text->font, text->height);
+
     str_width_first =
       DIA_RENDERER_GET_CLASS(renderer)->get_text_width(renderer,
-						text->line[text->cursor_row],
-						text->cursor_pos);
+						       text_get_line(text, text->cursor_row),
+						       text->cursor_pos);
     str_width_whole =
       DIA_RENDERER_GET_CLASS(renderer)->get_text_width(renderer,
-						text->line[text->cursor_row],
-						text->strlen[text->cursor_row]);
+						       text_get_line(text, text->cursor_row),
+						       text_get_line_strlen(text, text->cursor_row));
     curs_x = text->position.x + str_width_first;
 
     switch (text->alignment) {
@@ -462,29 +554,6 @@ text_draw(Text *text, DiaRenderer *renderer)
   }
 }
 
-/** Register that an object has a text that can be edited.
- *  This text will be inserted in the chain traversed by tabbing.
- *  The registration is temporary, intended to be done when the 
- *  object is selected.
- */
-void
-text_register_editable(Text *text, DiaObject *obj)
-{
-  DiagramData *dia;
-
-  text->parent_object = obj;
-  dia = text->parent_object->parent_layer->parent_diagram;
-  dia->text_edits = g_list_append(dia->text_edits, text);
-}
-
-/** Reset the list of editable texts. */
-void
-text_reset_editable(DiagramData *dia)
-{
-  g_list_free(dia->text_edits);
-  dia->text_edits = NULL;
-}
-
 void
 text_grab_focus(Text *text, DiaObject *object)
 {
@@ -496,9 +565,11 @@ void
 text_set_cursor_at_end( Text* text )
 {
   text->cursor_row = text->numlines - 1 ;
-  text->cursor_pos = text->strlen[text->cursor_row] ;
+  text->cursor_pos = text_get_line_strlen(text, text->cursor_row) ;
 }
 
+/* The renderer is only used to determine where the click is, so is not
+ * required when no point is given. */
 void
 text_set_cursor(Text *text, Point *clicked_point,
 		DiaRenderer *renderer)
@@ -510,110 +581,80 @@ text_set_cursor(Text *text, Point *clicked_point,
   int row;
   int i;
 
-  top = text->position.y - text->ascent;
+  if (clicked_point != NULL) {
+    top = text->position.y - text->ascent;
   
-  row = (int)floor((clicked_point->y - top) / text->height);
+    row = (int)floor((clicked_point->y - top) / text->height);
 
-  if (row < 0)
-    row = 0;
+    if (row < 0)
+      row = 0;
 
-  if (row >= text->numlines)
-    row = text->numlines - 1;
+    if (row >= text->numlines)
+      row = text->numlines - 1;
     
-  text->cursor_row = row;
-  text->cursor_pos = 0;
+    text->cursor_row = row;
+    text->cursor_pos = 0;
 
-  if (!renderer->is_interactive) {
-    message_error("Internal error: Select gives non interactive renderer!\n"
-		  "val: %d\n", renderer->is_interactive);
-    return;
-  }
-
-  DIA_RENDERER_GET_CLASS(renderer)->set_font(renderer, text->font, text->height);
-  str_width_whole =
-    DIA_RENDERER_GET_CLASS(renderer)->get_text_width(renderer,
-					      text->line[row],
-					      text->strlen[row]);
-  start_x = text->position.x;
-  switch (text->alignment) {
-  case ALIGN_LEFT:
-    break;
-  case ALIGN_CENTER:
-    start_x -= str_width_whole / 2.0;
-    break;
-  case ALIGN_RIGHT:
-    start_x -= str_width_whole;
-    break;
-  }
-
-  /* Do an ugly linear search for the cursor index:
-     TODO: Change to binary search */
-  
-  for (i=0;i<=text->strlen[row];i++) {
-    str_width_first =
-      DIA_RENDERER_GET_CLASS(renderer)->get_text_width(renderer, text->line[row], i);
-    if (clicked_point->x - start_x >= str_width_first) {
-      text->cursor_pos = i;
-    } else {
+    if (!renderer->is_interactive) {
+      message_error("Internal error: Select gives non interactive renderer!\n"
+		    "val: %d\n", renderer->is_interactive);
       return;
     }
+
+
+    DIA_RENDERER_GET_CLASS(renderer)->set_font(renderer, text->font, text->height);
+    str_width_whole =
+      DIA_RENDERER_GET_CLASS(renderer)->get_text_width(renderer,
+						       text_get_line(text, row),
+						       text_get_line_strlen(text, row));
+    start_x = text->position.x;
+    switch (text->alignment) {
+    case ALIGN_LEFT:
+      break;
+    case ALIGN_CENTER:
+      start_x -= str_width_whole / 2.0;
+      break;
+    case ALIGN_RIGHT:
+      start_x -= str_width_whole;
+      break;
+    }
+
+    /* Do an ugly linear search for the cursor index:
+       TODO: Change to binary search */
+  
+    for (i=0;i<=text_get_line_strlen(text, row);i++) {
+      str_width_first =
+	DIA_RENDERER_GET_CLASS(renderer)->get_text_width(renderer, text_get_line(text, row), i);
+      if (clicked_point->x - start_x >= str_width_first) {
+	text->cursor_pos = i;
+      } else {
+	return;
+      }
+    }
+    text->cursor_pos = text_get_line_strlen(text, row);
+  } else {
+    /* No clicked point, leave cursor where it is */
   }
-  text->cursor_pos = text->strlen[row];
 }
 
 static void
 text_join_lines(Text *text, int first_line)
 {
-  int i;
-  int len1, len2, alloclen1, alloclen2;
-  real width;
-  char *str1, *str2;
-  int numlines;
-
-  str1 = text->line[first_line];
-  str2 = text->line[first_line+1];
-  len1 = text->strlen[first_line];
-  len2 = text->strlen[first_line + 1];
-  alloclen1 = text->alloclen[first_line];
-  alloclen2 = text->alloclen[first_line + 1];
-
-  text->line[first_line] = NULL;
-  text->line[first_line+1] = NULL;
-
-  for (i=first_line+1;i<text->numlines-1;i++) {
-    text->line[i] = text->line[i+1];
-    text->strlen[i] = text->strlen[i+1];
-    text->alloclen[i] = text->alloclen[i+1];
-    text->row_width[i] = text->row_width[i+1];
-  }
-  text->strlen[first_line] = len1 + len2;
-  text->alloclen[first_line] = alloclen1 + alloclen2;
-  text->line[first_line] =
-    g_malloc(sizeof(char)*(text->alloclen[first_line]));
-  strcpy(text->line[first_line], str1);
-  strcat(text->line[first_line], str2);
-  g_free(str1);
-  g_free(str2);
+  gchar *combined_line;
+  int len1;
   
-  text->numlines -= 1;
-  numlines = text->numlines;
-  text->line = g_realloc(text->line, sizeof(char *)*numlines);
-  text->strlen = g_realloc(text->strlen, sizeof(int)*numlines);
-  text->alloclen = g_realloc(text->alloclen, sizeof(int)*numlines);
-  text->row_width = g_realloc(text->row_width, sizeof(real)*numlines);
+  len1 = text_get_line_strlen(text, first_line);
 
-  text->row_width[first_line] = 
-    dia_font_string_width(text->line[first_line], text->font, text->height);
+  combined_line = g_strconcat(text_get_line(text, first_line), 
+			      text_get_line(text, first_line + 1), NULL);
+  text_delete_line(text, first_line);
+  text_set_line_text(text, first_line, combined_line);
+  g_free(combined_line);
 
-  width = 0.0;
-  for (i=0;i<text->numlines;i++) {
-    width = MAX(width, text->row_width[i]);
-  }
-  text->max_width = width;
+  text->max_width = MAX(text->max_width, text_get_line_width(text, first_line));
 
   text->cursor_row = first_line;
   text->cursor_pos = len1;
-  
 }
 
 static void
@@ -622,36 +663,33 @@ text_delete_forward(Text *text)
   int row;
   int i;
   real width;
-  gchar *start, *end;
-  int len;
+  gchar *line;
+  gchar *utf8_before, *utf8_after;
+  gchar *str1, *str;
   
   row = text->cursor_row;
   
-  if (text->cursor_pos >= text->strlen[row]) {
-    if (row+1 < text->numlines)
+  if (text->cursor_pos >= text_get_line_strlen(text, row)) {
+    if (row + 1 < text->numlines)
       text_join_lines(text, row);
     return;
   }
-  start = text->line[row];
-  for (i = 0; i < text->cursor_pos; i++) {
-    start = g_utf8_next_char (start);
-  }
-  end = g_utf8_next_char (start);
 
-  len = strlen (text->line[row]);
-  memmove (start, end, text->line[row] + len - start);
+  line = text_get_line(text, row);
+  utf8_before = g_utf8_offset_to_pointer(line, (glong)(text->cursor_pos));
+  utf8_after = g_utf8_offset_to_pointer(utf8_before, 1);
+  str1 = g_strndup(line, utf8_before - line);
+  str = g_strconcat(str1, utf8_after, NULL);
+  text_set_line_text(text, row, str);
+  g_free(str1);
+  g_free(str);
 
-  text->strlen[row] = g_utf8_strlen(text->line[row], -1);/*text->strlen[row]--;*/
-  
-  if (text->cursor_pos > text->strlen[text->cursor_row])
-    text->cursor_pos = text->strlen[text->cursor_row];
+  if (text->cursor_pos > text_get_line_strlen(text, text->cursor_row))
+    text->cursor_pos = text_get_line_strlen(text, text->cursor_row);
 
-  text->row_width[row] = dia_font_string_width(text->line[row],
-                                              text->font,
-                                              text->height);
   width = 0.0;
-  for (i=0;i<text->numlines;i++) {
-    width = MAX(width, text->row_width[i]);
+  for (i = 0; i < text->numlines; i++) {
+    width = MAX(width, text_get_line_width(text, i));
   }
   text->max_width = width;
 }
@@ -662,8 +700,9 @@ text_delete_backward(Text *text)
   int row;
   int i;
   real width;
-  gchar *start, *end;
-  int len;
+  gchar *line;
+  gchar *utf8_before, *utf8_after;
+  gchar *str1, *str;
   
   row = text->cursor_row;
   
@@ -672,26 +711,23 @@ text_delete_backward(Text *text)
       text_join_lines(text, row-1);
     return;
   }
-  start = g_utf8_offset_to_pointer(text->line[row], 
-				   (glong)(text->cursor_pos-1));
-  end = g_utf8_offset_to_pointer(start, 1);
-  len = g_utf8_offset_to_pointer(text->line[row], text->strlen[row]) - end;
-  memmove (start, end, len + 1);
 
-  text->strlen[row] = g_utf8_strlen(text->line[row], -1);
+  line = text_get_line(text, row);
+  utf8_before = g_utf8_offset_to_pointer(line, (glong)(text->cursor_pos - 1));
+  utf8_after = g_utf8_offset_to_pointer(utf8_before, 1);
+  str1 = g_strndup(line, utf8_before - line);
+  str = g_strconcat(str1, utf8_after, NULL);
+  text_set_line_text(text, row, str);
+  g_free(str);
+  g_free(str1);
 
   text->cursor_pos --;
-  
-  if (text->cursor_pos > text->strlen[text->cursor_row])
-    text->cursor_pos = text->strlen[text->cursor_row];
-
-  text->row_width[row] = dia_font_string_width(text->line[row],
-                                              text->font,
-                                              text->height);
+  if (text->cursor_pos > text_get_line_strlen(text, text->cursor_row))
+    text->cursor_pos = text_get_line_strlen(text, text->cursor_row);
 
   width = 0.0;
-  for (i=0;i<text->numlines;i++) {
-    width = MAX(width, text->row_width[i]);
+  for (i = 0; i < text->numlines; i++) {
+    width = MAX(width, text_get_line_width(text, i));
   }
   text->max_width = width;
 }
@@ -700,100 +736,59 @@ static void
 text_split_line(Text *text)
 {
   int i;
-  int row;
-  int numlines;
   char *line;
-  int orig_len;
   real width;
-  gchar *str;
-  int orig_alloclen;
+  gchar *utf8_before;
+  gchar *str1, *str2;
   
-  text->numlines += 1;
-  numlines = text->numlines;
-  text->line = g_realloc(text->line, sizeof(char *)*numlines);
-  text->strlen = g_realloc(text->strlen, sizeof(int)*numlines);
-  text->alloclen = g_realloc(text->alloclen, sizeof(int)*numlines);
-  text->row_width = g_realloc(text->row_width, sizeof(real)*numlines);
+  /* Split the lines at cursor_pos */
+  line = text_get_line(text, text->cursor_row);
+  text_insert_line(text, text->cursor_row);
+  utf8_before = g_utf8_offset_to_pointer(line, (glong)(text->cursor_pos));
+  str1 = g_strndup(line, utf8_before - line);
+  str2 = g_strdup(utf8_before); /* Must copy before dealloc */
+  text_set_line_text(text, text->cursor_row, str1);
+  text_set_line_text(text, text->cursor_row + 1, str2);
+  g_free(str2);
+  g_free(str1);
 
-  row = text->cursor_row;
-  for (i=text->numlines-1;i>row+1;i--) {
-    text->line[i] = text->line[i-1];
-    text->strlen[i] = text->strlen[i-1];
-    text->alloclen[i] = text->alloclen[i-1];
-    text->row_width[i] = text->row_width[i-1];
-  }
-  /* row and row+1 needs to be changed: */
-  line = text->line[row];
-  orig_len = text->strlen[row];
-  orig_alloclen = text->alloclen[row];
-  
-  text->strlen[row] = text->cursor_pos;
-  str = text->line[row];
-  for (i = 0; i < text->cursor_pos; i++) {
-    str = g_utf8_next_char (str);
-  }
-  text->alloclen[row] = str - text->line[row] + 1;
-  text->line[row] = g_strndup (line, str - text->line[row]);
-  
-  text->strlen[row + 1] = orig_len - text->strlen[row];
-  text->alloclen[row + 1] = orig_alloclen - strlen (text->line[row]) + 1;
-  text->line[row + 1] = g_strndup (str, text->alloclen[row + 1] - 1);
-
-  g_free(line);
-
-  text->row_width[row] = 
-    dia_font_string_width(text->line[row], text->font, text->height);
-  text->row_width[row+1] = 
-    dia_font_string_width(text->line[row+1], text->font, text->height);
+  text->cursor_row ++;
+  text->cursor_pos = 0;
 
   width = 0.0;
   for (i=0;i<text->numlines;i++) {
-    width = MAX(width, text->row_width[i]);
+    width = MAX(width, text_get_line_width(text, i));
   }
   text->max_width = width;
-
-  text->cursor_row += 1;
-  text->cursor_pos = 0;
 }
 
 static void
 text_insert_char(Text *text, gunichar c)
 {
-  int row;
-  int i;
-  gchar *line, *str;
   gchar ch[7];
-  int unilen, length;
+  int unilen;
+  int row;
+  gchar *line, *str;
+  gchar *utf8_before;
+  gchar *str1;
 
+  /* Make a string of the the char */
   unilen = g_unichar_to_utf8 (c, ch);
   ch[unilen] = 0;
   
   row = text->cursor_row;
 
-  length = strlen (text->line[row]);
-  if ((length + unilen + 1) > text->alloclen[row]) {
-    text->alloclen[row] = length * 2 + unilen + 1;
-    text->line[row] = g_realloc (text->line[row], sizeof (gchar) * text->alloclen[row]);
-  }
+  /* Copy the before and after parts with the new char in between */
+  line = text_get_line(text, row);
+  utf8_before = g_utf8_offset_to_pointer(line, (glong)(text->cursor_pos));
+  str1 = g_strndup(line, utf8_before - line);
+  str = g_strconcat(str1, ch, utf8_before, NULL);
+  text_set_line_text(text, row, str);
+  g_free(str);
+  g_free(str1);
 
-  str = text->line[row];
-  for (i = 0; i < text->cursor_pos; i++) {
-    str = g_utf8_next_char (str);
-  }
-
-  line = text->line[row];
-  /* copy the part to the right of the insertion */
-  for (i = length; &line[i] >= str; i--) {
-    line[i + unilen] = line[i];
-  }
-  strncpy (str, ch, unilen);
-  line[length + unilen] = 0; /* null terminate */
-  text->cursor_pos += 1;
-  text->strlen[row] = g_utf8_strlen(text->line[row], -1);/* length + unilen; */
-
-  text->row_width[row] =
-      dia_font_string_width(text->line[row], text->font, text->height);
-  text->max_width = MAX(text->max_width, text->row_width[row]);
+  text->cursor_pos++;
+  text->max_width = MAX(text->max_width, text_get_line_width(text, row));
 }
 
 static int
@@ -816,8 +811,8 @@ text_key_event(Focus *focus, guint keyval, const gchar *str, int strlen,
         if (text->cursor_row<0)
           text->cursor_row = 0;
 
-        if (text->cursor_pos > text->strlen[text->cursor_row])
-          text->cursor_pos = text->strlen[text->cursor_row];
+        if (text->cursor_pos > text_get_line_strlen(text, text->cursor_row))
+          text->cursor_pos = text_get_line_strlen(text, text->cursor_row);
 
         break;
       case GDK_Down:
@@ -825,8 +820,8 @@ text_key_event(Focus *focus, guint keyval, const gchar *str, int strlen,
         if (text->cursor_row >= text->numlines)
           text->cursor_row = text->numlines - 1;
 
-        if (text->cursor_pos > text->strlen[text->cursor_row])
-          text->cursor_pos = text->strlen[text->cursor_row];
+        if (text->cursor_pos > text_get_line_strlen(text, text->cursor_row))
+          text->cursor_pos = text_get_line_strlen(text, text->cursor_row);
     
         break;
       case GDK_Left:
@@ -836,19 +831,19 @@ text_key_event(Focus *focus, guint keyval, const gchar *str, int strlen,
         break;
       case GDK_Right:
         text->cursor_pos++;
-        if (text->cursor_pos > text->strlen[text->cursor_row])
-          text->cursor_pos = text->strlen[text->cursor_row];
+        if (text->cursor_pos > text_get_line_strlen(text, text->cursor_row))
+          text->cursor_pos = text_get_line_strlen(text, text->cursor_row);
         break;
       case GDK_Home:
         text->cursor_pos = 0;
         break;
       case GDK_End:
-        text->cursor_pos = text->strlen[text->cursor_row];
+        text->cursor_pos = text_get_line_strlen(text, text->cursor_row);
         break;
       case GDK_Delete:
         return_val = TRUE;
         row = text->cursor_row;
-        if (text->cursor_pos >= text->strlen[row]) {
+        if (text->cursor_pos >= text_get_line_strlen(text, row)) {
           if (row+1 < text->numlines) {
             *change = text_create_change(text, TYPE_JOIN_ROW, 'Q',
                                          text->cursor_pos, row);
@@ -857,7 +852,7 @@ text_key_event(Focus *focus, guint keyval, const gchar *str, int strlen,
             break;
           }
         } else {
-          utf = text->line[row];
+          utf = text_get_line(text, row);
           for (i = 0; i < text->cursor_pos; i++)
             utf = g_utf8_next_char (utf);
           c = g_utf8_get_char (utf);
@@ -872,13 +867,13 @@ text_key_event(Focus *focus, guint keyval, const gchar *str, int strlen,
         if (text->cursor_pos <= 0) {
           if (row > 0) {
             *change = text_create_change(text, TYPE_JOIN_ROW, 'Q',
-                                         text->strlen[row-1], row-1);
+                                         text_get_line_strlen(text, row-1), row-1);
           } else {
             return_val = FALSE;
             break;
           }
         } else {
-          utf = text->line[row];
+          utf = text_get_line(text, row);
           for (i = 0; i < (text->cursor_pos - 1); i++)
             utf = g_utf8_next_char (utf);
           c = g_utf8_get_char (utf);
@@ -889,6 +884,7 @@ text_key_event(Focus *focus, guint keyval, const gchar *str, int strlen,
         text_delete_backward(text);
         break;
       case GDK_Return:
+      case GDK_KP_Enter:
         return_val = TRUE;
         *change = text_create_change(text, TYPE_SPLIT_ROW, 'Q',
                                      text->cursor_pos, text->cursor_row);
@@ -916,8 +912,8 @@ text_key_event(Focus *focus, guint keyval, const gchar *str, int strlen,
 int text_is_empty(Text *text)
 {
   int i;
-  for (i=0;i<text->numlines;i++) {
-    if (text->strlen[i]!=0) {
+  for (i = 0; i < text->numlines; i++) {
+    if (text_get_line_strlen(text, i) != 0) {
       return FALSE;
     }
   }
@@ -966,7 +962,7 @@ data_add_text(AttributeNode attr, Text *text)
 Text *
 data_text(AttributeNode text_attr)
 {
-  char *string = "";
+  char *string = NULL;
   DiaFont *font;
   real height;
   Point pos = {0.0, 0.0};
@@ -1008,7 +1004,7 @@ data_text(AttributeNode text_attr)
   if (attr != NULL)
     align = data_enum(attribute_first_data(attr));
   
-  text = new_text(string, font, height, &pos, &col, align);
+  text = new_text(string ? string : "", font, height, &pos, &col, align);
   if (font) dia_font_unref(font);
   if (string) g_free(string);
   return text;
@@ -1031,11 +1027,9 @@ void
 text_set_attributes(Text *text, TextAttributes *attr)
 {
   if (text->font != attr->font) {
-    DiaFont *old_font = text->font;
-    text->font = dia_font_ref(attr->font);
-    dia_font_unref(old_font);
+    text_set_font(text, attr->font);
   }
-  text->height = attr->height;
+  text_set_height(text, attr->height);
   text->position = attr->position;
   text->color = attr->color;
   text->alignment = attr->alignment;
