@@ -30,16 +30,16 @@ typedef enum {
 #include "focus.h"
 #include "properties.h"
 
+#define USE_TEXTLINE_FOR_LINES
+
 struct _Text {
   /** The object that owns this text. */
   DiaObject *parent_object;
   /* don't change these values directly, use the text_set* functions */
   
   /* Text data: */
-  gchar **line;
   int numlines;
-  int *strlen;  /* in characters */
-  int *alloclen; /* in bytes */
+  TextLine **lines;
 
   /* Attributes: */
   DiaFont *font;
@@ -57,7 +57,6 @@ struct _Text {
   real ascent; /* **average** ascent */
   real descent; /* **average** descent */
   real max_width;
-  real *row_width;
 };
 
 
@@ -66,6 +65,7 @@ Text *new_text(const char *string, DiaFont *font, real height,
 	       Point *pos, Color *color, Alignment align);
 void text_destroy(Text *text);
 Text *text_copy(Text *text);
+gchar *text_get_line(Text *text, int line);
 char *text_get_string_copy(Text *text);
 void text_set_string(Text *text, const char *string);
 void text_set_height(Text *text, real height);
@@ -85,6 +85,12 @@ int text_delete_all(Text *text, ObjectChange **change);
 void text_get_attributes(Text *text, TextAttributes *attr);
 void text_set_attributes(Text *text, TextAttributes *attr);
 
+real text_get_line_width(Text *text, int line_no);
+int text_get_line_strlen(Text *text, int line_no);
+real text_get_max_width(Text *text);
+real text_get_ascent(Text *text);
+real text_get_descent(Text *text);
+
 void data_add_text(AttributeNode attr, Text *text);
 Text *data_text(AttributeNode attr);
 
