@@ -91,8 +91,6 @@ Color orthflow_color_signal   = { 0.0f, 0.0f, 1.0f };
 #define ORTHFLOW_ARROWWIDTH 0.5
 #define HANDLE_MOVE_TEXT (HANDLE_CUSTOM2)
 
-static DiaFont *orthflow_font = NULL;
-
 static ObjectChange* orthflow_move_handle(Orthflow *orthflow, Handle *handle,
 					  Point *to, ConnectionPoint *cp,
 					  HandleMoveReason reason, 
@@ -134,7 +132,8 @@ static ObjectTypeOps orthflow_type_ops =
 DiaObjectType orthflow_type =
 {
   "FS - Orthflow",		/* name */
-  0,				/* version */
+  /* Version 0 had no autorouting and so shouldn't have it set by default. */
+  1,                      /* version */
   (char **) orthflow_xpm,	/* pixmap */
   &orthflow_type_ops		/* ops */
 };
@@ -387,15 +386,6 @@ orthflow_draw(Orthflow *orthflow, DiaRenderer *renderer)
 					  ORTHFLOW_WIDTH,
 					  render_color,
 					  NULL, &arrow); 
-#if 0
-  arrow_draw(renderer, 
-	     &points[n-1], &points[n-2],
-	     ORTHFLOW_ARROWLEN, ORTHFLOW_ARROWWIDTH, 
-	     render_color, &color_white);
-#endif
-
-  renderer_ops->set_font(renderer, orthflow_font,
-                          ORTHFLOW_FONTHEIGHT);
 
   text_draw(orthflow->text, renderer);
 }
@@ -440,11 +430,6 @@ orthflow_create(Point *startpoint,
   } else {
     Color* color = &orthflow_color_signal;
 
-    if (orthflow_font == NULL) {
-	    orthflow_font = dia_font_new_from_style(DIA_FONT_SANS|DIA_FONT_ITALIC,
-                                              1.0);
-    }
-
     switch (orthflow->type) {
     case ORTHFLOW_ENERGY:
       color = &orthflow_color_energy ;
@@ -457,8 +442,6 @@ orthflow_create(Point *startpoint,
       break ;
     }
 
-    orthflow->text = new_text("", orthflow_font, ORTHFLOW_FONTHEIGHT, 
-                              &p, color, ALIGN_CENTER);
   }
 #endif
 
@@ -571,11 +554,6 @@ orthflow_load(ObjectNode obj_node, int version, const char *filename)
   DiaObject *obj;
   PolyBBExtras *extra;
 
-  if (orthflow_font == NULL) {
-    orthflow_font = dia_font_new_from_style(DIA_FONT_SANS|DIA_FONT_ITALIC,
-                                            1.0);
-  }
-
   orthflow = g_malloc0(sizeof(Orthflow));
 
   orth = &orthflow->orth;
