@@ -20,66 +20,37 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 #include <string.h>
+#include <stdlib.h>
 
-#include "config.h"
 #include "intl.h"
 #include "object.h"
 #include "connection.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "handle.h"
 #include "arrows.h"
-#include "lazyprops.h"
+#include "properties.h"
 #include "text.h"
 
 #include "pixmaps/annotation.xpm"
 
-typedef struct _Annotation Annotation;
-typedef struct _AnnotationState AnnotationState;
-typedef struct _AnnotationDialog AnnotationDialog;
-typedef struct _AnnotationDefaultsDialog AnnotationDefaultsDialog;
-typedef struct _AnnotationDefaults AnnotationDefaults;
-
-struct _AnnotationState {
-  ObjectState obj_state;
-
-  TextAttributes text_attrib;
-};
-
-struct _Annotation {
+typedef struct _Annotation {
   Connection connection;
 
   Handle text_handle;
 
   Text *text;
-};
 
-struct _AnnotationDialog {
-  AttributeDialog dialog;
-  Annotation *parent;
-  
-  TextFontAttribute text_font;
-  TextFontHeightAttribute text_fontheight;
-  TextColorAttribute text_color;  
-};
-  
-struct _AnnotationDefaults {
-  Font *text_font;
-  real text_fontheight;
-  Color text_color;
-};
+  TextAttributes attrs; 
+} Annotation;
 
-struct _AnnotationDefaultsDialog {
-  AttributeDialog dialog;
-  AnnotationDefaults *parent;
-  
-  FontAttribute text_font;
-  FontHeightAttribute text_fontheight;
-  TextColorAttribute text_color;  
-};
   
 #define ANNOTATION_LINE_WIDTH 0.05
 #define ANNOTATION_BORDER 0.2
@@ -88,52 +59,38 @@ struct _AnnotationDefaultsDialog {
 
 #define HANDLE_MOVE_TEXT (HANDLE_CUSTOM1)
 
-
-static AnnotationDefaults annotation_defaults;
-static AnnotationDialog *annotation_dialog;
-static AnnotationDefaultsDialog *annotation_defaults_dialog;
-
-static void annotation_move_handle(Annotation *annotation, Handle *handle,
-				   Point *to, HandleMoveReason reason, ModifierKeys modifiers);
-static void annotation_move(Annotation *annotation, Point *to);
+static ObjectChange* annotation_move_handle(Annotation *annotation, Handle *handle,
+					    Point *to, ConnectionPoint *cp,
+					    HandleMoveReason reason, ModifierKeys modifiers);
+static ObjectChange* annotation_move(Annotation *annotation, Point *to);
 static void annotation_select(Annotation *annotation, Point *clicked_point,
-			      Renderer *interactive_renderer);
-static void annotation_draw(Annotation *annotation, Renderer *renderer);
-static Object *annotation_create(Point *startpoint,
+			      DiaRenderer *interactive_renderer);
+static void annotation_draw(Annotation *annotation, DiaRenderer *renderer);
+static DiaObject *annotation_create(Point *startpoint,
 				 void *user_data,
 				 Handle **handle1,
 				 Handle **handle2);
 static real annotation_distance_from(Annotation *annotation, Point *point);
 static void annotation_update_data(Annotation *annotation);
 static void annotation_destroy(Annotation *annotation);
-static Object *annotation_copy(Annotation *annotation);
-static PROPDLG_TYPE annotation_get_properties(Annotation *annotation);
-static ObjectChange *annotation_apply_properties(Annotation *annotation);
-
-static void annotation_init_defaults();
-static PROPDLG_TYPE annotation_get_defaults();
-static void annotation_apply_defaults();
-
-static AnnotationState *annotation_get_state(Annotation *annotation);
-static void annotation_set_state(Annotation *annotation,
-				 AnnotationState *state);
-
-static void annotation_save(Annotation *annotation, ObjectNode obj_node,
-			    const char *filename);
-static Object *annotation_load(ObjectNode obj_node, int version,
+static DiaObject *annotation_load(ObjectNode obj_node, int version,
 			       const char *filename);
-
+static PropDescription *annotation_describe_props(Annotation *annotation);
+static void annotation_get_props(Annotation *annotation, 
+                                 GPtrArray *props);
+static void annotation_set_props(Annotation *annotation, 
+                                 GPtrArray *props);
 
 static ObjectTypeOps annotation_type_ops =
 {
   (CreateFunc) annotation_create,
-  (LoadFunc)   annotation_load,
-  (SaveFunc)   annotation_save,
-  (GetDefaultsFunc)   annotation_get_defaults,
-  (ApplyDefaultsFunc) annotation_apply_defaults
+  (LoadFunc)   annotation_load,/*using_properties*/
+  (SaveFunc)   object_save_using_properties,
+  (GetDefaultsFunc)   NULL,
+  (ApplyDefaultsFunc) NULL
 };
 
-ObjectType sadtannotation_type =
+DiaObjectType sadtannotation_type =
 {
   "SADT - annotation",        /* name */
   0,                         /* version */
@@ -146,14 +103,91 @@ static ObjectOps annotation_ops = {
   (DrawFunc)            annotation_draw,
   (DistanceFunc)        annotation_distance_from,
   (SelectFunc)          annotation_select,
-  (CopyFunc)            annotation_copy,
+  (CopyFunc)            object_copy_using_properties,
   (MoveFunc)            annotation_move,
   (MoveHandleFunc)      annotation_move_handle,
-  (GetPropertiesFunc)   annotation_get_properties,
-  (ApplyPropertiesFunc) annotation_apply_properties,
-  (ObjectMenuFunc)      NULL
+  (GetPropertiesFunc)   object_create_props_dialog,
+  (ApplyPropertiesFunc) object_apply_props_from_dialog,
+  (ObjectMenuFunc)      NULL,
+  (DescribePropsFunc)   annotation_describe_props,
+  (GetPropsFunc)        annotation_get_props,
+  (SetPropsFunc)        annotation_set_props
 };
 
+#undef TEMPORARY_EVENT_TEST
+
+#ifdef TEMPORARY_EVENT_TEST
+static gboolean 
+handle_btn1(Annotation *annotation, Property *prop) {
+  Color col;
+  text_get_attributes(annotation->text,&annotation->attrs);
+  col = annotation->attrs.color;
+  /* g_message("in handle_btn1 for object %p col=%.2f:%.2f:%.2f",
+     annotation,col.red,col.green,col.blue); */
+  col.red = g_random_double();
+  col.green = g_random_double();
+  col.blue = g_random_double();
+  annotation->attrs.color = col;
+  text_set_attributes(annotation->text,&annotation->attrs);
+  /* g_message("end of handle_btn1 for object %p col=%.2f:%.2f:%.2f",
+     annotation,col.red,col.green,col.blue); */
+  return TRUE;
+}
+#endif
+
+static PropDescription annotation_props[] = {
+  CONNECTION_COMMON_PROPERTIES,
+#ifdef TEMPORARY_EVENT_TEST
+  {"btn1", PROP_TYPE_BUTTON, PROP_FLAG_VISIBLE|PROP_FLAG_DONT_SAVE, 
+   NULL, "Click Me !", NULL, 
+   (PropEventHandler)handle_btn1},
+#endif
+  { "text", PROP_TYPE_TEXT, 0,NULL,NULL},
+  PROP_STD_TEXT_ALIGNMENT,
+  PROP_STD_TEXT_FONT,
+  PROP_STD_TEXT_HEIGHT,
+  PROP_STD_TEXT_COLOUR,
+  {"pos", PROP_TYPE_POINT, PROP_FLAG_DONT_SAVE},
+  PROP_DESC_END
+};
+
+static PropDescription *
+annotation_describe_props(Annotation *annotation) 
+{
+  if (annotation_props[0].quark == 0) {
+    prop_desc_list_calculate_quarks(annotation_props);
+  }
+  return annotation_props;
+}    
+
+static PropOffset annotation_offsets[] = {
+  CONNECTION_COMMON_PROPERTIES_OFFSETS,
+  {"text",PROP_TYPE_TEXT,offsetof(Annotation,text)},
+  {"text_alignment",PROP_TYPE_ENUM,offsetof(Annotation,attrs.alignment)},
+  {"text_font",PROP_TYPE_FONT,offsetof(Annotation,attrs.font)},
+  {"text_height",PROP_TYPE_REAL,offsetof(Annotation,attrs.height)},
+  {"text_colour",PROP_TYPE_COLOUR,offsetof(Annotation,attrs.color)},
+  {"pos", PROP_TYPE_POINT, offsetof(Annotation,text_handle.pos)},
+  { NULL,0,0 }
+};
+
+static void
+annotation_get_props(Annotation *annotation, GPtrArray *props)
+{  
+  text_get_attributes(annotation->text,&annotation->attrs);
+  object_get_props_from_offsets(&annotation->connection.object,
+                                annotation_offsets,props);
+}
+
+static void
+annotation_set_props(Annotation *annotation, GPtrArray *props)
+{
+  object_set_props_from_offsets(&annotation->connection.object,
+                                annotation_offsets,props);
+  apply_textattr_properties(props,annotation->text,"text",&annotation->attrs);
+  annotation_update_data(annotation);
+}
+
 static real
 annotation_distance_from(Annotation *annotation, Point *point)
 {
@@ -169,17 +203,18 @@ annotation_distance_from(Annotation *annotation, Point *point)
 
 static void
 annotation_select(Annotation *annotation, Point *clicked_point,
-	    Renderer *interactive_renderer)
+	    DiaRenderer *interactive_renderer)
 {
   text_set_cursor(annotation->text, clicked_point, interactive_renderer);
-  text_grab_focus(annotation->text, (Object *)annotation);
+  text_grab_focus(annotation->text, &annotation->connection.object);
   
   connection_update_handles(&annotation->connection);
 }
 
-static void
+static ObjectChange*
 annotation_move_handle(Annotation *annotation, Handle *handle,
-		 Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		       Point *to, ConnectionPoint *cp,
+		       HandleMoveReason reason, ModifierKeys modifiers)
 {
   Point p1, p2;
   Point *endpoints;
@@ -195,24 +230,26 @@ annotation_move_handle(Annotation *annotation, Handle *handle,
     endpoints = &(conn->endpoints[0]); 
     if (handle->id == HANDLE_MOVE_STARTPOINT) {
       p1 = endpoints[0];
-      connection_move_handle(conn, handle->id, to, reason);
+      connection_move_handle(conn, handle->id, to, cp, reason, modifiers);
       p2 = endpoints[0];
       point_sub(&p2, &p1);
       point_add(&annotation->text->position, &p2);
       point_add(&p2,&(endpoints[1]));
-      connection_move_handle(conn, HANDLE_MOVE_ENDPOINT, &p2, reason);   
+      connection_move_handle(conn, HANDLE_MOVE_ENDPOINT, &p2, NULL, reason, 0);
     } else {      
       p1 = endpoints[1];
-      connection_move_handle(conn, handle->id, to, reason);
+      connection_move_handle(conn, handle->id, to, cp, reason, modifiers);
       p2 = endpoints[1];
       point_sub(&p2, &p1);
       point_add(&annotation->text->position, &p2);
     }
   }
   annotation_update_data(annotation);
+
+  return NULL;
 }
 
-static void
+static ObjectChange*
 annotation_move(Annotation *annotation, Point *to)
 {
   Point start_to_end;
@@ -231,30 +268,34 @@ annotation_move(Annotation *annotation, Point *to)
   point_add(&annotation->text->position, &delta);
   
   annotation_update_data(annotation);
+
+  return NULL;
 }
 
 static void
-annotation_draw(Annotation *annotation, Renderer *renderer)
+annotation_draw(Annotation *annotation, DiaRenderer *renderer)
 {
   Point *endpoints;
   Point vect,rvect,v1,v2;
   Point pts[4];
   real vlen;
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
 
   assert(annotation != NULL);
   assert(renderer != NULL);
 
   endpoints = &annotation->connection.endpoints[0];
   
-  renderer->ops->set_linewidth(renderer, ANNOTATION_LINE_WIDTH);
-  renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
-  renderer->ops->set_linecaps(renderer, LINECAPS_BUTT);
+  renderer_ops->set_linewidth(renderer, ANNOTATION_LINE_WIDTH);
+  renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
+  renderer_ops->set_linecaps(renderer, LINECAPS_BUTT);
 
   vect = annotation->connection.endpoints[1];
   point_sub(&vect,&annotation->connection.endpoints[0]);
   vlen = distance_point_point(&annotation->connection.endpoints[0],
 			      &annotation->connection.endpoints[1]);
   if (vlen > 0.0) {
+    /* draw the squiggle */
     point_scale(&vect,1/vlen);
     rvect.y = vect.x;
     rvect.x = -vect.y;
@@ -274,14 +315,14 @@ annotation_draw(Annotation *annotation, Renderer *renderer)
     point_add(&pts[1],&v1);
     point_sub(&pts[2],&v1);
     pts[3] = annotation->connection.endpoints[1];
-    renderer->ops->draw_polyline(renderer,
+    renderer_ops->draw_polyline(renderer,
 			     pts, sizeof(pts) / sizeof(pts[0]),
 			     &color_black);
   }
   text_draw(annotation->text,renderer);
 }
 
-static Object *
+static DiaObject *
 annotation_create(Point *startpoint,
 		  void *user_data,
 		  Handle **handle1,
@@ -289,31 +330,35 @@ annotation_create(Point *startpoint,
 {
   Annotation *annotation;
   Connection *conn;
-  Object *obj; 
+  LineBBExtras *extra;
+  DiaObject *obj; 
   Point offs;
   Point defaultlen = { 1.0, 1.0 };
+  DiaFont* font;
 
-  annotation_init_defaults();
-  
-  annotation = g_malloc(sizeof(Annotation));
+  annotation = g_malloc0(sizeof(Annotation));
 
   conn = &annotation->connection;
   conn->endpoints[0] = *startpoint;
   conn->endpoints[1] = *startpoint;
   point_add(&conn->endpoints[1], &defaultlen);
  
-  obj = (Object *) annotation;
-  
+  obj = &conn->object;
+  extra = &conn->extra_spacing;
+
   obj->type = &sadtannotation_type;
   obj->ops = &annotation_ops;
   
   connection_init(conn, 3, 0);
 
-  annotation->text = new_text("", annotation_defaults.text_font,
-			      annotation_defaults.text_fontheight,
-			      &conn->endpoints[1],
-			      &annotation_defaults.text_color,
-			      ALIGN_CENTER);
+  font = dia_font_new_from_style(DIA_FONT_SANS,ANNOTATION_FONTHEIGHT);
+  annotation->text = new_text("", font,
+                              ANNOTATION_FONTHEIGHT,
+                              &conn->endpoints[1],
+                              &color_black,
+                              ALIGN_CENTER);
+  dia_font_unref(font);
+
   offs.x = .3 * ANNOTATION_FONTHEIGHT;
   if (conn->endpoints[1].y < conn->endpoints[0].y)
     offs.y = 1.3 * ANNOTATION_FONTHEIGHT;
@@ -326,12 +371,16 @@ annotation_create(Point *startpoint,
   annotation->text_handle.connect_type = HANDLE_NONCONNECTABLE;
   annotation->text_handle.connected_to = NULL;
   obj->handles[2] = &annotation->text_handle;
-  
+
+  extra->start_trans = 
+    extra->end_trans = ANNOTATION_ZLEN;
+  extra->start_long = 
+    extra->end_long = ANNOTATION_LINE_WIDTH/2.0;
   annotation_update_data(annotation);
 
   *handle1 = obj->handles[0];
   *handle2 = obj->handles[1];
-  return (Object *)annotation;
+  return &annotation->connection.object;
 }
 
 
@@ -340,206 +389,36 @@ annotation_destroy(Annotation *annotation)
 {
   connection_destroy(&annotation->connection);
 
-  g_free(annotation->text);
+  text_destroy(annotation->text);
 }
 
-static Object *
-annotation_copy(Annotation *annotation)
-{
-  Annotation *newannotation;
-  Connection *conn, *newconn;
-  Object *newobj;
-  
-  conn = &annotation->connection;
-  
-  newannotation = g_malloc(sizeof(Annotation));
-  newconn = &newannotation->connection;
-  newobj = (Object *) newannotation;
-
-  connection_copy(conn, newconn);
-
-  newannotation->text_handle = annotation->text_handle;
-  newobj->handles[2] = &newannotation->text_handle;
-
-
-  newannotation->text = text_copy(annotation->text);
-
-  return (Object *)newannotation;
-}
-
-static AnnotationState *
-annotation_get_state(Annotation *annotation)
-{
-  AnnotationState *state = g_new(AnnotationState, 1);
-  state->obj_state.free = NULL;
-  text_get_attributes(annotation->text, &state->text_attrib);
-
-  return state;
-}
-
-static void
-annotation_set_state(Annotation *annotation, AnnotationState *state)
-{
-  text_set_attributes(annotation->text, &state->text_attrib);
-  g_free(state);
-  
-  annotation_update_data(annotation);
-}
-
-
 static void
 annotation_update_data(Annotation *annotation)
 {
   Connection *conn = &annotation->connection;
-  Object *obj = (Object *) annotation;
+  DiaObject *obj = &conn->object;
   Rectangle textrect;
   
+  if (connpoint_is_autogap(conn->endpoint_handles[0].connected_to) ||
+      connpoint_is_autogap(conn->endpoint_handles[1].connected_to)) {
+    connection_adjust_for_autogap(conn);
+  }
   obj->position = conn->endpoints[0];
 
   annotation->text_handle.pos = annotation->text->position;
 
   connection_update_handles(conn);
 
-  /* Boundingbox: */
   connection_update_boundingbox(conn);
-
-  /* fix boundingbox for ANNOTATION_LINE_WIDTH: */
-  obj->bounding_box.top -= ANNOTATION_BORDER/2;
-  obj->bounding_box.left -= ANNOTATION_BORDER/2;
-  obj->bounding_box.bottom += ANNOTATION_BORDER/2;
-  obj->bounding_box.right += ANNOTATION_BORDER/2;
-
-  /* Add boundingbox for text: */
-
   text_calc_boundingbox(annotation->text,&textrect);
-  textrect.top -= ANNOTATION_BORDER/2;
-  textrect.left -= ANNOTATION_BORDER/2;
-  textrect.bottom += ANNOTATION_BORDER/2;
-  textrect.right += ANNOTATION_BORDER/2;
-
   rectangle_union(&obj->bounding_box, &textrect);
 }
 
-
-static void
-annotation_save(Annotation *annotation, ObjectNode obj_node,
-		const char *filename)
-{
-  connection_save(&annotation->connection, obj_node);
-
-  data_add_text(new_attribute(obj_node, "text"),
-		  annotation->text);
-}
-
-static Object *
+static DiaObject *
 annotation_load(ObjectNode obj_node, int version, const char *filename)
 {
-  Annotation *annotation;
-  Connection *conn;
-  Object *obj;
-
-  annotation_init_defaults();
-  
-  annotation = g_malloc(sizeof(Annotation));
-
-  conn = &annotation->connection;
-  obj = (Object *) annotation;
-
-  obj->type = &sadtannotation_type;
-  obj->ops = &annotation_ops;
-
-  connection_load(conn, obj_node);
-  
-  connection_init(conn, 3, 0);
-
-  annotation->text = load_text(obj_node,"text",NULL);
-  if (!annotation->text) {
-    annotation->text = new_text("", annotation_defaults.text_font,
-				annotation_defaults.text_fontheight,
-				&conn->endpoints[1],
-				&annotation_defaults.text_color,
-				ALIGN_CENTER);
-  }
-  
-  annotation->text_handle.id = HANDLE_MOVE_TEXT;
-  annotation->text_handle.type = HANDLE_MINOR_CONTROL;
-  annotation->text_handle.connect_type = HANDLE_NONCONNECTABLE;
-  annotation->text_handle.connected_to = NULL;
-  obj->handles[2] = &annotation->text_handle;
-  
-  annotation_update_data(annotation);
-  
-  return (Object *)annotation;
-}
-
-static void 
-annotation_init_defaults(void) {
-  static int defaults_initialized = 0;
-
-  if (!defaults_initialized) {
-    annotation_defaults.text_font = font_getfont("Helvetica");
-    annotation_defaults.text_fontheight = ANNOTATION_FONTHEIGHT;
-    annotation_defaults.text_color = color_black;
-    defaults_initialized = 1;
-  }
-}
-
-static PROPDLG_TYPE
-annotation_get_defaults()
-{
-  AnnotationDefaultsDialog *dlg = annotation_defaults_dialog;
-
-  annotation_init_defaults();
-  PROPDLG_CREATE(dlg, &annotation_defaults);
-  PROPDLG_SHOW_FONT(dlg,text_font,_("Font:"));
-  PROPDLG_SHOW_FONTHEIGHT(dlg,text_fontheight,_("Font size:"));
-  PROPDLG_SHOW_COLOR(dlg,text_color,_("Font color:"));
-  PROPDLG_READY(dlg);
-  annotation_defaults_dialog = dlg;
-
-  PROPDLG_RETURN(dlg);
-}
-
-static void annotation_apply_defaults()
-{
-  AnnotationDefaultsDialog *dlg = annotation_defaults_dialog;
-
-  PROPDLG_APPLY_FONT(dlg,text_font);
-  PROPDLG_APPLY_FONTHEIGHT(dlg,text_fontheight);
-  PROPDLG_APPLY_COLOR(dlg,text_color);
-}
-    
-static ObjectChange *
-annotation_apply_properties(Annotation *annotation)
-{
-  AnnotationDialog *dlg = annotation_dialog;
-  ObjectState *old_state;
-  
-  old_state = (ObjectState *)annotation_get_state(annotation);
-
-  PROPDLG_APPLY_TEXTFONT(dlg,text);
-  PROPDLG_APPLY_TEXTFONTHEIGHT(dlg,text);
-  PROPDLG_APPLY_TEXTCOLOR(dlg,text);
-
-  annotation_update_data(annotation);
-  return new_object_state_change((Object *)annotation, old_state, 
-				 (GetStateFunc)annotation_get_state,
-				 (SetStateFunc)annotation_set_state);
-}
-
-static PROPDLG_TYPE 
-annotation_get_properties(Annotation *annotation)
-{
-  AnnotationDialog *dlg = annotation_dialog;
-
-  PROPDLG_CREATE(dlg,annotation);
-  PROPDLG_SHOW_TEXTFONT(dlg,text,_("Font:"));
-  PROPDLG_SHOW_TEXTFONTHEIGHT(dlg,text,_("Font size:"));
-  PROPDLG_SHOW_TEXTCOLOR(dlg,text,_("Font color:"));
-  PROPDLG_READY(dlg);
-  annotation_dialog = dlg;
-
-  PROPDLG_RETURN(dlg);
+  return object_load_using_properties(&sadtannotation_type,
+                                      obj_node,version,filename);
 }
 
 
@@ -550,4 +429,3 @@ annotation_get_properties(Annotation *annotation)
 
 
 
-
