@@ -2,7 +2,7 @@
  * Copyright (C) 1998 Alexander Larsson
  *
  * SADT Activity/Data box -- objects for drawing SADT diagrams.
- * Copyright (C) 2000 Cyrille Chepelov
+ * Copyright (C) 2000, 2001 Cyrille Chepelov
  * 
  * Forked from Flowchart toolbox -- objects for drawing flowcharts.
  * Copyright (C) 1999 James Henstridge.
@@ -27,7 +27,6 @@
 #endif
 
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 #include <string.h>
 #include <glib.h>
@@ -36,14 +35,14 @@
 #include "object.h"
 #include "element.h"
 #include "connectionpoint.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "attributes.h"
 #include "text.h"
 #include "widgets.h"
 #include "message.h"
 #include "connpoint_line.h"
 #include "color.h"
-#include "lazyprops.h"
+#include "properties.h"
 
 #include "pixmaps/sadtbox.xpm"
 
@@ -60,21 +59,8 @@ typedef enum {
   ANCHOR_END
 } AnchorShape;
 
-typedef struct _Box Box;
-typedef struct _BoxPropertiesDialog BoxPropertiesDialog;
-typedef struct _BoxDefaultsDialog BoxDefaultsDialog;
-typedef struct _BoxState BoxState;
-typedef struct _BoxDefaults BoxDefaults;
 
-struct _BoxState {
-  ObjectState obj_state;
-
-  real padding;
-  TextAttributes text_attrib;
-  gchar *id;
-};
-
-struct _Box {
+typedef struct _Box {
   Element element;
 
   ConnPointLine *north,*south,*east,*west;
@@ -82,79 +68,43 @@ struct _Box {
   Text *text;
   gchar *id;
   real padding;
-};
 
-struct _BoxPropertiesDialog {
-  AttributeDialog dialog;
-  Box *parent;
-
-  RealAttribute padding;
-  TextFontAttribute text_font;
-  TextFontHeightAttribute text_fontheight;
-  TextColorAttribute text_color;
-  StringAttribute id;
-};
-
-struct _BoxDefaults {
-  Font *font;
-  real font_size;
-  Color fontcolor;
-  real padding;
-};
-
-struct _BoxDefaultsDialog {
-  AttributeDialog dialog;
-  BoxDefaults *parent;
-
-  RealAttribute padding;
-  FontAttribute font;
-  FontHeightAttribute font_size;
-  ColorAttribute fontcolor;
-};
-
-
-static BoxPropertiesDialog *sadtbox_properties_dialog;
-static BoxDefaultsDialog *sadtbox_defaults_dialog;
-static BoxDefaults default_properties;
+  TextAttributes attrs;
+} Box;
 
 static real sadtbox_distance_from(Box *box, Point *point);
 static void sadtbox_select(Box *box, Point *clicked_point,
-		       Renderer *interactive_renderer);
-static void sadtbox_move_handle(Box *box, Handle *handle,
-			    Point *to, HandleMoveReason reason, 
+		       DiaRenderer *interactive_renderer);
+static ObjectChange* sadtbox_move_handle(Box *box, Handle *handle,
+					 Point *to, ConnectionPoint *cp,
+					 HandleMoveReason reason, 
 			    ModifierKeys modifiers);
-static void sadtbox_move(Box *box, Point *to);
-static void sadtbox_draw(Box *box, Renderer *renderer);
+static ObjectChange* sadtbox_move(Box *box, Point *to);
+static void sadtbox_draw(Box *box, DiaRenderer *renderer);
 static void sadtbox_update_data(Box *box, AnchorShape horix, AnchorShape vert);
-static Object *sadtbox_create(Point *startpoint,
+static DiaObject *sadtbox_create(Point *startpoint,
 			  void *user_data,
 			  Handle **handle1,
 			  Handle **handle2);
 static void sadtbox_destroy(Box *box);
-static Object *sadtbox_copy(Box *box);
-static PROPDLG_TYPE sadtbox_get_properties(Box *box);
-static ObjectChange *sadtbox_apply_properties(Box *box);
-
-static BoxState *sadtbox_get_state(Box *box);
-static void sadtbox_set_state(Box *box, BoxState *state);
-
-static void sadtbox_save(Box *box, ObjectNode obj_node, const char *filename);
-static Object *sadtbox_load(ObjectNode obj_node, int version, const char *filename);
-static PROPDLG_TYPE sadtbox_get_defaults(void);
-static void sadtbox_apply_defaults(void);
-
+static DiaObject *sadtbox_load(ObjectNode obj_node, int version, 
+                            const char *filename);
 static DiaMenu *sadtbox_get_object_menu(Box *box, Point *clickedpoint);
 
+static PropDescription *sadtbox_describe_props(Box *box);
+static void sadtbox_get_props(Box *box, GPtrArray *props);
+static void sadtbox_set_props(Box *box, GPtrArray *props);
+
 static ObjectTypeOps sadtbox_type_ops =
 {
   (CreateFunc) sadtbox_create,
-  (LoadFunc)   sadtbox_load,
-  (SaveFunc)   sadtbox_save,
-  (GetDefaultsFunc)   sadtbox_get_defaults,
-  (ApplyDefaultsFunc) sadtbox_apply_defaults
+  (LoadFunc)   sadtbox_load/*using_properties*/,
+  (SaveFunc)   object_save_using_properties,
+  (GetDefaultsFunc)   NULL,
+  (ApplyDefaultsFunc) NULL,
 };
 
-ObjectType sadtbox_type =
+DiaObjectType sadtbox_type =
 {
   "SADT - box",  /* name */
   0,                 /* version */
@@ -168,94 +118,78 @@ static ObjectOps sadtbox_ops = {
   (DrawFunc)            sadtbox_draw,
   (DistanceFunc)        sadtbox_distance_from,
   (SelectFunc)          sadtbox_select,
-  (CopyFunc)            sadtbox_copy,
+  (CopyFunc)            object_copy_using_properties,
   (MoveFunc)            sadtbox_move,
   (MoveHandleFunc)      sadtbox_move_handle,
-  (GetPropertiesFunc)   sadtbox_get_properties,
-  (ApplyPropertiesFunc) sadtbox_apply_properties,
-  (ObjectMenuFunc)      sadtbox_get_object_menu
+  (GetPropertiesFunc)   object_create_props_dialog,
+  (ApplyPropertiesFunc) object_apply_props_from_dialog,
+  (ObjectMenuFunc)      sadtbox_get_object_menu,
+  (DescribePropsFunc)   sadtbox_describe_props,
+  (GetPropsFunc)        sadtbox_get_props,
+  (SetPropsFunc)        sadtbox_set_props
 };
 
-static ObjectChange *
-sadtbox_apply_properties(Box *box)
+static PropNumData text_padding_data = { 0.0, 10.0, 0.1 };
+
+static PropDescription box_props[] = {
+  ELEMENT_COMMON_PROPERTIES,
+  { "padding",PROP_TYPE_REAL,PROP_FLAG_VISIBLE,
+    N_("Text padding"), NULL, &text_padding_data},
+  { "text", PROP_TYPE_TEXT, 0,NULL,NULL},
+  PROP_STD_TEXT_ALIGNMENT,
+  PROP_STD_TEXT_FONT,
+  PROP_STD_TEXT_HEIGHT,
+  PROP_STD_TEXT_COLOUR,
+  { "id", PROP_TYPE_STRING, PROP_FLAG_VISIBLE|PROP_FLAG_DONT_MERGE,
+    N_("Activity/Data identifier"),
+    N_("The identifier which appears in the lower right corner of the Box")},
+  { "cpl_north",PROP_TYPE_CONNPOINT_LINE, 0, NULL, NULL},
+  { "cpl_west",PROP_TYPE_CONNPOINT_LINE, 0, NULL, NULL},
+  { "cpl_south",PROP_TYPE_CONNPOINT_LINE, 0, NULL, NULL},
+  { "cpl_east",PROP_TYPE_CONNPOINT_LINE, 0, NULL, NULL},
+  PROP_DESC_END
+};
+
+static PropDescription *
+sadtbox_describe_props(Box *box) 
 {
-  ObjectState *old_state;
-  BoxPropertiesDialog *dlg = sadtbox_properties_dialog;
-
-  PROPDLG_SANITY_CHECK(dlg,box);
-  
-  old_state = (ObjectState *)sadtbox_get_state(box);
-
-  PROPDLG_APPLY_REAL(dlg, padding);
-  PROPDLG_APPLY_TEXTFONT(dlg,text);
-  PROPDLG_APPLY_TEXTFONTHEIGHT(dlg,text);
-  PROPDLG_APPLY_TEXTCOLOR(dlg,text);
-  PROPDLG_APPLY_STRING(dlg,id);
-
-  sadtbox_update_data(box, ANCHOR_MIDDLE, ANCHOR_MIDDLE);
-  return new_object_state_change(&box->element.object, 
-                                 old_state, 
-				 (GetStateFunc)sadtbox_get_state,
-				 (SetStateFunc)sadtbox_set_state);
-}
-
-static PROPDLG_TYPE
-sadtbox_get_properties(Box *box)
-{
-  BoxPropertiesDialog *dlg = sadtbox_properties_dialog;
-  
-  PROPDLG_CREATE(dlg,box);
-  
-  PROPDLG_SHOW_REAL(dlg,padding,_("Text padding:"),0.0,10.0,0.1);
-  PROPDLG_SHOW_TEXTFONT(dlg,text,_("Font:"));
-  PROPDLG_SHOW_TEXTFONTHEIGHT(dlg,text,_("Font size:"));
-  PROPDLG_SHOW_TEXTCOLOR(dlg,text,_("Font color:"));
-  PROPDLG_SHOW_STRING(dlg,id,_("Activity/Data identifier"));
-  
-  PROPDLG_READY(dlg);
-  sadtbox_properties_dialog = dlg;
-
-  PROPDLG_RETURN(dlg);
-}
-static void
-sadtbox_apply_defaults(void)
-{
-  BoxDefaultsDialog *dlg = sadtbox_defaults_dialog;
-
-  PROPDLG_APPLY_REAL(dlg,padding);
-  PROPDLG_APPLY_FONT(dlg,font);
-  PROPDLG_APPLY_FONTHEIGHT(dlg,font_size);
-  PROPDLG_APPLY_COLOR(dlg,fontcolor);
-}
-
-static void
-init_default_values(void) {
-  static int defaults_initialized = 0;
-
-  if (!defaults_initialized) {
-    default_properties.padding = 0.5;
-    default_properties.font = font_getfont("Helvetica-Bold");
-    default_properties.font_size = 0.8;
-    default_properties.fontcolor = color_black;
-    defaults_initialized = 1;
+  if (box_props[0].quark == 0) {
+    prop_desc_list_calculate_quarks(box_props);
   }
+  return box_props;
+}    
+
+static PropOffset box_offsets[] = {
+  ELEMENT_COMMON_PROPERTIES_OFFSETS,
+  { "padding",PROP_TYPE_REAL,offsetof(Box,padding)},
+  { "text", PROP_TYPE_TEXT, offsetof(Box,text)},
+  { "text_alignment",PROP_TYPE_ENUM,offsetof(Box,attrs.alignment)},
+  { "text_font",PROP_TYPE_FONT,offsetof(Box,attrs.font)},
+  { "text_height",PROP_TYPE_REAL,offsetof(Box,attrs.height)},
+  { "text_colour",PROP_TYPE_COLOUR,offsetof(Box,attrs.color)},
+  { "id", PROP_TYPE_STRING, offsetof(Box,id)},
+  { "cpl_north",PROP_TYPE_CONNPOINT_LINE, offsetof(Box,north)},
+  { "cpl_west",PROP_TYPE_CONNPOINT_LINE, offsetof(Box,west)},
+  { "cpl_south",PROP_TYPE_CONNPOINT_LINE, offsetof(Box,south)},
+  { "cpl_east",PROP_TYPE_CONNPOINT_LINE, offsetof(Box,east)},
+  {NULL}
+};
+
+static void
+sadtbox_get_props(Box *box, GPtrArray *props)
+{  
+  text_get_attributes(box->text,&box->attrs);
+  object_get_props_from_offsets(&box->element.object,
+                                box_offsets,props);
 }
 
-static GtkWidget *
-sadtbox_get_defaults(void)
+static void
+sadtbox_set_props(Box *box, GPtrArray *props)
 {
-  BoxDefaultsDialog *dlg = sadtbox_defaults_dialog;
-
-  init_default_values();
-  PROPDLG_CREATE(dlg, &default_properties);
-  PROPDLG_SHOW_REAL(dlg,padding,_("Text padding:"),0.0,10.0,0.1);
-  PROPDLG_SHOW_FONT(dlg,font,_("Font:"));
-  PROPDLG_SHOW_FONTHEIGHT(dlg,font_size,_("Font size:"));
-  PROPDLG_SHOW_COLOR(dlg,fontcolor,_("Font color:"));
-  PROPDLG_READY(dlg);
-  sadtbox_defaults_dialog = dlg;
-
-  PROPDLG_RETURN(dlg);
+  object_set_props_from_offsets(&box->element.object,
+                                box_offsets,props);
+  apply_textattr_properties(props,box->text,"text",&box->attrs);
+  sadtbox_update_data(box, ANCHOR_MIDDLE, ANCHOR_MIDDLE);
 }
 
 static real
@@ -273,16 +207,17 @@ sadtbox_distance_from(Box *box, Point *point)
 
 static void
 sadtbox_select(Box *box, Point *clicked_point,
-	   Renderer *interactive_renderer)
+	   DiaRenderer *interactive_renderer)
 {
   text_set_cursor(box->text, clicked_point, interactive_renderer);
   text_grab_focus(box->text, &box->element.object);
   element_update_handles(&box->element);
 }
 
-static void
+static ObjectChange*
 sadtbox_move_handle(Box *box, Handle *handle,
-		Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		    Point *to, ConnectionPoint *cp,
+		    HandleMoveReason reason, ModifierKeys modifiers)
 {
   AnchorShape horiz = ANCHOR_MIDDLE, vert = ANCHOR_MIDDLE;
 
@@ -290,7 +225,7 @@ sadtbox_move_handle(Box *box, Handle *handle,
   assert(handle!=NULL);
   assert(to!=NULL);
 
-  element_move_handle(&box->element, handle->id, to, reason);
+  element_move_handle(&box->element, handle->id, to, cp, reason, modifiers);
 
   switch (handle->id) {
   case HANDLE_RESIZE_NW:
@@ -313,19 +248,24 @@ sadtbox_move_handle(Box *box, Handle *handle,
     break;
   }
   sadtbox_update_data(box, horiz, vert);
+
+  return NULL;
 }
 
-static void
+static ObjectChange*
 sadtbox_move(Box *box, Point *to)
 {
   box->element.corner = *to;
   
   sadtbox_update_data(box, ANCHOR_MIDDLE, ANCHOR_MIDDLE);
+
+  return NULL;
 }
 
 static void
-sadtbox_draw(Box *box, Renderer *renderer)
+sadtbox_draw(Box *box, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   Point lr_corner,pos;
   Element *elem;
   real idfontheight;
@@ -338,18 +278,18 @@ sadtbox_draw(Box *box, Renderer *renderer)
   lr_corner.x = elem->corner.x + elem->width;
   lr_corner.y = elem->corner.y + elem->height;
 
-  renderer->ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
-  renderer->ops->fill_rect(renderer, 
+  renderer_ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
+  renderer_ops->fill_rect(renderer, 
 			   &elem->corner,
 			   &lr_corner, 
 			   &SADTBOX_BG_COLOR);
 
 
-  renderer->ops->set_linewidth(renderer, SADTBOX_LINE_WIDTH);
-  renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
-  renderer->ops->set_linejoin(renderer, LINEJOIN_MITER);
+  renderer_ops->set_linewidth(renderer, SADTBOX_LINE_WIDTH);
+  renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
+  renderer_ops->set_linejoin(renderer, LINEJOIN_MITER);
 
-  renderer->ops->draw_rect(renderer, 
+  renderer_ops->draw_rect(renderer, 
 			   &elem->corner,
 			   &lr_corner, 
 			   &SADTBOX_FG_COLOR);
@@ -358,57 +298,22 @@ sadtbox_draw(Box *box, Renderer *renderer)
   text_draw(box->text, renderer);
 
   idfontheight = .75 * box->text->height;
-  renderer->ops->set_font(renderer, box->text->font, idfontheight);
+  renderer_ops->set_font(renderer, box->text->font, idfontheight);
   pos = lr_corner;
   pos.x -= .3 * idfontheight;
   pos.y -= .3 * idfontheight;
-  renderer->ops->draw_string(renderer,
+  renderer_ops->draw_string(renderer,
 			     box->id,
 			     &pos, ALIGN_RIGHT,
 			     &box->text->color);
 }
 
-static void
-sadtbox_free_state(ObjectState *objstate)
-{
-  BoxState *state = (BoxState *)objstate;
-  g_free(state->id);
-  /*  g_free(state); */ /* huh ?? NOT HERE !!! */
-}
-
-static BoxState *
-sadtbox_get_state(Box *box)
-{
-  BoxState *state = g_new0(BoxState, 1);
-
-  state->obj_state.free = sadtbox_free_state;
-
-  state->id = g_strdup(box->id);
-  state->padding = box->padding;
-  text_get_attributes(box->text, &state->text_attrib);
-
-  return state;
-}
-
-static void
-sadtbox_set_state(Box *box, BoxState *state)
-{
-  box->padding = state->padding;
-  text_set_attributes(box->text, &state->text_attrib);
-  if (box->id) g_free(box->id);
-  box->id = state->id;
-  g_free(state);
-  
-  sadtbox_update_data(box, ANCHOR_MIDDLE, ANCHOR_MIDDLE);
-}
-
-
 static void
 sadtbox_update_data(Box *box, AnchorShape horiz, AnchorShape vert)
 {
   Element *elem = &box->element;
   ElementBBExtras *extra = &elem->extra_spacing;
-  Object *obj = &elem->object;
+  DiaObject *obj = &elem->object;
   Point center, bottom_right;
   Point p;
   real width, height;
@@ -421,6 +326,7 @@ sadtbox_update_data(Box *box, AnchorShape horiz, AnchorShape vert)
   center.y += elem->height/2;
   bottom_right.y += elem->height;
 
+  text_calc_boundingbox(box->text, NULL);
   width = box->text->max_width + box->padding*2;
   height = box->text->height * box->text->numlines + box->padding*2;
 
@@ -448,7 +354,7 @@ sadtbox_update_data(Box *box, AnchorShape horiz, AnchorShape vert)
   p = elem->corner;
   p.x += elem->width / 2.0;
   p.y += elem->height / 2.0 - box->text->height * box->text->numlines / 2 +
-    font_ascent(box->text->font, box->text->height);
+    box->text->ascent;
   text_set_position(box->text, &p);
 
   extra->border_trans = SADTBOX_LINE_WIDTH / 2.0;
@@ -460,7 +366,9 @@ sadtbox_update_data(Box *box, AnchorShape horiz, AnchorShape vert)
 
   /* Update connections: */
   nw = elem->corner;
-  se = bottom_right;
+  se.x = elem->corner.x + elem->width;
+  se.y = elem->corner.y + elem->height;
+
   ne.x = se.x;
   ne.y = nw.y;
   sw.y = se.y;
@@ -510,7 +418,7 @@ sadtbox_create_change(Box *box, ObjectChange *inner, ConnPointLine *cpl) {
 }
 
 static ObjectChange *
-sadtbox_add_connpoint_callback(Object *obj, Point *clicked, gpointer data) 
+sadtbox_add_connpoint_callback(DiaObject *obj, Point *clicked, gpointer data) 
 {
   ObjectChange *change;
   ConnPointLine *cpl;
@@ -523,7 +431,7 @@ sadtbox_add_connpoint_callback(Object *obj, Point *clicked, gpointer data)
 }
 
 static ObjectChange *
-sadtbox_remove_connpoint_callback(Object *obj, Point *clicked, gpointer data) 
+sadtbox_remove_connpoint_callback(DiaObject *obj, Point *clicked, gpointer data) 
 {
   ObjectChange *change;
   ConnPointLine *cpl;
@@ -561,7 +469,7 @@ sadtbox_get_object_menu(Box *box, Point *clickedpoint)
 }
 
 
-static Object *
+static DiaObject *
 sadtbox_create(Point *startpoint,
 	   void *user_data,
 	   Handle **handle1,
@@ -569,11 +477,10 @@ sadtbox_create(Point *startpoint,
 {
   Box *box;
   Element *elem;
-  Object *obj;
+  DiaObject *obj;
   Point p;
-
-  init_default_values();
-
+  DiaFont* font;
+  
   box = g_malloc0(sizeof(Box));
   elem = &box->element;
   obj = &elem->object;
@@ -586,17 +493,22 @@ sadtbox_create(Point *startpoint,
   elem->width = DEFAULT_WIDTH;
   elem->height = DEFAULT_HEIGHT;
 
-  box->padding = default_properties.padding;
+  box->padding = 0.5; /* default_values.padding; */
   
   p = *startpoint;
   p.x += elem->width / 2.0;
-  p.y += elem->height / 2.0 + default_properties.font_size / 2;
-  box->text = new_text("", default_properties.font,
-		       default_properties.font_size, &p, 
-		       &SADTBOX_FG_COLOR,
-		       ALIGN_CENTER);
+  p.y += elem->height / 2.0 + /*default_properties.font_size*/ 0.8 / 2;
+
+  font = dia_font_new_from_style( DIA_FONT_SANS|DIA_FONT_BOLD ,0.8);
+  
+  box->text = new_text("", font,
+                       0.8, &p, 
+                       &color_black,
+                       ALIGN_CENTER);
+  dia_font_unref(font);
+  
   box->id = g_strdup("A0"); /* should be made better. 
-				   Automatic counting ? */
+                               Automatic counting ? */
 
   element_init(elem, 8, 0);
 
@@ -628,91 +540,16 @@ sadtbox_destroy(Box *box)
   element_destroy(&box->element);
 }
 
-static Object *
-sadtbox_copy(Box *box)
-{
-  Box *newbox;
-  Element *elem, *newelem;
-  Object *newobj;
-  int rcc;
 
-  elem = &box->element;
-  
-  newbox = g_malloc0(sizeof(Box));
-  newelem = &newbox->element;
-  newobj = &newelem->object;
-
-  element_copy(elem, newelem);
-  rcc = 0;
-  newbox->north = connpointline_copy(newobj,box->north,&rcc);
-  newbox->west = connpointline_copy(newobj,box->west,&rcc);
-  newbox->south = connpointline_copy(newobj,box->south,&rcc);
-  newbox->east = connpointline_copy(newobj,box->east,&rcc);
-  g_assert(rcc == newobj->num_connections);
-  
-  newbox->id = g_strdup(box->id);
-  newbox->padding = box->padding;
-  newbox->text = text_copy(box->text);  
-
-  return &newbox->element.object;
-}
-
-
-static void
-sadtbox_save(Box *box, ObjectNode obj_node, const char *filename)
-{
-  element_save(&box->element, obj_node);
-
-  save_real(obj_node,"padding",box->padding);
-  save_text(obj_node,"text",box->text);
-  save_int(obj_node,"cpl_north",box->north->num_connections);
-  save_int(obj_node,"cpl_west",box->west->num_connections);
-  save_int(obj_node,"cpl_south",box->south->num_connections);
-  save_int(obj_node,"cpl_east",box->east->num_connections);
-  save_string(obj_node,"id",box->id);
-}
-
-static Object *
+static DiaObject *
 sadtbox_load(ObjectNode obj_node, int version, const char *filename)
 {
-  Box *box;
-  Element *elem;
-  Object *obj;
-  int realconncount;
-  Point p = {0.0,0.0};
-
-  box = g_malloc0(sizeof(Box));
-  elem = &box->element;
-  obj = &elem->object;
-  
-  obj->type = &sadtbox_type;
-  obj->ops = &sadtbox_ops;
-
-  element_load(elem, obj_node);
-
-  box->padding = load_real(obj_node,"padding",default_properties.padding);
-  box->text = load_text(obj_node,"text",NULL);
-  if (!box->text) 
-    box->text = new_text("", default_properties.font,
-			 default_properties.font_size, &p, 
-			 &SADTBOX_FG_COLOR,
-			 ALIGN_CENTER);
-
-  element_init(elem, 8, 0);
-  realconncount = 0;
-
-  box->north = connpointline_load(obj,obj_node,"cpl_north",4,&realconncount);
-  box->west = connpointline_load(obj,obj_node,"cpl_west",3,&realconncount);
-  box->south = connpointline_load(obj,obj_node,"cpl_south",1,&realconncount);
-  box->east = connpointline_load(obj,obj_node,"cpl_east",3,&realconncount);
-
-  box->id = load_string(obj_node,"id",NULL);
-  if (!box->id) box->id = g_strdup("A0");
-  
-  box->element.extra_spacing.border_trans = SADTBOX_LINE_WIDTH/2.0;
-  sadtbox_update_data(box, ANCHOR_MIDDLE, ANCHOR_MIDDLE);
-
-  return &box->element.object;
+  return object_load_using_properties(&sadtbox_type,
+                                      obj_node,version,filename);
 }
 
 
+
+
+
+
