@@ -39,10 +39,12 @@
 
 typedef struct _Classicon Classicon;
 
+#define NUM_CONNECTIONS 9
+
 struct _Classicon {
   Element element;
 
-  ConnectionPoint connections[8];
+  ConnectionPoint connections[NUM_CONNECTIONS];
   
   int stereotype;
   int is_object;
@@ -70,7 +72,8 @@ static real classicon_distance_from(Classicon *cicon, Point *point);
 static void classicon_select(Classicon *cicon, Point *clicked_point,
 			     DiaRenderer *interactive_renderer);
 static ObjectChange* classicon_move_handle(Classicon *cicon, Handle *handle,
-					   Point *to, HandleMoveReason reason, ModifierKeys modifiers);
+					   Point *to, ConnectionPoint *cp,
+					   HandleMoveReason reason, ModifierKeys modifiers);
 static ObjectChange* classicon_move(Classicon *cicon, Point *to);
 static void classicon_draw(Classicon *cicon, DiaRenderer *renderer);
 static DiaObject *classicon_create(Point *startpoint,
@@ -200,7 +203,8 @@ classicon_select(Classicon *cicon, Point *clicked_point,
 
 static ObjectChange*
 classicon_move_handle(Classicon *cicon, Handle *handle,
-		      Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		      Point *to, ConnectionPoint *cp,
+		      HandleMoveReason reason, ModifierKeys modifiers)
 {
   assert(cicon!=NULL);
   assert(handle!=NULL);
@@ -317,10 +321,10 @@ classicon_draw(Classicon *icon, DiaRenderer *renderer)
     renderer_ops->set_linewidth(renderer, 0.01);
     if (icon->stereotype==CLASSICON_BOUNDARY)
       x += r/2.0;
-    p1.y = p2.y = icon->text->position.y + icon->text->descent;
+    p1.y = p2.y = icon->text->position.y + text_get_descent(icon->text);
     for (i=0; i<icon->text->numlines; i++) { 
-      p1.x = x + (w - icon->text->row_width[i])/2;
-      p2.x = p1.x + icon->text->row_width[i];
+      p1.x = x + (w - text_get_line_width(icon->text, i))/2;
+      p2.x = p1.x + text_get_line_width(icon->text, i);
       renderer_ops->draw_line(renderer,
 			       &p1, &p2,
 			       &icon->line_color);
@@ -399,6 +403,9 @@ classicon_update_data(Classicon *cicon)
   cicon->connections[7].pos.x = elem->corner.x + elem->width;
   cicon->connections[7].pos.y = elem->corner.y + elem->height;
   cicon->connections[7].directions = DIR_SOUTH|DIR_EAST;
+  cicon->connections[8].pos.x = elem->corner.x + elem->width/2;
+  cicon->connections[8].pos.y = elem->corner.y + elem->height/2;
+  cicon->connections[8].directions = DIR_ALL;
   
   element_update_boundingbox(elem);
 
@@ -447,13 +454,14 @@ classicon_create(Point *startpoint,
 
   dia_font_unref(font);
   
-  element_init(elem, 8, 8);
+  element_init(elem, 8, NUM_CONNECTIONS);
   
-  for (i=0;i<8;i++) {
+  for (i=0;i<NUM_CONNECTIONS;i++) {
     obj->connections[i] = &cicon->connections[i];
     cicon->connections[i].object = obj;
     cicon->connections[i].connected = NULL;
   }
+  cicon->connections[8].flags = CP_FLAGS_MAIN;
   elem->extra_spacing.border_trans = 0.0;
   classicon_update_data(cicon);
 
