@@ -21,16 +21,16 @@
 #endif
 
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 #include <string.h>
 
 #include "intl.h"
 #include "object.h"
 #include "connection.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "handle.h"
 #include "properties.h"
+#include "attributes.h"
 
 #include "pixmaps/implements.xpm"
 
@@ -46,7 +46,10 @@ struct _Implements {
 
   Point circle_center; /* Calculated from diameter*/
 
-  char *text;
+  Color text_color;
+  Color line_color;
+
+  gchar *text;
   Point text_pos;
   real text_width;
 
@@ -59,41 +62,41 @@ struct _Implements {
 #define HANDLE_MOVE_TEXT (HANDLE_CUSTOM2)
 
 
-static Font *implements_font = NULL;
+static DiaFont *implements_font = NULL;
 
-static void implements_move_handle(Implements *implements, Handle *handle,
-				   Point *to, HandleMoveReason reason, ModifierKeys modifiers);
-static void implements_move(Implements *implements, Point *to);
+static ObjectChange* implements_move_handle(Implements *implements, Handle *handle,
+					    Point *to, ConnectionPoint *cp,
+					    HandleMoveReason reason, ModifierKeys modifiers);
+static ObjectChange* implements_move(Implements *implements, Point *to);
 static void implements_select(Implements *implements, Point *clicked_point,
-			      Renderer *interactive_renderer);
-static void implements_draw(Implements *implements, Renderer *renderer);
-static Object *implements_create(Point *startpoint,
+			      DiaRenderer *interactive_renderer);
+static void implements_draw(Implements *implements, DiaRenderer *renderer);
+static DiaObject *implements_create(Point *startpoint,
 				 void *user_data,
 				 Handle **handle1,
 				 Handle **handle2);
 static real implements_distance_from(Implements *implements, Point *point);
 static void implements_update_data(Implements *implements);
 static void implements_destroy(Implements *implements);
-static Object *implements_copy(Implements *implements);
 
 static PropDescription *implements_describe_props(Implements *implements);
-static void implements_get_props(Implements * implements, Property *props, guint nprops);
-static void implements_set_props(Implements * implements, Property *props, guint nprops);
+static void implements_get_props(Implements * implements, GPtrArray *props);
+static void implements_set_props(Implements * implements, GPtrArray *props);
 
-static void implements_save(Implements *implements, ObjectNode obj_node,
-			    const char *filename);
-static Object *implements_load(ObjectNode obj_node, int version,
+static DiaObject *implements_load(ObjectNode obj_node, int version,
 			       const char *filename);
 
 
 static ObjectTypeOps implements_type_ops =
 {
   (CreateFunc) implements_create,
-  (LoadFunc)   implements_load,
-  (SaveFunc)   implements_save
+  (LoadFunc)   implements_load,/*using_properties*/     /* load */
+  (SaveFunc)   object_save_using_properties,      /* save */
+  (GetDefaultsFunc)   NULL, 
+  (ApplyDefaultsFunc) NULL
 };
 
-ObjectType implements_type =
+DiaObjectType implements_type =
 {
   "UML - Implements",   /* name */
   0,                   /* version */
@@ -106,7 +109,7 @@ static ObjectOps implements_ops = {
   (DrawFunc)            implements_draw,
   (DistanceFunc)        implements_distance_from,
   (SelectFunc)          implements_select,
-  (CopyFunc)            implements_copy,
+  (CopyFunc)            object_copy_using_properties,
   (MoveFunc)            implements_move,
   (MoveHandleFunc)      implements_move_handle,
   (GetPropertiesFunc)   object_create_props_dialog,
@@ -118,9 +121,14 @@ static ObjectOps implements_ops = {
 };
 
 static PropDescription implements_props[] = {
-  OBJECT_COMMON_PROPERTIES,
-  { "interface", PROP_TYPE_STRING, PROP_FLAG_VISIBLE,
+  CONNECTION_COMMON_PROPERTIES,
+  PROP_STD_LINE_COLOUR_OPTIONAL, 
+  /* can't use PROP_STD_TEXT_COLOUR_OPTIONAL cause it has PROP_FLAG_DONT_SAVE. It is designed to fill the Text object - not some subset */
+  PROP_STD_TEXT_COLOUR_OPTIONS(PROP_FLAG_VISIBLE|PROP_FLAG_STANDARD|PROP_FLAG_OPTIONAL),
+  { "text", PROP_TYPE_STRING, PROP_FLAG_VISIBLE,
     N_("Interface:"), NULL, NULL },
+  { "text_pos", PROP_TYPE_POINT, 0, NULL, NULL, NULL },
+  { "diameter", PROP_TYPE_REAL, 0, NULL, NULL, NULL },
   PROP_DESC_END
 };
 
@@ -133,25 +141,27 @@ implements_describe_props(Implements *implements)
 }
 
 static PropOffset implements_offsets[] = {
-  OBJECT_COMMON_PROPERTIES_OFFSETS,
-  { "interface", PROP_TYPE_STRING, offsetof(Implements, text) },
+  CONNECTION_COMMON_PROPERTIES_OFFSETS,
+  { "line_colour",PROP_TYPE_COLOUR,offsetof(Implements, line_color) },
+  { "text_colour",PROP_TYPE_COLOUR,offsetof(Implements, text_color) },
+  { "text", PROP_TYPE_STRING, offsetof(Implements, text) },
+  { "text_pos", PROP_TYPE_POINT, offsetof(Implements, text_pos) },
+  { "diameter", PROP_TYPE_REAL, offsetof(Implements, circle_diameter) },
   { NULL, 0, 0 }
 };
 
 static void
-implements_get_props(Implements * implements, Property *props, guint nprops)
+implements_get_props(Implements * implements, GPtrArray *props)
 {
-  if (object_get_props_from_offsets(&implements->connection.object, 
-                                    implements_offsets, props, nprops))
-    return;
+  object_get_props_from_offsets(&implements->connection.object, 
+                                implements_offsets, props);
 }
 
 static void
-implements_set_props(Implements *implements, Property *props, guint nprops)
+implements_set_props(Implements *implements, GPtrArray *props)
 {
-  if (!object_set_props_from_offsets(&implements->connection.object, 
-                                     implements_offsets, props, nprops)) {
-  }
+  object_set_props_from_offsets(&implements->connection.object, 
+                                implements_offsets, props);
   implements_update_data(implements);
 }
 
@@ -174,14 +184,15 @@ implements_distance_from(Implements *implements, Point *point)
 
 static void
 implements_select(Implements *implements, Point *clicked_point,
-	    Renderer *interactive_renderer)
+	    DiaRenderer *interactive_renderer)
 {
   connection_update_handles(&implements->connection);
 }
 
-static void
+static ObjectChange*
 implements_move_handle(Implements *implements, Handle *handle,
-		 Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		       Point *to, ConnectionPoint *cp,
+		       HandleMoveReason reason, ModifierKeys modifiers)
 {
   Point v1, v2;
   
@@ -202,15 +213,19 @@ implements_move_handle(Implements *implements, Handle *handle,
       implements->circle_diameter = 0.03;
   } else {
     v1 = implements->connection.endpoints[1];
-    connection_move_handle(&implements->connection, handle->id, to, reason);
+    connection_move_handle(&implements->connection, handle->id, to, cp, 
+			   reason, modifiers);
+    connection_adjust_for_autogap(&implements->connection);
     point_sub(&v1, &implements->connection.endpoints[1]);
     point_sub(&implements->text_pos, &v1);
   }
 
   implements_update_data(implements);
+
+  return NULL;
 }
 
-static void
+static ObjectChange*
 implements_move(Implements *implements, Point *to)
 {
   Point start_to_end;
@@ -229,11 +244,14 @@ implements_move(Implements *implements, Point *to)
   point_add(&implements->text_pos, &delta);
   
   implements_update_data(implements);
+
+  return NULL;
 }
 
 static void
-implements_draw(Implements *implements, Renderer *renderer)
+implements_draw(Implements *implements, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   Point *endpoints;
   
   assert(implements != NULL);
@@ -241,33 +259,33 @@ implements_draw(Implements *implements, Renderer *renderer)
 
   endpoints = &implements->connection.endpoints[0];
   
-  renderer->ops->set_linewidth(renderer, IMPLEMENTS_WIDTH);
-  renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
-  renderer->ops->set_linecaps(renderer, LINECAPS_BUTT);
+  renderer_ops->set_linewidth(renderer, IMPLEMENTS_WIDTH);
+  renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
+  renderer_ops->set_linecaps(renderer, LINECAPS_BUTT);
 
-  renderer->ops->draw_line(renderer,
+  renderer_ops->draw_line(renderer,
 			   &endpoints[0], &endpoints[1],
-			   &color_black);
+			   &implements->line_color);
 
-  renderer->ops->fill_ellipse(renderer, &implements->circle_center,
+  renderer_ops->fill_ellipse(renderer, &implements->circle_center,
 			      implements->circle_diameter,
 			      implements->circle_diameter,
 			      &color_white);
-  renderer->ops->draw_ellipse(renderer, &implements->circle_center,
+  renderer_ops->draw_ellipse(renderer, &implements->circle_center,
 			      implements->circle_diameter,
 			      implements->circle_diameter,
-			      &color_black);
+			      &implements->line_color);
 
 
-  renderer->ops->set_font(renderer, implements_font, IMPLEMENTS_FONTHEIGHT);
+  renderer_ops->set_font(renderer, implements_font, IMPLEMENTS_FONTHEIGHT);
   if (implements->text)
-    renderer->ops->draw_string(renderer,
+    renderer_ops->draw_string(renderer,
 			       implements->text,
 			       &implements->text_pos, ALIGN_LEFT,
-			       &color_black);
+			       &implements->text_color);
 }
 
-static Object *
+static DiaObject *
 implements_create(Point *startpoint,
 		  void *user_data,
 		  Handle **handle1,
@@ -275,11 +293,13 @@ implements_create(Point *startpoint,
 {
   Implements *implements;
   Connection *conn;
-  Object *obj;
+  DiaObject *obj;
   Point defaultlen = { 1.0, 1.0 };
 
-  if (implements_font == NULL)
-    implements_font = font_getfont("Courier");
+  if (implements_font == NULL) {
+    implements_font = 
+      dia_font_new_from_style(DIA_FONT_MONOSPACE, IMPLEMENTS_FONTHEIGHT);
+  }
   
   implements = g_malloc0(sizeof(Implements));
 
@@ -295,6 +315,8 @@ implements_create(Point *startpoint,
   
   connection_init(conn, 4, 0);
 
+  implements->line_color = attributes_get_foreground();
+  implements->text_color = color_black;
   implements->text = NULL;
   implements->text_width = 0.0;
   implements->text_pos = conn->endpoints[1];
@@ -328,42 +350,12 @@ implements_destroy(Implements *implements)
   g_free(implements->text);
 }
 
-static Object *
-implements_copy(Implements *implements)
-{
-  Implements *newimplements;
-  Connection *conn, *newconn;
-  Object *newobj;
-  
-  conn = &implements->connection;
-  
-  newimplements = g_malloc0(sizeof(Implements));
-  newconn = &newimplements->connection;
-  newobj = &newconn->object;
-
-  connection_copy(conn, newconn);
-
-  newimplements->text_handle = implements->text_handle;
-  newobj->handles[2] = &newimplements->text_handle;
-  newimplements->circle_handle = implements->circle_handle;
-  newobj->handles[3] = &newimplements->circle_handle;
-
-  newimplements->circle_diameter = implements->circle_diameter;
-  newimplements->circle_center = implements->circle_center;
-
-  newimplements->text = strdup(implements->text);
-  newimplements->text_pos = implements->text_pos;
-  newimplements->text_width = implements->text_width;
-
-  return &newimplements->connection.object;
-}
-
 static void
 implements_update_data(Implements *implements)
 {
   Connection *conn = &implements->connection;
-  Object *obj = &conn->object;
-  ConnectionBBExtras *extra = &conn->extra_spacing;
+  DiaObject *obj = &conn->object;
+  LineBBExtras *extra = &conn->extra_spacing;
   Point delta;
   Point point;
   real len;
@@ -371,10 +363,14 @@ implements_update_data(Implements *implements)
 
   implements->text_width = 0.0;
   if (implements->text)
-    implements->text_width = font_string_width(implements->text,
-					       implements_font,
-					       IMPLEMENTS_FONTHEIGHT);
+    implements->text_width = dia_font_string_width(implements->text,
+                                                   implements_font,
+                                                   IMPLEMENTS_FONTHEIGHT);
 
+  if (connpoint_is_autogap(conn->endpoint_handles[0].connected_to) ||
+      connpoint_is_autogap(conn->endpoint_handles[1].connected_to)) {
+    connection_adjust_for_autogap(conn);
+  }
   obj->position = conn->endpoints[0];
 
   implements->text_handle.pos = implements->text_pos;
@@ -408,77 +404,18 @@ implements_update_data(Implements *implements)
   /* Add boundingbox for text: */
   rect.left = implements->text_pos.x;
   rect.right = rect.left + implements->text_width;
-  rect.top = implements->text_pos.y - font_ascent(implements_font, IMPLEMENTS_FONTHEIGHT);
+  rect.top = implements->text_pos.y;
+  if (implements->text)
+    rect.top -= dia_font_ascent(implements->text,
+				implements_font,
+				IMPLEMENTS_FONTHEIGHT);
   rect.bottom = rect.top + IMPLEMENTS_FONTHEIGHT;
   rectangle_union(&obj->bounding_box, &rect);
 }
 
-
-static void
-implements_save(Implements *implements, ObjectNode obj_node,
-		const char *filename)
-{
-  connection_save(&implements->connection, obj_node);
-
-  data_add_real(new_attribute(obj_node, "diameter"),
-		implements->circle_diameter);
-  data_add_string(new_attribute(obj_node, "text"),
-		  implements->text);
-  data_add_point(new_attribute(obj_node, "text_pos"),
-		 &implements->text_pos);
-}
-
-static Object *
+static DiaObject *
 implements_load(ObjectNode obj_node, int version, const char *filename)
 {
-  Implements *implements;
-  AttributeNode attr;
-  Connection *conn;
-  Object *obj;
-
-  if (implements_font == NULL)
-    implements_font = font_getfont("Courier");
-
-  implements = g_malloc0(sizeof(Implements));
-
-  conn = &implements->connection;
-  obj = &conn->object;
-
-  obj->type = &implements_type;
-  obj->ops = &implements_ops;
-
-  connection_load(conn, obj_node);
-  
-  connection_init(conn, 4, 0);
-
-  implements->circle_diameter = 1.0;
-  attr = object_find_attribute(obj_node, "diameter");
-  if (attr != NULL)
-    implements->circle_diameter = data_real(attribute_first_data(attr));
-
-  implements->text = NULL;
-  attr = object_find_attribute(obj_node, "text");
-  if (attr != NULL)
-    implements->text = data_string(attribute_first_data(attr));
-
-  attr = object_find_attribute(obj_node, "text_pos");
-  if (attr != NULL)
-    data_point(attribute_first_data(attr), &implements->text_pos);
-
-  implements->text_handle.id = HANDLE_MOVE_TEXT;
-  implements->text_handle.type = HANDLE_MINOR_CONTROL;
-  implements->text_handle.connect_type = HANDLE_NONCONNECTABLE;
-  implements->text_handle.connected_to = NULL;
-  obj->handles[2] = &implements->text_handle;
-  
-  implements->circle_handle.id = HANDLE_CIRCLE_SIZE;
-  implements->circle_handle.type = HANDLE_MINOR_CONTROL;
-  implements->circle_handle.connect_type = HANDLE_NONCONNECTABLE;
-  implements->circle_handle.connected_to = NULL;
-  obj->handles[3] = &implements->circle_handle;
-
-  implements_update_data(implements);
-  
-  return &implements->connection.object;
+  return object_load_using_properties(&implements_type,
+                                      obj_node,version,filename);
 }
-
