@@ -38,10 +38,12 @@ typedef struct _UsecasePropertiesDialog UsecasePropertiesDialog;
 typedef struct _Usecase Usecase;
 typedef struct _UsecaseState UsecaseState;
 
+#define NUM_CONNECTIONS 9
+
 struct _Usecase {
   Element element;
 
-  ConnectionPoint connections[8];
+  ConnectionPoint connections[NUM_CONNECTIONS];
 
   Text *text;
   int text_outside;
@@ -76,12 +78,12 @@ static ObjectChange* usecase_move_handle(Usecase *usecase, Handle *handle,
 					 HandleMoveReason reason, ModifierKeys modifiers);
 static ObjectChange* usecase_move(Usecase *usecase, Point *to);
 static void usecase_draw(Usecase *usecase, DiaRenderer *renderer);
-static Object *usecase_create(Point *startpoint,
+static DiaObject *usecase_create(Point *startpoint,
 			      void *user_data,
 			      Handle **handle1,
 			      Handle **handle2);
 static void usecase_destroy(Usecase *usecase);
-static Object *usecase_load(ObjectNode obj_node, int version,
+static DiaObject *usecase_load(ObjectNode obj_node, int version,
 			    const char *filename);
 static void usecase_update_data(Usecase *usecase);
 static PropDescription *usecase_describe_props(Usecase *usecase);
@@ -97,7 +99,7 @@ static ObjectTypeOps usecase_type_ops =
   (ApplyDefaultsFunc) NULL
 };
 
-ObjectType usecase_type =
+DiaObjectType usecase_type =
 {
   "UML - Usecase",   /* name */
   0,                      /* version */
@@ -180,7 +182,7 @@ usecase_set_props(Usecase *usecase, GPtrArray *props)
 static real
 usecase_distance_from(Usecase *usecase, Point *point)
 {
-  Object *obj = &usecase->element.object;
+  DiaObject *obj = &usecase->element.object;
   return distance_rectangle_point(&obj->bounding_box, point);
 }
 
@@ -286,7 +288,7 @@ usecase_update_data(Usecase *usecase)
   Point c, half, r,p;
   
   Element *elem = &usecase->element;
-  Object *obj = &elem->object;
+  DiaObject *obj = &elem->object;
   
   text_calc_boundingbox(usecase->text, NULL);
   w = usecase->text->max_width;
@@ -357,6 +359,8 @@ usecase_update_data(Usecase *usecase)
       usecase->connections[7].pos.x = c.x + half.x;
       usecase->connections[7].pos.y = c.y + half.y;
   }
+  usecase->connections[8].pos.x = c.x;
+  usecase->connections[8].pos.y = c.y;
 
   usecase->connections[0].directions = DIR_NORTH|DIR_WEST;
   usecase->connections[1].directions = DIR_NORTH;
@@ -366,6 +370,7 @@ usecase_update_data(Usecase *usecase)
   usecase->connections[5].directions = DIR_SOUTH|DIR_WEST;
   usecase->connections[6].directions = DIR_SOUTH;
   usecase->connections[7].directions = DIR_SOUTH|DIR_EAST;
+  usecase->connections[8].directions = DIR_ALL;
 
   h = usecase->text->height*usecase->text->numlines;
   p = usecase->element.corner;
@@ -385,7 +390,7 @@ usecase_update_data(Usecase *usecase)
 
 }
 
-static Object *
+static DiaObject *
 usecase_create(Point *startpoint,
 	       void *user_data,
   	       Handle **handle1,
@@ -393,7 +398,7 @@ usecase_create(Point *startpoint,
 {
   Usecase *usecase;
   Element *elem;
-  Object *obj;
+  DiaObject *obj;
   Point p;
   DiaFont *font;
   int i;
@@ -421,13 +426,14 @@ usecase_create(Point *startpoint,
   text_get_attributes(usecase->text,&usecase->attrs);
   usecase->text_outside = 0;
   usecase->collaboration = 0;
-  element_init(elem, 8, 8);
+  element_init(elem, 8, NUM_CONNECTIONS);
   
-  for (i=0;i<8;i++) {
+  for (i=0;i<NUM_CONNECTIONS;i++) {
     obj->connections[i] = &usecase->connections[i];
     usecase->connections[i].object = obj;
     usecase->connections[i].connected = NULL;
   }
+  usecase->connections[8].flags = CP_FLAGS_MAIN;
   elem->extra_spacing.border_trans = 0.0;
   usecase_update_data(usecase);
 
@@ -448,7 +454,7 @@ usecase_destroy(Usecase *usecase)
   element_destroy(&usecase->element);
 }
 
-static Object *
+static DiaObject *
 usecase_load(ObjectNode obj_node, int version, const char *filename)
 {
   return object_load_using_properties(&usecase_type,
