@@ -38,7 +38,7 @@
 #include "properties.h"
 #include "create.h"
 
-#include "pixmaps/bezier.xpm"
+#include "tool-icons.h"
 
 #define DEFAULT_WIDTH 0.15
 
@@ -52,6 +52,7 @@ struct _Bezierline {
   real dashlength;
   real line_width;
   Arrow start_arrow, end_arrow;
+  real absolute_start_gap, absolute_end_gap;
 };
 
 
@@ -62,14 +63,14 @@ static ObjectChange* bezierline_move(Bezierline *bezierline, Point *to);
 static void bezierline_select(Bezierline *bezierline, Point *clicked_point,
 			      DiaRenderer *interactive_renderer);
 static void bezierline_draw(Bezierline *bezierline, DiaRenderer *renderer);
-static Object *bezierline_create(Point *startpoint,
+static DiaObject *bezierline_create(Point *startpoint,
 				 void *user_data,
 				 Handle **handle1,
 				 Handle **handle2);
 static real bezierline_distance_from(Bezierline *bezierline, Point *point);
 static void bezierline_update_data(Bezierline *bezierline);
 static void bezierline_destroy(Bezierline *bezierline);
-static Object *bezierline_copy(Bezierline *bezierline);
+static DiaObject *bezierline_copy(Bezierline *bezierline);
 
 static PropDescription *bezierline_describe_props(Bezierline *bezierline);
 static void bezierline_get_props(Bezierline *bezierline, GPtrArray *props);
@@ -77,10 +78,14 @@ static void bezierline_set_props(Bezierline *bezierline, GPtrArray *props);
 
 static void bezierline_save(Bezierline *bezierline, ObjectNode obj_node,
 			  const char *filename);
-static Object *bezierline_load(ObjectNode obj_node, int version,
+static DiaObject *bezierline_load(ObjectNode obj_node, int version,
 			     const char *filename);
 static DiaMenu *bezierline_get_object_menu(Bezierline *bezierline, Point *clickedpoint);
 
+static void compute_gap_points(Bezierline *bezierline, Point *gap_points);
+static real approx_bez_length(BezierConn *bez);
+static void exchange_bez_gap_points(BezierConn * bez, Point* gap_points);
+
 static ObjectTypeOps bezierline_type_ops =
 {
   (CreateFunc)bezierline_create,   /* create */
@@ -90,18 +95,18 @@ static ObjectTypeOps bezierline_type_ops =
   (ApplyDefaultsFunc) NULL
 };
 
-static ObjectType bezierline_type =
+static DiaObjectType bezierline_type =
 {
   "Standard - BezierLine",   /* name */
   0,                         /* version */
-  (char **) bezier_xpm,      /* pixmap */
+  (char **) bezierline_icon,      /* pixmap */
   
   &bezierline_type_ops,      /* ops */
   NULL,                      /* pixmap_file */
   0                          /* default_user_data */
 };
 
-ObjectType *_bezierline_type = (ObjectType *) &bezierline_type;
+DiaObjectType *_bezierline_type = (DiaObjectType *) &bezierline_type;
 
 
 static ObjectOps bezierline_ops = {
@@ -120,6 +125,8 @@ static ObjectOps bezierline_ops = {
   (SetPropsFunc)        bezierline_set_props,
 };
 
+static PropNumData gap_range = { -G_MAXFLOAT, G_MAXFLOAT, 0.1};
+
 static PropDescription bezierline_props[] = {
   BEZCONN_COMMON_PROPERTIES,
   PROP_STD_LINE_WIDTH,
@@ -127,6 +134,12 @@ static PropDescription bezierline_props[] = {
   PROP_STD_LINE_STYLE,
   PROP_STD_START_ARROW,
   PROP_STD_END_ARROW,
+  PROP_FRAME_BEGIN("gaps",0,N_("Line gaps")),
+  { "absolute_start_gap", PROP_TYPE_REAL, PROP_FLAG_VISIBLE,
+    N_("Absolute start gap"), NULL, &gap_range },
+  { "absolute_end_gap", PROP_TYPE_REAL, PROP_FLAG_VISIBLE,
+    N_("Absolute end gap"), NULL, &gap_range },
+  PROP_FRAME_END("gaps",0),
   PROP_DESC_END
 };
 
@@ -146,6 +159,8 @@ static PropOffset bezierline_offsets[] = {
     offsetof(Bezierline, line_style), offsetof(Bezierline, dashlength) },
   { "start_arrow", PROP_TYPE_ARROW, offsetof(Bezierline, start_arrow) },
   { "end_arrow", PROP_TYPE_ARROW, offsetof(Bezierline, end_arrow) },
+  { "absolute_start_gap", PROP_TYPE_REAL, offsetof(Bezierline, absolute_start_gap) },
+  { "absolute_end_gap", PROP_TYPE_REAL, offsetof(Bezierline, absolute_end_gap) },
   { NULL, 0, 0 }
 };
 
@@ -168,11 +183,19 @@ static real
 bezierline_distance_from(Bezierline *bezierline, Point *point)
 {
   BezierConn *bez = &bezierline->bez;
-  return bezierconn_distance_from(bez, point, bezierline->line_width);
-}
-
-static Handle *bezierline_closest_handle(Bezierline *bezierline, Point *point) {
-  return bezierconn_closest_handle(&bezierline->bez, point);
+  if (connpoint_is_autogap(bez->object.handles[0]->connected_to) ||
+      connpoint_is_autogap(bez->object.handles[3*(bez->numpoints-1)]->connected_to) ||
+      bezierline->absolute_start_gap || bezierline->absolute_end_gap) {
+    Point gap_points[4];
+    real distance;
+    compute_gap_points(bezierline, gap_points);
+    exchange_bez_gap_points(bez,gap_points);
+    distance = bezierconn_distance_from(bez, point, bezierline->line_width);
+    exchange_bez_gap_points(bez,gap_points);
+    return distance;
+  } else {
+    return bezierconn_distance_from(bez, point, bezierline->line_width);
+  }
 }
 
 static int bezierline_closest_segment(Bezierline *bezierline, Point *point) {
@@ -230,9 +253,108 @@ bezierline_move(Bezierline *bezierline, Point *to)
   return NULL;
 }
 
+static void exchange_bez_gap_points(BezierConn * bez, Point* gap_points)
+{
+        Point tmp_points[4];
+        tmp_points[0] = bez->points[0].p1;
+        tmp_points[1] = bez->points[1].p1;
+        tmp_points[2] = bez->points[bez->numpoints-1].p2;
+        tmp_points[3] = bez->points[bez->numpoints-1].p3;
+        bez->points[0].p1 = gap_points[0];
+        bez->points[1].p1 = gap_points[1];
+        bez->points[bez->numpoints-1].p2 = gap_points[2];
+        bez->points[bez->numpoints-1].p3 = gap_points[3];
+        gap_points[0] = tmp_points[0];
+        gap_points[1] = tmp_points[1];
+        gap_points[2] = tmp_points[2];
+        gap_points[3] = tmp_points[3];
+}
+static real approx_bez_length(BezierConn *bez)
+{
+        /* Approximates the length of the bezier curve 
+         * by the length of the polyline joining its points */
+        Point *current, *last, vec;
+        real length = .0;
+        int i;
+        current = &bez->points[0].p1;
+        for (i=1; i < bez->numpoints ; i++){
+            last = current;
+            current = &bez->points[i].p3;
+            point_copy(&vec,last);
+            point_sub(&vec,current);
+            length += point_len(&vec);
+        }
+        return length;
+}
+
+static void compute_gap_points(Bezierline *bezierline, Point *gap_points)
+{
+        real first_length, last_length, bez_length;
+        BezierConn *bez = &bezierline->bez;
+        Point vec_start, vec_end;
+
+
+        gap_points[0] = bez->points[0].p1;
+        gap_points[1] = bez->points[1].p1;
+        gap_points[2] = bez->points[bez->numpoints-1].p2;
+        gap_points[3] = bez->points[bez->numpoints-1].p3;
+        
+        point_copy(&vec_start, &gap_points[1]);
+        point_sub(&vec_start, &gap_points[0]);
+        point_normalize(&vec_start); /* unit vector pointing from first point */
+        point_copy(&vec_end, &gap_points[2]);
+        point_sub(&vec_end, &gap_points[3]);
+        point_normalize(&vec_end); /* unit vector pointing from last point */
+
+                
+        bez_length = approx_bez_length(bez) ; 
+        first_length = distance_point_point(&gap_points[0],&gap_points[1]);
+        last_length = distance_point_point(&gap_points[2],&gap_points[3]);
+        
+        if (connpoint_is_autogap(bez->object.handles[0]->connected_to) && 
+               (bez->object.handles[0])->connected_to != NULL && 
+               (bez->object.handles[0])->connected_to->object != NULL ) {
+            Point end; 
+            point_copy(&end, &gap_points[0]);
+            point_add_scaled(&end, &vec_start, bez_length); /* far away on the same slope */
+            end = calculate_object_edge(&gap_points[0], &end, 
+                            (bez->object.handles[0])->connected_to->object);
+            point_sub(&end, &gap_points[0]); /* vector from old start to new start */
+            /* move points */
+            point_add(&gap_points[0], &end);
+            point_add(&gap_points[1], &end);
+        }
+
+        if (connpoint_is_autogap(bez->object.handles[3*(bez->numpoints-1)]->connected_to) && 
+                (bez->object.handles[3*(bez->numpoints-1)])->connected_to != NULL &&
+                (bez->object.handles[3*(bez->numpoints-1)])->connected_to->object != NULL) {
+            Point end; 
+            point_copy(&end, &gap_points[3]);
+            point_add_scaled(&end, &vec_end, bez_length); /* far away on the same slope */
+            end = calculate_object_edge(&gap_points[3], &end, 
+                            (bez->object.handles[3*(bez->numpoints-1)])->connected_to->object);
+            point_sub(&end, &gap_points[3]); /* vector from old end to new end */
+            /* move points */
+            point_add(&gap_points[3], &end);
+            point_add(&gap_points[2], &end);
+        }
+
+        
+        /* adds the absolute start gap  according to the slope at the first point */
+        point_add_scaled(&gap_points[0], &vec_start, bezierline->absolute_start_gap);
+        point_add_scaled(&gap_points[1], &vec_start, bezierline->absolute_start_gap);
+
+        /* adds the absolute end gap  according to the slope at the last point */
+        point_add_scaled(&gap_points[2], &vec_end, bezierline->absolute_end_gap);
+        point_add_scaled(&gap_points[3], &vec_end, bezierline->absolute_end_gap);
+
+
+}
 static void
 bezierline_draw(Bezierline *bezierline, DiaRenderer *renderer)
 {
+  Point gap_points[4]; /* two first and two last bez points */
+        
   BezierConn *bez = &bezierline->bez;
   DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   
@@ -242,11 +364,25 @@ bezierline_draw(Bezierline *bezierline, DiaRenderer *renderer)
   renderer_ops->set_linejoin(renderer, LINEJOIN_MITER);
   renderer_ops->set_linecaps(renderer, LINECAPS_BUTT);
 
+  if (connpoint_is_autogap(bez->object.handles[0]->connected_to) ||
+      connpoint_is_autogap(bez->object.handles[3*(bez->numpoints-1)]->connected_to) ||
+      bezierline->absolute_start_gap || bezierline->absolute_end_gap) {
+
+    compute_gap_points(bezierline,gap_points);
+    exchange_bez_gap_points(bez,gap_points);
+    renderer_ops->draw_bezier_with_arrows(renderer, bez->points, bez->numpoints,
+					 bezierline->line_width,
+					 &bezierline->line_color,
+					 &bezierline->start_arrow,
+					 &bezierline->end_arrow);
+    exchange_bez_gap_points(bez,gap_points);
+  } else {
   renderer_ops->draw_bezier_with_arrows(renderer, bez->points, bez->numpoints,
 					 bezierline->line_width,
 					 &bezierline->line_color,
 					 &bezierline->start_arrow,
 					 &bezierline->end_arrow);
+  }
 
 #if 0
   renderer_ops->draw_bezier(renderer, bez->points, bez->numpoints,
@@ -275,12 +411,12 @@ bezierline_draw(Bezierline *bezierline, DiaRenderer *renderer)
    * only checking while selected.  But we'll do that if needed.
    */
   if (renderer->is_interactive &&
-      dia_object_is_selected((Object*)bezierline)) {
+      dia_object_is_selected((DiaObject*)bezierline)) {
     bezierconn_draw_control_lines(&bezierline->bez, renderer);
   }
 }
 
-static Object *
+static DiaObject *
 bezierline_create(Point *startpoint,
 		  void *user_data,
 		  Handle **handle1,
@@ -288,7 +424,7 @@ bezierline_create(Point *startpoint,
 {
   Bezierline *bezierline;
   BezierConn *bez;
-  Object *obj;
+  DiaObject *obj;
   Point defaultlen = { .3, .3 };
 
   bezierline = g_new0(Bezierline, 1);
@@ -335,12 +471,12 @@ bezierline_destroy(Bezierline *bezierline)
   bezierconn_destroy(&bezierline->bez);
 }
 
-static Object *
+static DiaObject *
 bezierline_copy(Bezierline *bezierline)
 {
   Bezierline *newbezierline;
   BezierConn *bez, *newbez;
-  Object *newobj;
+  DiaObject *newobj;
   
   bez = &bezierline->bez;
  
@@ -356,6 +492,8 @@ bezierline_copy(Bezierline *bezierline)
   newbezierline->dashlength = bezierline->dashlength;
   newbezierline->start_arrow = bezierline->start_arrow;
   newbezierline->end_arrow = bezierline->end_arrow;
+  newbezierline->absolute_start_gap = bezierline->absolute_start_gap;
+  newbezierline->absolute_end_gap = bezierline->absolute_end_gap;
 
   return &newbezierline->bez.object;
 }
@@ -365,7 +503,7 @@ static void
 bezierline_update_data(Bezierline *bezierline)
 {
   BezierConn *bez = &bezierline->bez;
-  Object *obj = &bez->object;
+  DiaObject *obj = &bez->object;
   PolyBBExtras *extra = &bez->extra_spacing;
 
   bezierconn_update_data(bez);
@@ -382,15 +520,36 @@ bezierline_update_data(Bezierline *bezierline)
   extra->start_long = (bezierline->line_width / 2.0);
   extra->end_long   = (bezierline->line_width / 2.0);
 
-  bezierconn_update_boundingbox(bez);
-
   obj->position = bez->points[0].p1;
+
+  if (connpoint_is_autogap(bez->object.handles[0]->connected_to) ||
+      connpoint_is_autogap(bez->object.handles[3*(bez->numpoints-1)]->connected_to) ||
+      bezierline->absolute_start_gap || bezierline->absolute_end_gap) {
+    Point gap_points[4];
+    compute_gap_points(bezierline, gap_points);
+    exchange_bez_gap_points(bez,gap_points);
+    bezierconn_update_boundingbox(bez);
+    exchange_bez_gap_points(bez,gap_points);
+          
+  } else {
+          bezierconn_update_boundingbox(bez);
+  }
+
 }
 
 static void
 bezierline_save(Bezierline *bezierline, ObjectNode obj_node,
 	      const char *filename)
 {
+  if (connpoint_is_autogap(bezierline->bez.object.handles[0]->connected_to) ||
+      connpoint_is_autogap(bezierline->bez.object.handles[3*(bezierline->bez.numpoints-1)]->connected_to) ||
+      bezierline->absolute_start_gap || bezierline->absolute_end_gap) {
+    Point gap_points[4];
+    compute_gap_points(bezierline, gap_points);
+    exchange_bez_gap_points(&bezierline->bez,gap_points);
+    bezierconn_update_boundingbox(&bezierline->bez);
+    exchange_bez_gap_points(&bezierline->bez,gap_points);
+  } 
   bezierconn_save(&bezierline->bez, obj_node);
 
   if (!color_equals(&bezierline->line_color, &color_black))
@@ -411,31 +570,34 @@ bezierline_save(Bezierline *bezierline, ObjectNode obj_node,
 		  bezierline->dashlength);
   
   if (bezierline->start_arrow.type != ARROW_NONE) {
-    data_add_enum(new_attribute(obj_node, "start_arrow"),
-		  bezierline->start_arrow.type);
-    data_add_real(new_attribute(obj_node, "start_arrow_length"),
-		  bezierline->start_arrow.length);
-    data_add_real(new_attribute(obj_node, "start_arrow_width"),
-		  bezierline->start_arrow.width);
+    save_arrow(obj_node, &bezierline->start_arrow, "start_arrow",
+	     "start_arrow_length", "start_arrow_width");
   }
-  
+
   if (bezierline->end_arrow.type != ARROW_NONE) {
-    data_add_enum(new_attribute(obj_node, "end_arrow"),
-		  bezierline->end_arrow.type);
-    data_add_real(new_attribute(obj_node, "end_arrow_length"),
-		  bezierline->end_arrow.length);
-    data_add_real(new_attribute(obj_node, "end_arrow_width"),
-		  bezierline->end_arrow.width);
+    save_arrow(obj_node, &bezierline->end_arrow, "end_arrow",
+	     "end_arrow_length", "end_arrow_width");
   }
+
+  if (bezierline->absolute_start_gap)
+    data_add_real(new_attribute(obj_node, "absolute_start_gap"),
+                 bezierline->absolute_start_gap);
+  if (bezierline->absolute_end_gap)
+    data_add_real(new_attribute(obj_node, "absolute_end_gap"),
+                 bezierline->absolute_end_gap);
 }
 
-static Object *
+static DiaObject *
 bezierline_load(ObjectNode obj_node, int version, const char *filename)
 {
   Bezierline *bezierline;
   BezierConn *bez;
-  Object *obj;
+  DiaObject *obj;
   AttributeNode attr;
+  /* When the bezier is loaded it is not yet connected to the other objects
+   * hence we cannot compute it's real bounding box if auto gaps are used 
+   * bb is a backup bounding box */
+  Rectangle bb; 
 
   bezierline = g_new0(Bezierline, 1);
 
@@ -467,39 +629,30 @@ bezierline_load(ObjectNode obj_node, int version, const char *filename)
   if (attr != NULL)
     bezierline->dashlength = data_real(attribute_first_data(attr));
 
-  bezierline->start_arrow.type = ARROW_NONE;
-  bezierline->start_arrow.length = DEFAULT_ARROW_LENGTH;
-  bezierline->start_arrow.width = DEFAULT_ARROW_WIDTH;
-  attr = object_find_attribute(obj_node, "start_arrow");
-  if (attr != NULL)
-    bezierline->start_arrow.type = data_enum(attribute_first_data(attr));
-  attr = object_find_attribute(obj_node, "start_arrow_length");
-  if (attr != NULL)
-    bezierline->start_arrow.length = data_real(attribute_first_data(attr));
-  attr = object_find_attribute(obj_node, "start_arrow_width");
-  if (attr != NULL)
-    bezierline->start_arrow.width = data_real(attribute_first_data(attr));
+  load_arrow(obj_node, &bezierline->start_arrow, "start_arrow",
+	     "start_arrow_length", "start_arrow_width");
 
-  bezierline->end_arrow.type = ARROW_NONE;
-  bezierline->end_arrow.length = 0.8;
-  bezierline->end_arrow.width = 0.8;
-  attr = object_find_attribute(obj_node, "end_arrow");
-  if (attr != NULL)
-    bezierline->end_arrow.type = data_enum(attribute_first_data(attr));
-  attr = object_find_attribute(obj_node, "end_arrow_length");
-  if (attr != NULL)
-    bezierline->end_arrow.length = data_real(attribute_first_data(attr));
-  attr = object_find_attribute(obj_node, "end_arrow_width");
-  if (attr != NULL)
-    bezierline->end_arrow.width = data_real(attribute_first_data(attr));
+  load_arrow(obj_node, &bezierline->end_arrow, "end_arrow",
+	     "end_arrow_length", "end_arrow_width");
 
-  bezierline_update_data(bezierline);
+  bezierline->absolute_start_gap = 0.0;
+  attr = object_find_attribute(obj_node, "absolute_start_gap");
+  if (attr != NULL)
+    bezierline->absolute_start_gap =  data_real( attribute_first_data(attr) );
+  bezierline->absolute_end_gap = 0.0;
+  attr = object_find_attribute(obj_node, "absolute_end_gap");
+  if (attr != NULL)
+    bezierline->absolute_end_gap =  data_real( attribute_first_data(attr) );
+  
+  bb = obj->bounding_box;
+  bezierline_update_data(bezierline); /* screws up the bounding box if auto_gap */
+  obj->bounding_box =  bb;
 
   return &bezierline->bez.object;
 }
 
 static ObjectChange *
-bezierline_add_segment_callback (Object *obj, Point *clicked, gpointer data)
+bezierline_add_segment_callback (DiaObject *obj, Point *clicked, gpointer data)
 {
   Bezierline *bezierline = (Bezierline*) obj;
   int segment;
@@ -512,7 +665,7 @@ bezierline_add_segment_callback (Object *obj, Point *clicked, gpointer data)
 }
 
 static ObjectChange *
-bezierline_delete_segment_callback (Object *obj, Point *clicked, gpointer data)
+bezierline_delete_segment_callback (DiaObject *obj, Point *clicked, gpointer data)
 {
   int seg_nr;
   Bezierline *bezierline = (Bezierline*) obj;
@@ -527,7 +680,7 @@ bezierline_delete_segment_callback (Object *obj, Point *clicked, gpointer data)
 }
 
 static ObjectChange *
-bezierline_set_corner_type_callback (Object *obj, Point *clicked, gpointer data)
+bezierline_set_corner_type_callback (DiaObject *obj, Point *clicked, gpointer data)
 {
   Handle *closest;
   Bezierline *bezierline = (Bezierline*) obj;
