@@ -24,26 +24,25 @@
 #endif
 
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 
 #include "intl.h"
 #include "object.h"
 #include "beziershape.h"
 #include "connectionpoint.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "attributes.h"
 #include "widgets.h"
 #include "diamenu.h"
 #include "message.h"
 #include "properties.h"
+#include "create.h"
 
-#include "pixmaps/beziergon.xpm"
+#include "tool-icons.h"
 
 #define DEFAULT_WIDTH 0.15
 
 typedef struct _BeziergonProperties BeziergonProperties;
-typedef struct _BeziergonDefaultsDialog BeziergonDefaultsDialog;
 
 typedef struct _Beziergon {
   BezierShape bezier;
@@ -56,71 +55,58 @@ typedef struct _Beziergon {
   real line_width;
 } Beziergon;
 
-struct _BeziergonProperties {
+static struct _BeziergonProperties {
   gboolean show_background;
-};
+} default_properties = { TRUE };
 
-struct _BeziergonDefaultsDialog {
-  GtkWidget *vbox;
-
-  GtkToggleButton *show_background;
-};
-
-
-static BeziergonDefaultsDialog *beziergon_defaults_dialog;
-static BeziergonProperties default_properties;
-
-
-static void beziergon_move_handle(Beziergon *beziergon, Handle *handle,
-		Point *to, HandleMoveReason reason, ModifierKeys modifiers);
-static void beziergon_move(Beziergon *beziergon, Point *to);
+static ObjectChange* beziergon_move_handle(Beziergon *beziergon, Handle *handle,
+					   Point *to, ConnectionPoint *cp,
+					   HandleMoveReason reason, ModifierKeys modifiers);
+static ObjectChange* beziergon_move(Beziergon *beziergon, Point *to);
 static void beziergon_select(Beziergon *beziergon, Point *clicked_point,
-			     Renderer *interactive_renderer);
-static void beziergon_draw(Beziergon *beziergon, Renderer *renderer);
-static Object *beziergon_create(Point *startpoint,
+			     DiaRenderer *interactive_renderer);
+static void beziergon_draw(Beziergon *beziergon, DiaRenderer *renderer);
+static DiaObject *beziergon_create(Point *startpoint,
 				void *user_data,
 				Handle **handle1,
 				Handle **handle2);
 static real beziergon_distance_from(Beziergon *beziergon, Point *point);
 static void beziergon_update_data(Beziergon *beziergon);
 static void beziergon_destroy(Beziergon *beziergon);
-static Object *beziergon_copy(Beziergon *beziergon);
+static DiaObject *beziergon_copy(Beziergon *beziergon);
 
 static PropDescription *beziergon_describe_props(Beziergon *beziergon);
-static void beziergon_get_props(Beziergon *beziergon, Property *props,
-				guint nprops);
-static void beziergon_set_props(Beziergon *beziergon, Property *props,
-				guint nprops);
+static void beziergon_get_props(Beziergon *beziergon, GPtrArray *props);
+static void beziergon_set_props(Beziergon *beziergon, GPtrArray *props);
 
 static void beziergon_save(Beziergon *beziergon, ObjectNode obj_node,
 			  const char *filename);
-static Object *beziergon_load(ObjectNode obj_node, int version,
+static DiaObject *beziergon_load(ObjectNode obj_node, int version,
 			     const char *filename);
 static DiaMenu *beziergon_get_object_menu(Beziergon *beziergon,
 					  Point *clickedpoint);
 
-static GtkWidget *beziergon_get_defaults(void);
-static void beziergon_apply_defaults(void);
-
 static ObjectTypeOps beziergon_type_ops =
 {
   (CreateFunc)beziergon_create,   /* create */
   (LoadFunc)  beziergon_load,     /* load */
   (SaveFunc)  beziergon_save,      /* save */
-  (GetDefaultsFunc)   beziergon_get_defaults,
-  (ApplyDefaultsFunc) beziergon_apply_defaults
+  (GetDefaultsFunc)   NULL,
+  (ApplyDefaultsFunc) NULL
 };
 
-static ObjectType beziergon_type =
+static DiaObjectType beziergon_type =
 {
   "Standard - Beziergon",   /* name */
   0,                         /* version */
-  (char **) beziergon_xpm,      /* pixmap */
+  (char **) beziergon_icon,      /* pixmap */
   
-  &beziergon_type_ops       /* ops */
+  &beziergon_type_ops,      /* ops */
+  NULL,                     /* pixmap_file */
+  0                         /* default_user_data */
 };
 
-ObjectType *_beziergon_type = (ObjectType *) &beziergon_type;
+DiaObjectType *_beziergon_type = (DiaObjectType *) &beziergon_type;
 
 
 static ObjectOps beziergon_ops = {
@@ -140,7 +126,7 @@ static ObjectOps beziergon_ops = {
 };
 
 static PropDescription beziergon_props[] = {
-  OBJECT_COMMON_PROPERTIES,
+  BEZSHAPE_COMMON_PROPERTIES,
   PROP_STD_LINE_WIDTH,
   PROP_STD_LINE_COLOUR,
   PROP_STD_LINE_STYLE,
@@ -158,7 +144,7 @@ beziergon_describe_props(Beziergon *beziergon)
 }
 
 static PropOffset beziergon_offsets[] = {
-  OBJECT_COMMON_PROPERTIES_OFFSETS,
+  BEZSHAPE_COMMON_PROPERTIES_OFFSETS,
   { "line_width", PROP_TYPE_REAL, offsetof(Beziergon, line_width) },
   { "line_colour", PROP_TYPE_COLOUR, offsetof(Beziergon, line_color) },
   { "line_style", PROP_TYPE_LINESTYLE,
@@ -169,73 +155,20 @@ static PropOffset beziergon_offsets[] = {
 };
 
 static void
-beziergon_get_props(Beziergon *beziergon, Property *props, guint nprops)
+beziergon_get_props(Beziergon *beziergon, GPtrArray *props)
 {
   object_get_props_from_offsets(&beziergon->bezier.object, beziergon_offsets,
-				props, nprops);
+				props);
 }
 
 static void
-beziergon_set_props(Beziergon *beziergon, Property *props, guint nprops)
+beziergon_set_props(Beziergon *beziergon, GPtrArray *props)
 {
   object_set_props_from_offsets(&beziergon->bezier.object, beziergon_offsets,
-				props, nprops);
+				props);
   beziergon_update_data(beziergon);
 }
 
-static void
-beziergon_apply_defaults(void)
-{
-  default_properties.show_background = gtk_toggle_button_get_active(beziergon_defaults_dialog->show_background);
-}
-
-static void
-init_default_values(void) {
-  static int defaults_initialized = 0;
-
-  if (!defaults_initialized) {
-    default_properties.show_background = 1;
-    defaults_initialized = 1;
-  }
-}
-
-static GtkWidget *
-beziergon_get_defaults(void)
-{
-  GtkWidget *vbox;
-  GtkWidget *hbox;
-  GtkWidget *checkbox;
-
-  if (beziergon_defaults_dialog == NULL) {
-  
-    init_default_values();
-
-    beziergon_defaults_dialog = g_new(BeziergonDefaultsDialog, 1);
-
-    vbox = gtk_vbox_new(FALSE, 5);
-    beziergon_defaults_dialog->vbox = vbox;
-
-    gtk_object_ref(GTK_OBJECT(vbox));
-    gtk_object_sink(GTK_OBJECT(vbox));
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    checkbox = gtk_check_button_new_with_label(_("Draw background"));
-    beziergon_defaults_dialog->show_background = GTK_TOGGLE_BUTTON( checkbox );
-    gtk_widget_show(checkbox);
-    gtk_widget_show(hbox);
-    gtk_box_pack_start (GTK_BOX (hbox), checkbox, TRUE, TRUE, 0);
-    gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
-
-    gtk_widget_show (vbox);
-  }
-
-  gtk_toggle_button_set_active(beziergon_defaults_dialog->show_background, 
-			       default_properties.show_background);
-
-  return beziergon_defaults_dialog->vbox;
-}
-
-
 static real
 beziergon_distance_from(Beziergon *beziergon, Point *point)
 {
@@ -243,12 +176,6 @@ beziergon_distance_from(Beziergon *beziergon, Point *point)
 				   beziergon->line_width);
 }
 
-static Handle *
-beziergon_closest_handle(Beziergon *beziergon, Point *point)
-{
-  return beziershape_closest_handle(&beziergon->bezier, point);
-}
-
 static int
 beziergon_closest_segment(Beziergon *beziergon, Point *point)
 {
@@ -258,34 +185,41 @@ beziergon_closest_segment(Beziergon *beziergon, Point *point)
 
 static void
 beziergon_select(Beziergon *beziergon, Point *clicked_point,
-		  Renderer *interactive_renderer)
+		  DiaRenderer *interactive_renderer)
 {
   beziershape_update_data(&beziergon->bezier);
 }
 
-static void
+static ObjectChange*
 beziergon_move_handle(Beziergon *beziergon, Handle *handle,
-		Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		      Point *to, ConnectionPoint *cp,
+		      HandleMoveReason reason, ModifierKeys modifiers)
 {
   assert(beziergon!=NULL);
   assert(handle!=NULL);
   assert(to!=NULL);
 
-  beziershape_move_handle(&beziergon->bezier, handle, to, reason);
+  beziershape_move_handle(&beziergon->bezier, handle, to, cp, reason, modifiers);
   beziergon_update_data(beziergon);
+
+  return NULL;
 }
 
 
-static void
+static ObjectChange*
 beziergon_move(Beziergon *beziergon, Point *to)
 {
   beziershape_move(&beziergon->bezier, to);
   beziergon_update_data(beziergon);
+
+  return NULL;
 }
 
 static void
-beziergon_draw(Beziergon *beziergon, Renderer *renderer)
+beziergon_draw(Beziergon *beziergon, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
+
   BezierShape *bezier = &beziergon->bezier;
   BezPoint *points;
   int n;
@@ -293,26 +227,27 @@ beziergon_draw(Beziergon *beziergon, Renderer *renderer)
   points = &bezier->points[0];
   n = bezier->numpoints;
 
-  renderer->ops->set_linewidth(renderer, beziergon->line_width);
-  renderer->ops->set_linestyle(renderer, beziergon->line_style);
-  renderer->ops->set_dashlength(renderer, beziergon->dashlength);
-  renderer->ops->set_linejoin(renderer, LINEJOIN_MITER);
-  renderer->ops->set_linecaps(renderer, LINECAPS_BUTT);
+  renderer_ops->set_linewidth(renderer, beziergon->line_width);
+  renderer_ops->set_linestyle(renderer, beziergon->line_style);
+  renderer_ops->set_dashlength(renderer, beziergon->dashlength);
+  renderer_ops->set_linejoin(renderer, LINEJOIN_MITER);
+  renderer_ops->set_linecaps(renderer, LINECAPS_BUTT);
 
   if (beziergon->show_background)
-    renderer->ops->fill_bezier(renderer, points, n, &beziergon->inner_color);
+    renderer_ops->fill_bezier(renderer, points, n, &beziergon->inner_color);
 
-  renderer->ops->draw_bezier(renderer, points, n, &beziergon->line_color);
+  renderer_ops->draw_bezier(renderer, points, n, &beziergon->line_color);
 
   /* these lines should only be displayed when object is selected.
    * Unfortunately the draw function is not aware of the selected
    * state.  This is a compromise until I fix this properly. */
-  if (renderer->is_interactive) {
+  if (renderer->is_interactive &&
+      dia_object_is_selected((DiaObject*)beziergon)) {
     beziershape_draw_control_lines(&beziergon->bezier, renderer);
   }
 }
 
-static Object *
+static DiaObject *
 beziergon_create(Point *startpoint,
 		  void *user_data,
 		  Handle **handle1,
@@ -320,13 +255,10 @@ beziergon_create(Point *startpoint,
 {
   Beziergon *beziergon;
   BezierShape *bezier;
-  Object *obj;
+  DiaObject *obj;
   Point defaultx = { 1.0, 0.0 };
   Point defaulty = { 0.0, 1.0 };
 
-  init_default_values();
-
-  /*beziergon_init_defaults();*/
   beziergon = g_new0(Beziergon, 1);
   bezier = &beziergon->bezier;
   obj = &bezier->object;
@@ -334,24 +266,30 @@ beziergon_create(Point *startpoint,
   obj->type = &beziergon_type;
   obj->ops = &beziergon_ops;
 
-  beziershape_init(bezier);
+  if (user_data == NULL) {
+    beziershape_init(bezier, 3);
 
-  bezier->points[0].p1 = *startpoint;
-  bezier->points[0].p3 = *startpoint;
-  bezier->points[2].p3 = *startpoint;
+    bezier->points[0].p1 = *startpoint;
+    bezier->points[0].p3 = *startpoint;
+    bezier->points[2].p3 = *startpoint;
 
-  bezier->points[1].p1 = *startpoint;
-  point_add(&bezier->points[1].p1, &defaultx);
-  bezier->points[2].p2 = *startpoint;
-  point_sub(&bezier->points[2].p2, &defaultx);
+    bezier->points[1].p1 = *startpoint;
+    point_add(&bezier->points[1].p1, &defaultx);
+    bezier->points[2].p2 = *startpoint;
+    point_sub(&bezier->points[2].p2, &defaultx);
 
-  bezier->points[1].p3 = *startpoint;
-  point_add(&bezier->points[1].p3, &defaulty);
-  bezier->points[1].p2 = bezier->points[1].p3;
-  point_add(&bezier->points[1].p2, &defaultx);
-  bezier->points[2].p1 = bezier->points[1].p3;
-  point_sub(&bezier->points[2].p1, &defaultx);
-  
+    bezier->points[1].p3 = *startpoint;
+    point_add(&bezier->points[1].p3, &defaulty);
+    bezier->points[1].p2 = bezier->points[1].p3;
+    point_add(&bezier->points[1].p2, &defaultx);
+    bezier->points[2].p1 = bezier->points[1].p3;
+    point_sub(&bezier->points[2].p1, &defaultx);
+  } else {
+    BezierCreateData *bcd = (BezierCreateData*)user_data;
+
+    beziershape_init(bezier, bcd->num_points);
+    beziershape_set_points(bezier, bcd->num_points, bcd->points);
+  }  
   beziergon->line_width =  attributes_get_default_linewidth();
   beziergon->line_color = attributes_get_foreground();
   beziergon->inner_color = attributes_get_background();
@@ -372,12 +310,12 @@ beziergon_destroy(Beziergon *beziergon)
   beziershape_destroy(&beziergon->bezier);
 }
 
-static Object *
+static DiaObject *
 beziergon_copy(Beziergon *beziergon)
 {
   Beziergon *newbeziergon;
   BezierShape *bezier, *newbezier;
-  Object *newobj;
+  DiaObject *newobj;
 
   bezier = &beziergon->bezier;
  
@@ -401,8 +339,8 @@ static void
 beziergon_update_data(Beziergon *beziergon)
 {
   BezierShape *bezier = &beziergon->bezier;
-  Object *obj = &bezier->object;
-  BezierShapeBBExtras *extra = &bezier->extra_spacing;
+  DiaObject *obj = &bezier->object;
+  ElementBBExtras *extra = &bezier->extra_spacing;
   
   beziershape_update_data(bezier);
   
@@ -444,12 +382,12 @@ beziergon_save(Beziergon *beziergon, ObjectNode obj_node,
   
 }
 
-static Object *
+static DiaObject *
 beziergon_load(ObjectNode obj_node, int version, const char *filename)
 {
   Beziergon *beziergon;
   BezierShape *bezier;
-  Object *obj;
+  DiaObject *obj;
   AttributeNode attr;
 
   beziergon = g_malloc0(sizeof(Beziergon));
@@ -498,7 +436,7 @@ beziergon_load(ObjectNode obj_node, int version, const char *filename)
 }
 
 static ObjectChange *
-beziergon_add_segment_callback (Object *obj, Point *clicked, gpointer data)
+beziergon_add_segment_callback (DiaObject *obj, Point *clicked, gpointer data)
 {
   Beziergon *bezier = (Beziergon*) obj;
   int segment;
@@ -512,7 +450,7 @@ beziergon_add_segment_callback (Object *obj, Point *clicked, gpointer data)
 }
 
 static ObjectChange *
-beziergon_delete_segment_callback (Object *obj, Point *clicked, gpointer data)
+beziergon_delete_segment_callback (DiaObject *obj, Point *clicked, gpointer data)
 {
   int seg_nr;
   Beziergon *bezier = (Beziergon*) obj;
@@ -526,7 +464,7 @@ beziergon_delete_segment_callback (Object *obj, Point *clicked, gpointer data)
 }
 
 static ObjectChange *
-beziergon_set_corner_type_callback (Object *obj, Point *clicked, gpointer data)
+beziergon_set_corner_type_callback (DiaObject *obj, Point *clicked, gpointer data)
 {
   Handle *closest;
   Beziergon *beziergon = (Beziergon *) obj;
