@@ -34,7 +34,7 @@
 #include "properties.h"
 #include "autoroute.h"
 
-#include "pixmaps/zigzag.xpm"
+#include "tool-icons.h"
 
 #define DEFAULT_WIDTH 0.15
 
@@ -91,8 +91,9 @@ static ObjectTypeOps zigzagline_type_ops =
 static DiaObjectType zigzagline_type =
 {
   "Standard - ZigZagLine",   /* name */
-  0,                         /* version */
-  (char **) zigzag_xpm,      /* pixmap */
+  /* Version 0 had no autorouting and so shouldn't have it set by default. */
+  1,                      /* version */
+  (char **) zigzagline_icon,      /* pixmap */
   
   &zigzagline_type_ops       /* ops */
 };
@@ -231,6 +232,7 @@ zigzagline_draw(Zigzagline *zigzagline, DiaRenderer *renderer)
 						  &zigzagline->start_arrow,
 						  &zigzagline->end_arrow,
 						  zigzagline->corner_radius);
+
 }
 
 static DiaObject *
@@ -389,21 +391,13 @@ zigzagline_save(Zigzagline *zigzagline, ObjectNode obj_node,
 		  zigzagline->line_style);
   
   if (zigzagline->start_arrow.type != ARROW_NONE) {
-    data_add_enum(new_attribute(obj_node, "start_arrow"),
-		  zigzagline->start_arrow.type);
-    data_add_real(new_attribute(obj_node, "start_arrow_length"),
-		  zigzagline->start_arrow.length);
-    data_add_real(new_attribute(obj_node, "start_arrow_width"),
-		  zigzagline->start_arrow.width);
+    save_arrow(obj_node, &zigzagline->start_arrow, "start_arrow",
+	     "start_arrow_length", "start_arrow_width");
   }
-  
+
   if (zigzagline->end_arrow.type != ARROW_NONE) {
-    data_add_enum(new_attribute(obj_node, "end_arrow"),
-		  zigzagline->end_arrow.type);
-    data_add_real(new_attribute(obj_node, "end_arrow_length"),
-		  zigzagline->end_arrow.length);
-    data_add_real(new_attribute(obj_node, "end_arrow_width"),
-		  zigzagline->end_arrow.width);
+    save_arrow(obj_node, &zigzagline->end_arrow, "end_arrow",
+	     "end_arrow_length", "end_arrow_width");
   }
 
   if (zigzagline->line_style != LINESTYLE_SOLID && 
@@ -449,31 +443,11 @@ zigzagline_load(ObjectNode obj_node, int version, const char *filename)
   if (attr != NULL)
     zigzagline->line_style = data_enum(attribute_first_data(attr));
 
-  zigzagline->start_arrow.type = ARROW_NONE;
-  zigzagline->start_arrow.length = DEFAULT_ARROW_LENGTH;
-  zigzagline->start_arrow.width = DEFAULT_ARROW_WIDTH;
-  attr = object_find_attribute(obj_node, "start_arrow");
-  if (attr != NULL)
-    zigzagline->start_arrow.type = data_enum(attribute_first_data(attr));
-  attr = object_find_attribute(obj_node, "start_arrow_length");
-  if (attr != NULL)
-    zigzagline->start_arrow.length = data_real(attribute_first_data(attr));
-  attr = object_find_attribute(obj_node, "start_arrow_width");
-  if (attr != NULL)
-    zigzagline->start_arrow.width = data_real(attribute_first_data(attr));
+  load_arrow(obj_node, &zigzagline->start_arrow, "start_arrow",
+	     "start_arrow_length", "start_arrow_width");
 
-  zigzagline->end_arrow.type = ARROW_NONE;
-  zigzagline->end_arrow.length = DEFAULT_ARROW_LENGTH;
-  zigzagline->end_arrow.width = DEFAULT_ARROW_WIDTH;
-  attr = object_find_attribute(obj_node, "end_arrow");
-  if (attr != NULL)
-    zigzagline->end_arrow.type = data_enum(attribute_first_data(attr));
-  attr = object_find_attribute(obj_node, "end_arrow_length");
-  if (attr != NULL)
-    zigzagline->end_arrow.length = data_real(attribute_first_data(attr));
-  attr = object_find_attribute(obj_node, "end_arrow_width");
-  if (attr != NULL)
-    zigzagline->end_arrow.width = data_real(attribute_first_data(attr));
+  load_arrow(obj_node, &zigzagline->end_arrow, "end_arrow",
+	     "end_arrow_length", "end_arrow_width");
 
   zigzagline->dashlength = DEFAULT_LINESTYLE_DASHLEN;
   attr = object_find_attribute(obj_node, "dashlength");
