@@ -25,12 +25,13 @@
 #include <config.h>
 #endif
 
+#include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <math.h>
 #include <glib.h>
-#include <stdlib.h>
 #include <locale.h>
+#include <glib/gstdio.h>
 
 #include "intl.h"
 #include "message.h"
@@ -431,7 +432,7 @@ DiaObject *read_entity_polyline_dxf(FILE *filedxf, DxfData *data, DiagramData *d
                 {
                     points++;
 		
-                    p = realloc( p, sizeof( Point ) * points );
+                    p = g_realloc( p, sizeof( Point ) * points );
 		
                         /*printf( "Vertex %d\n", points );*/
 		  
@@ -464,7 +465,7 @@ DiaObject *read_entity_polyline_dxf(FILE *filedxf, DxfData *data, DiagramData *d
                 break;
             case 42:
                     /* FIXME - the bulge code doesn't work */
-                p = realloc( p, sizeof( Point ) * ( points + 10 ));
+                p = g_realloc( p, sizeof( Point ) * ( points + 10 ));
 
                 start = p[points-2];
                 end = p[points-1];
@@ -551,10 +552,7 @@ DiaObject *read_entity_polyline_dxf(FILE *filedxf, DxfData *data, DiagramData *d
    
     if( closed )
     {
-        ++points;
-        p = realloc( p, sizeof( Point ) * points );
-        p[points-1].x = p[0].x;
-        p[points-1].y = p[0].y;
+        otype = object_get_type("Standard - Polygon");
     }
    
     pcd->num_points = points;
@@ -562,7 +560,7 @@ DiaObject *read_entity_polyline_dxf(FILE *filedxf, DxfData *data, DiagramData *d
    
     memcpy( pcd->points, p, sizeof( Point ) * pcd->num_points );
    
-    free( p );
+    g_free( p );
 
     polyline_obj = otype->ops->create( NULL, pcd, &h1, &h2 );
 
@@ -940,7 +938,7 @@ DiaObject *read_entity_text_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
             height = atof(data->value) * coord_scale * measure_scale;
 	   /*printf( "text height %f\n", height );*/
             break;
-	 case 62: 
+        case 62: 
 	   colour = atoi(data->value);
 	   text_colour.red = acad_pal[colour].r / 255.0;
 	   text_colour.green = acad_pal[colour].g / 255.0;
@@ -993,7 +991,7 @@ DiaObject *read_entity_text_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
     } while(codedxf != 0);
     setlocale(LC_NUMERIC,old_locale);
   
-   location.y += y_offset * height;
+    location.y += y_offset * height;
    
     text_obj = otype->ops->create(&location, otype->default_user_data,
                                   &h1, &h2);
@@ -1009,6 +1007,7 @@ DiaObject *read_entity_text_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
     tprop->attr.position.y = location.y;
 
     attributes_get_default_font(&tprop->attr.font, &tprop->attr.height);
+    tprop->attr.height = height;
     tprop->attr.color = text_colour;
         
     text_obj->ops->set_props(text_obj, props);
@@ -1294,14 +1293,17 @@ void read_section_blocks_dxf(FILE *filedxf, DxfData *data, DiagramData *dia)
 }
 
 /* imports the given dxf-file, returns TRUE if successful */
-gboolean import_dxf(const gchar *filename, DiagramData *dia, void* user_data){
+gboolean
+import_dxf(const gchar *filename, DiagramData *dia, void* user_data)
+{
     FILE *filedxf;
     DxfData *data;
     int codedxf;
     
-    filedxf = fopen(filename,"r");
+    filedxf = g_fopen(filename,"r");
     if(filedxf == NULL){
-        message_error(_("Couldn't open: '%s' for reading.\n"), filename);
+        message_error(_("Couldn't open: '%s' for reading.\n"), 
+		      dia_message_filename(filename));
         return FALSE;
     }
     
@@ -1310,11 +1312,18 @@ gboolean import_dxf(const gchar *filename, DiagramData *dia, void* user_data){
     do {
         if(read_dxf_codes(filedxf, data) == FALSE) {
             g_free(data);
-                g_error(_("read_dxf_codes failed\n") );
+	    message_error(_("read_dxf_codes failed on '%s'\n"),
+			  dia_message_filename(filename) );
             return FALSE;
         }
         else {
             codedxf = atoi(data->code);
+            if (0 == codedxf && strstr(data->code, "AutoCAD Binary DXF")) {
+                g_free(data);
+	        message_error(_("Binary DXF from '%s' not supported\n"),
+			      dia_message_filename(filename) );
+                return FALSE;
+            }
             if(codedxf == 2) {
                 if(strcmp(data->value, "ENTITIES") == 0) {
 		   /*printf( "reading section entities\n" );*/
