@@ -19,8 +19,6 @@
 
 #include <config.h>
 
-#include <glib.h>
-
 #include "pydia-object.h"
 #include "pydia-image.h"
 
@@ -47,7 +45,7 @@ static void
 PyDiaImage_Dealloc(PyDiaImage *self)
 {
   dia_image_release (self->image);
-  PyMem_DEL(self);
+  PyObject_DEL(self);
 }
 
 /*
@@ -84,21 +82,16 @@ PyDiaImage_GetAttr(PyDiaImage *self, gchar *attr)
   else if (!strcmp(attr, "height"))
     return PyInt_FromLong(dia_image_height(self->image));
   else if (!strcmp(attr, "filename")) {
-    char* s = dia_image_filename(self->image);
-    PyObject* py_s = PyString_FromString(s);
-    g_free (s);
-    return py_s;
+    return PyString_FromString(dia_image_filename(self->image));
   }
   else if (!strcmp(attr, "uri")) {
     GError* error = NULL;
-    char* s1 = dia_image_filename(self->image);
-    char* s2 = g_filename_to_uri(s1, NULL, &error);
-    g_free(s1);
-    if (s2) {
-      PyObject* py_s = PyString_FromString(s2);
-      g_free(s2);
+    char* s = g_filename_to_uri(dia_image_filename(self->image), NULL, &error);
+    if (s) {
+      PyObject* py_s = PyString_FromString(s);
+      g_free(s);
       return py_s;
-    } 
+    }
     else {
       PyErr_SetString(PyExc_RuntimeError, error->message);
       g_error_free (error);
@@ -131,14 +124,13 @@ static PyObject *
 PyDiaImage_Str(PyDiaImage *self)
 {
   PyObject* py_s;
-  gchar* name = dia_image_filename(self->image);
+  const gchar* name = dia_image_filename(self->image);
   gchar* s = g_strdup_printf("%ix%i,%s",
                              dia_image_width(self->image),
                              dia_image_height(self->image),
-                             name);
+                             name ? name : "(null)");
   py_s = PyString_FromString(s);
   g_free (s);
-  g_free (name);
   return py_s;
 }
 
@@ -148,7 +140,7 @@ PyDiaImage_Str(PyDiaImage *self)
 PyTypeObject PyDiaImage_Type = {
     PyObject_HEAD_INIT(&PyType_Type)
     0,
-    "DiaImage",
+    "dia.Image",
     sizeof(PyDiaImage),
     0,
     (destructor)PyDiaImage_Dealloc,
@@ -163,6 +155,9 @@ PyTypeObject PyDiaImage_Type = {
     (hashfunc)PyDiaImage_Hash,
     (ternaryfunc)0,
     (reprfunc)PyDiaImage_Str,
-    0L,0L,0L,0L,
-    NULL
+    (getattrofunc)0,
+    (setattrofunc)0,
+    (PyBufferProcs *)0,
+    0L, /* Flags */
+    "dia.Image gets passed into DiaRenderer.draw_image"
 };
