@@ -37,9 +37,11 @@
 #include <stdio.h>
 #include <string.h>
 #include <math.h>
-#include <glib.h>
 #include <errno.h>
 
+#include <glib.h>
+#include <glib/gstdio.h>
+
 #include "intl.h"
 #include "message.h"
 #include "geometry.h"
@@ -289,7 +291,7 @@ begin_render(DiaRenderer *self)
   fwrite_le(&renderer->Box.Width, sizeof(guint16), 2, renderer->file);
 #endif
   /* initialize a well known colormap, see LookupColor */
-  pPal = g_new(gint8, CC_LEN*CC_LEN*CC_LEN*3);
+  pPal = g_new(guint8, CC_LEN*CC_LEN*CC_LEN*3);
   for (i = 0; i < CC_LEN * CC_LEN * CC_LEN; i++)
   {
     pPal[3*i  ] = ((i % CC_LEN) * 255) / (CC_LEN - 1);   /* red */
@@ -324,6 +326,7 @@ end_render(DiaRenderer *self)
 
   WriteRecHead(renderer, WPG_END, 0); /* no data following */
   fclose(renderer->file);
+  renderer->file = NULL;
 }
 
 static void
@@ -339,8 +342,6 @@ set_linewidth(DiaRenderer *self, real linewidth)
 static void
 set_linecaps(DiaRenderer *self, LineCaps mode)
 {
-  WpgRenderer *renderer = WPG_RENDERER (self);
-
   DIAG_NOTE(g_message("set_linecaps %d", mode));
 
   switch(mode) {
@@ -358,8 +359,6 @@ set_linecaps(DiaRenderer *self, LineCaps mode)
 static void
 set_linejoin(DiaRenderer *self, LineJoin mode)
 {
-  WpgRenderer *renderer = WPG_RENDERER (self);
-
   DIAG_NOTE(g_message("set_join %d", mode));
 
   switch(mode) {
@@ -713,6 +712,7 @@ fill_ellipse(DiaRenderer *self,
   WriteFillAttr(renderer, colour, FALSE);
 }
 
+#ifdef USE_OUR_DRAW_BEZIER
 static void
 draw_bezier(DiaRenderer *self, 
             BezPoint *points,
@@ -794,6 +794,7 @@ draw_bezier(DiaRenderer *self,
   fwrite_le(pData, sizeof(gint16), numpoints*4, renderer->file);
   g_free(pData);
 }
+#endif /* USE_OUR_DRAW_BEZIER */
 
 static void
 draw_string(DiaRenderer *self,
@@ -947,7 +948,7 @@ draw_image(DiaRenderer *self,
   {
     FILE* f;
     int i, j;
-    f = fopen("c:\\temp\\wpg_bmp.raw", "wb");
+    f = g_fopen("c:\\temp\\wpg_bmp.raw", "wb");
     j = p - pOut;
     for (i = 0; i < j; i+=2)
     {
@@ -999,6 +1000,10 @@ wpg_renderer_finalize (GObject *object)
 {
   WpgRenderer *wpg_renderer = WPG_RENDERER (object);
 
+  if (wpg_renderer->file)
+    fclose(wpg_renderer->file);
+  wpg_renderer->file = NULL;
+
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -1042,9 +1047,10 @@ wpg_renderer_class_init (WpgRendererClass *klass)
   renderer_class->draw_polyline  = draw_polyline;
   renderer_class->draw_polygon   = draw_polygon;
 
-  /* The bezier implementation above does not match, use base class approximation
+#ifdef USE_OUR_DRAW_BEZIER
+  /* The bezier implementation above does not match, use base class approximation */
   renderer_class->draw_bezier   = draw_bezier;
-   */
+#endif
   /* Implemented by base class approximation
   renderer_class->fill_bezier   = fill_bezier;
    */
@@ -1060,10 +1066,11 @@ export_data(DiagramData *data, const gchar *filename,
   Rectangle *extent;
   real width, height;
 
-  file = fopen(filename, "wb"); /* "wb" for binary! */
+  file = g_fopen(filename, "wb"); /* "wb" for binary! */
 
   if (file == NULL) {
-    message_error(_("Can't open output file %s: %s\n"), filename, strerror(errno));
+    message_error(_("Can't open output file %s: %s\n"), 
+		  dia_message_filename(filename), strerror(errno));
     return;
   }
 
@@ -1164,10 +1171,11 @@ import_data (const gchar *filename, DiagramData *dia, void* user_data)
   gboolean bRet;
   WPGHead8 rh;
 
-  f = fopen(filename, "rb");
+  f = g_fopen(filename, "rb");
 
   if (NULL == f) {
-    message_error(_("Couldn't open: '%s' for reading.\n"), filename);
+    message_error(_("Couldn't open: '%s' for reading.\n"), 
+		  dia_message_filename(filename));
     bRet = FALSE;
   }
   
@@ -1179,7 +1187,8 @@ import_data (const gchar *filename, DiagramData *dia, void* user_data)
             && fhead.fid[2] == 'P' && fhead.fid[3] == 'C'
             && (1 == fhead.MajorVersion) && (0 == fhead.MinorVersion));
     if (!bRet)
-      message_error(_("File: %s type/version unsupported.\n"), filename);
+      message_error(_("File: %s type/version unsupported.\n"), 
+		    dia_message_filename(filename));
   }
 
   if (bRet) {
@@ -1323,6 +1332,20 @@ DiaImportFilter my_import_filter = {
 #endif /* WPG_WITH_IMPORT */
 
 /* --- dia plug-in interface --- */
+static gboolean
+_plugin_can_unload (PluginInfo *info)
+{
+  return TRUE;
+}
+
+static void
+_plugin_unload (PluginInfo *info)
+{
+  filter_unregister_export(&my_export_filter);
+#if WPG_WITH_IMPORT
+  filter_unregister_import(&my_import_filter);
+#endif
+}
 
 DIA_PLUGIN_CHECK_INIT
 
@@ -1331,7 +1354,8 @@ dia_plugin_init(PluginInfo *info)
 {
   if (!dia_plugin_info_init(info, "WPG",
                             _("WordPerfect Graphics export filter"),
-                            NULL, NULL))
+                            _plugin_can_unload,
+                            _plugin_unload))
     return DIA_PLUGIN_INIT_ERROR;
 
   filter_register_export(&my_export_filter);
